@@ -24,13 +24,16 @@ import gtk
 import glib
 import gobject
 import logging
+import tempfile
+import fcntl
+import errno
 
-from metarace import jsonconfig	# recurse?
+from metarace import jsonconfig
 
 # 'check' python version
 assert sys.version >= '2.6', "Missing pre-requisite: python >= 2.6"
 
-VERSION = u'1.6.9'
+VERSION = u'1.6.64'
 LIB_PATH = os.path.realpath(os.path.dirname(__file__))
 UI_PATH = os.path.join(LIB_PATH, u'ui')
 DB_PATH = os.path.join(LIB_PATH, u'data')
@@ -42,8 +45,7 @@ LOGO_FILE = u'metarace_icon.svg'
 SYSCONF_FILE = u'metarace.json'
 PDF_TEMPLATE_FILE = u'pdf_template.json'
 HTML_TEMPLATE_FILE = u'html_template.json'
-
-SCB_LINELEN = 24	# !! REMOVE
+LOGFORMAT = '%(asctime)s %(levelname)s:%(name)s: %(message)s'
 sysconf = jsonconfig.config() # system-defaults, populated by init() method
 
 def init(withgtk=True):
@@ -85,12 +87,11 @@ under certain conditions.\n\n")
         # Initialise threading in GDK
         gtk.gdk.threads_init()
 
-        # fix the gnome menubar accelerator mapping
-        mset = gtk.settings_get_default()
-        mset.set_string_property('gtk-menu-bar-accel', 'F24', 'override')
-
-        # Set the global default window icon
+        #  Try some window methods, but don't worry if they fail
         try:
+            # fix the gnome menubar accelerator mapping
+            mset = gtk.settings_get_default()
+            mset.set_string_property('gtk-menu-bar-accel', 'F24', 'override')
             gtk.window_set_default_icon_from_file(default_file(LOGO_FILE))
         except:
             pass # but ignore errors
@@ -132,10 +133,73 @@ def default_file(filename=u''):
     # raised at this point. A failure to open the returned file should
     # be handled appropriately in the calling code.
     return ret
-    
+
+class savefile(object):
+    """Tempfile-backed save file contextmanager."""
+    def __init__(self, filename, tempdir=u'.'):
+        self.__sfile = filename
+        self.__path = tempdir
+        self.__tfile = tempfile.NamedTemporaryFile(mode='wb', suffix=u'.tmp',
+                                prefix=u'sav_', dir=self.__path, delete=False)
+    def __enter__(self):
+        return self.__tfile
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.__tfile.close()
+        if exc_type is not None:
+            return False        # raise exception
+        # otherwise, file is saved ok in temp file
+        os.chmod(self.__tfile.name, 0o644)
+        try:
+            # Posix ensures this is atomic, in windows raises exception
+            os.rename(self.__tfile.name, self.__sfile)
+        except OSError:
+            os.unlink(self.__sfile)
+            os.rename(self.__tfile.name, self.__sfile)
+        return True
+
+def lockpath(configpath):
+    """Write a pid file in the given config unless already locked."""
+    lock = False
+    pid = os.getpid()
+    with open(os.path.join(configpath, u'lock.pid'), 'a+b') as f:
+        fcntl.flock(f, fcntl.LOCK_EX) 
+        # while waiting for lock, file may have been written to
+        f.seek(0)
+        oinf = f.read().strip()
+        if oinf.isdigit():
+            opid = int(oinf)
+            if opid != pid:
+                try:
+                    os.kill(opid, 0)
+                except OSError as e:
+                    if e.errno != errno.EPERM:
+                        lock = True	# assume process is not alive
+            else:
+                # old pid matches current proc, assume ok to lock
+                lock = True
+        else:
+            lock = True	# no existing lock
+        if lock:
+            # replace any existing content with current pid
+            f.seek(0)
+            f.truncate()
+            f.write('{0}\n'.format(pid))
+    # exit from context closes file and releases lock
+    return lock
+
+def unlockpath(configpath):
+    """Remove a lockfile dangerously."""
+    lf = os.path.join(configpath, u'lock.pid')
+    try:
+        os.unlink(lf)
+    except Exception as e:
+        print(u'Error removing lockfile: ' + repr(e))
+
 def help_docs(window):
     """Shell out to display help documents."""
     # TODO
+    # windows: use os.startfile
+    # nix: use gtk.help (old-style)
     print(u'Help Docs Not Implemented.')
 
 def about_dlg(window):
