@@ -65,13 +65,6 @@ key_abort = 'F5'	# (+CTRL)
 key_startlist = 'F3'
 key_results = 'F4'
 
-# SCB constants
-#SCB_STARTERS_NW = metarace.SCB_LINELEN - 9
-#SCB_STARTERS_FMT = [(3, 'r'), ' ', (SCB_STARTERS_NW,'l'), ' ', (4,'r')]
-
-SCB_RESMEDAL_NW = metarace.SCB_LINELEN - 7
-SCB_RESMEDAL_FMT = [(6,u'l'), u' ', (SCB_RESMEDAL_NW,u'l')]
-
 class classification(object):
     def loadconfig(self):
         """Load race config from disk."""
@@ -103,7 +96,7 @@ class classification(object):
         if eid and eid != EVENT_ID:
             self.log.error('Event configuration mismatch: '
                            + repr(eid) + ' != ' + repr(EVENT_ID))
-            self.readonly = True
+            #self.readonly = True
 
     def startlist_report(self, program=False):
         """Return a startlist report."""
@@ -171,7 +164,7 @@ class classification(object):
 
             yield [bib, rank, time, info]
 
-    def result_report(self):
+    def result_report(self, recurse=True): # by default include inners
         """Return a list of printing sections containing the race result."""
         ret = []
 
@@ -211,20 +204,21 @@ class classification(object):
             sec.lines.append(nrow)
         ret.append(sec)
 
-        # then append each of the specified events
-        for evno in self.showevents.split():
-            if evno:
-                self.log.info('looking for event: ' + repr(evno))
-                r = self.meet.get_event(evno, False)
-                if r is None:
-                    self.log.error('Invalid event in showplaces.')
-                    continue
-                r.loadconfig()	# now have queryable event handle
-                if r.onestart:	# go for result
-                    ret.extend(r.result_report())
-                else:		# go for startlist
-                    ret.extend(r.startlist_report())
-                r.destroy()
+        if recurse:
+            # then append each of the specified events
+            for evno in self.showevents.split():
+                if evno:
+                    self.log.info('looking for event: ' + repr(evno))
+                    r = self.meet.get_event(evno, False)
+                    if r is None:
+                        self.log.error('Invalid event in showplaces.')
+                        continue
+                    r.loadconfig()	# now have queryable event handle
+                    if r.onestart:	# go for result
+                        ret.extend(r.result_report())
+                    else:		# go for startlist
+                        ret.extend(r.startlist_report())
+                    r.destroy()
         return ret
 
     def addrider(self, bib='', place=''):
@@ -382,17 +376,38 @@ class classification(object):
         # Draw a 'medal ceremony' on the screen
         resvec = []
         count = 0
+        teamnames = False
+        name_w = self.meet.scb.linelen - 12
+        fmt = [(3,u'l'), (4,u'r'), u' ',(name_w, u'l'), (4,u'r')]
+        if self.series and self.series[0].lower() == u't':
+            teamnames = True
+            name_w = self.meet.scb.linelen - 8
+            fmt = [(3,u'l'), u' ',(name_w, u'l'), (4,u'r')]
+
         for r in self.riders:
-            resvec.append([r[COL_MEDAL],
-                           strops.fitname(r[COL_FIRST], r[COL_LAST],
-                                        SCB_RESMEDAL_NW)])
+            plstr = r[COL_PLACE].decode('utf-8')
+            if plstr.isdigit():
+                #if int(plstr) > 20:
+                    #break
+                plstr = plstr + u'.'
+            no = r[COL_BIB]
+            first=r[COL_FIRST].decode('utf-8')
+            last=r[COL_LAST].decode('utf-8')
+            club=r[COL_CLUB].decode('utf-8')
+            if not teamnames:
+                resvec.append([plstr, no, strops.fitname(first,last, name_w),
+                                     club])
+            else:
+                resvec.append([plstr, first, club])
             count += 1
-            if count > 2:	# for now hardcode g,s,v
-                break
         self.meet.scbwin = None
-        self.meet.scbwin = scbwin.scbintsprint(self.meet.scb,
-                                              self.event[u'pref'],
-                             self.event[u'info'], SCB_RESMEDAL_FMT, resvec)
+        header=self.meet.racenamecat(self.event)
+        ## TODO: Flag Provisional
+        evtstatus='Final Classification'.upper()
+        self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb,
+                                  head=self.meet.racenamecat(self.event),
+                                  subhead=evtstatus,
+                                  coldesc=fmt, rows=resvec)
         self.meet.scbwin.reset()
         return False
 
@@ -403,6 +418,10 @@ class classification(object):
             self.saveconfig()
         self.winopen = False
 
+    def rftimercb(self, e):
+        """Handle rftimer event."""
+        return False
+
     def timercb(self, e):
         """Handle a timer event."""
         return False
@@ -440,7 +459,7 @@ class classification(object):
             ns = se.get_text()
             if ns != self.series:
                 self.series = ns
-                self.meet.edb.editevent(self.event, series=ns)
+                self.event[u'seri'] = ns
 
             self.recalculate()
             glib.idle_add(self.delayed_announce)
