@@ -124,7 +124,7 @@ class dbexport(threading.Thread):
         self.queue.put_nowait(('PORT', port))
 
     def connected(self):
-        """Return true if SCB connected."""
+        """Return true if db connected."""
         return self.port is not None
 
     def __procsql(self, cmd, args):
@@ -159,12 +159,12 @@ class dbexport(threading.Thread):
                 elif m[0] == 'PORT':
                     if self.port is not None:
                         self.port.close()
-                        self.port = None	# should get __del__ here
-                    if m[1] is not None and m[1] != u'' and m[1] != u'NULL':
+                    if m[1] is not None and m[1] not in ['none', 'NULL']:
                         self.log.debug(u'Re-Connect port: ' + str(m[1]))
                         self.port = self.__connect(str(m[1]))
                     else:
                         self.log.debug(u'Not connected.')
+                        self.port = None	# should get __del__ here
             except Exception as e:
                 self.log.error(u'Exception: ' + repr(type(e)) + repr(e))
                 if self.port is not None:
