@@ -68,12 +68,6 @@ COL_START = 7
 COL_FINISH = 8
 COL_100 = 9		# 100m split ToD
 
-# scb consts
-SCB_NAME_WIDTH = metarace.SCB_LINELEN - 9
-SCB_RESNAME_WIDTH = metarace.SCB_LINELEN - 11
-SCB_RESULT_FMT = [(2, u'l'), (3, u'r'), u' ',
-                  (SCB_RESNAME_WIDTH, u'l'), u' ', (4, u'r')]
-
 # scb function key mappings (also trig announce)
 key_reannounce = 'F4'                # (+CTRL) calls into delayed announce
 key_startlist = 'F3'                 # startlist
@@ -227,26 +221,42 @@ class f200(object):
         self.meet.announce.gfx_set_title(u'Result: '
                             + self.event[u'pref'] + u' '
                             + self.event[u'info'])
+        name_w = self.meet.scb.linelen - 11
+        pcount = 0
+        rcount = 0
         for r in self.riders:
+            rcount += 1
             if r[COL_PLACE] is not None and r[COL_PLACE] != u'':
-                if r[COL_PLACE].isdigit() and int(r[COL_PLACE]) > 12:
-                    break
+                pcount += 1
+                plstr = r[COL_PLACE]
+                if r[COL_PLACE].isdigit():
+                    plstr += u'.'
+                    #if int(r[COL_PLACE]) > 12:
+                        #break
                 first = r[COL_FIRSTNAME].decode('utf-8')
                 last = r[COL_LASTNAME].decode('utf-8')
                 club = r[COL_CLUB].decode('utf-8')
-                plstr = r[COL_PLACE] + u'.'
+                bib = r[COL_BIB]
                 elapstr = ''
                 if r[COL_START] and r[COL_FINISH]:
                     elapstr = (r[COL_FINISH]-r[COL_START]).rawtime(3)
-                fmtplaces.append([r[COL_PLACE], r[COL_BIB],
-                            strops.fitname(first, last, SCB_RESNAME_WIDTH),
+                fmtplaces.append([plstr, bib,
+                            strops.fitname(first, last, name_w),
                                   club])
                 self.meet.announce.gfx_add_row([plstr, 
                             strops.resname(first, last, club),
                             elapstr])
-        self.meet.scbwin = scbwin.scbtable(self.meet.scb,
-                       self.meet.racenamecat(self.event),
-                       SCB_RESULT_FMT, fmtplaces, delay=80)
+        FMT = [(3,u'l'), (3,u'r'), u' ',(name_w, u'l'), (4,u'r')]
+
+        evtstatus = u'Standings'
+        if rcount > 0 and pcount == rcount:
+            evtstatus = u'Result'
+
+        self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb,
+                               head=self.meet.racenamecat(self.event),
+                               subhead=evtstatus.upper(),
+                               coldesc=FMT,
+                               rows=fmtplaces)
         self.meet.scbwin.reset()
 
     def todstr(self, col, cr, model, iter, data=None):
@@ -368,7 +378,7 @@ class f200(object):
         if eid and eid != EVENT_ID:
             self.log.error(u'Event configuration mismatch: '
                            + repr(eid) + u' != ' + repr(EVENT_ID))
-            self.readonly = True
+            #self.readonly = True
 
     def saveconfig(self):
         """Save race to disk."""
@@ -506,7 +516,7 @@ class f200(object):
         if substr:
             sec.subheading = substr
         if self.event[u'reco']:
-            sec.recordstr = self.event[u'reco']
+            sec.footer = self.event[u'reco']
         sec.lines = self.get_heats()
         ret.append(sec)
         return ret
@@ -643,7 +653,7 @@ class f200(object):
             ns = se.get_text()
             if ns != self.series:
                 self.series = ns
-                self.meet.edb.editevent(self.event, series=ns)
+                self.event[u'seri'] = ns
 
             # xfer starters if not empty
             slist = strops.reformat_riderlist(
@@ -682,7 +692,7 @@ class f200(object):
 
             yield [bib, rank, time, info]
 
-    def result_report(self):
+    def result_report(self, recurse=False):
         """Return a list of printing sections containing the race result."""
         self.placexfer()
         ret = []
@@ -698,12 +708,17 @@ class f200(object):
         sec.lines = []
         ftime = None
         for r in self.riders:
-            rno = r[COL_BIB]
+            rno = r[COL_BIB].decode('utf-8')
             rank = None
-            rname = strops.resname(r[COL_FIRSTNAME], r[COL_LASTNAME],
-                                      r[COL_CLUB])
+            rname = strops.resname(r[COL_FIRSTNAME].decode('utf-8'),
+                                   r[COL_LASTNAME].decode('utf-8'),
+                                   r[COL_CLUB].decode('utf-8'))
             rtime = None
             rcat = None		# should check ev[u'cate']
+            if self.event[u'cate']:
+                dbr = self.meet.rdb.getrider(rno, self.series)
+                if dbr is not None:
+                    rcat = self.meet.rdb.getvalue(dbr,4)
             info = None
             dtime = None
             if self.onestart:
@@ -720,7 +735,7 @@ class f200(object):
                         dtime = u'+' + (time - ftime).rawtime(2)
                     rtime = time.rawtime(3)
 
-            sec.lines.append([rank, rno, rname, None, rtime, dtime])
+            sec.lines.append([rank, rno, rname, rcat, rtime, dtime])
         ret.append(sec)
         return ret
 
@@ -802,6 +817,13 @@ class f200(object):
         glib.idle_add(self.delayed_announce)
         self.meet.delayed_export()
 
+    def rftimercb(self, e):
+        """Handle rftimer event."""
+        if e.refid == '':       # got a trigger
+            #return self.starttrig(e)
+            return False
+        return False
+
     def timercb(self, e):
         """Handle a timer event."""
         chan = timy.chan2id(e.chan)
@@ -1034,6 +1056,7 @@ class f200(object):
 
     def fmtridername(self, tp):
         """Prepare rider name for display on scoreboard."""
+        name_w = self.meet.scb.linelen - 9
         bib = tp.getrider().strip()
         if bib != '':
             name = '[New Rider]'
@@ -1044,11 +1067,11 @@ class f200(object):
                 club = r[COL_CLUB].decode('utf-8')
                 name = strops.fitname(first,
                                       last,
-                                      SCB_NAME_WIDTH)
+                                      name_w)
                 tp.namevec = [bib, strops.resname(first, last, club), '']
             return ' '.join([strops.truncpad(r[COL_BIB], 3, 'r'),
-                             strops.truncpad(name, SCB_NAME_WIDTH),
-                             strops.truncpad(r[COL_CLUB], 4, 'r')])
+                             strops.truncpad(name, name_w),
+                             strops.truncpad(club, 4, 'r')])
         else:
             tp.namevec = None
             return ''
@@ -1082,6 +1105,7 @@ class f200(object):
             self.meet.timer.arm(timy.CHAN_200)
             self.showtimerwin()
             self.meet.printimp(True)
+            self.meet.delayimp('0.01')
             if self.fs.status == 'armstart':
                 self.meet.scbwin.sett1('       0.0     ')
                 nstr = self.fs.biblbl.get_text()
@@ -1100,6 +1124,7 @@ class f200(object):
             self.fs.toidle()
         self.timerstat = 'idle'
         self.meet.printimp(False)
+        self.meet.delayimp('1.00')
         self.curstart = None
         self.lstart = None
         for i in range(0,8):
