@@ -21,12 +21,6 @@ This module provides a class 'flap' which implements the 'race'
 interface and manages data, timing and scoreboard for the
 UCI flying lap time trial.
 
-Peculiarities when compared to ittt:
-
- - times are hard truncated to thousandths
- - dead heat on thousandths is broken by time over last 200m
- - autoarm is the default
-
 """
 
 import gtk
@@ -56,20 +50,16 @@ EVENT_ID = 'trackflap-1.3'
 COL_BIB = 0
 COL_FIRSTNAME = 1
 COL_LASTNAME = 2
+COL_CAT = 2
 COL_CLUB = 3
-COL_COMMENT = 4		# 
-COL_SEED = 5		# Change to seed for future versions
+COL_INRACE = 3		# mix with comment for dns/dnf etc
+COL_COMMENT = 4	
+COL_SEED = 5
 COL_PLACE = 6
 COL_START = 7
 COL_FINISH = 8
 COL_200 = 9		# 200m split ToD
 
-# scb consts
-SCB_NAME_WIDTH = metarace.SCB_LINELEN - 9
-SCB_RESNAME_WIDTH = metarace.SCB_LINELEN - 11
-SCB_RESULT_FMT = [(2, 'l'), (3, 'r'), ' ',
-                  (SCB_RESNAME_WIDTH, 'l'), ' ', (4, 'r')]
-
 # scb function key mappings (also trig announce)
 key_reannounce = 'F4'                # (+CTRL) calls into delayed announce
 key_startlist = 'F3'                 # startlist
@@ -219,17 +209,32 @@ class flap(object):
         self.meet.scbwin = None
         self.timerwin = False     # TODO: bib width enhancement
         fmtplaces = []
+        name_w = self.meet.scb.linelen - 11
+        pcount = 0
+        rcount = 0
         for r in self.riders:
+            rcount += 1
             if r[COL_PLACE] is not None and r[COL_PLACE] != '':
-                if r[COL_PLACE].isdigit() and int(r[COL_PLACE]) > 20:
-                    break
-                fmtplaces.append([r[COL_PLACE], r[COL_BIB],
-                                  strops.fitname(r[COL_FIRSTNAME],
-                                  r[COL_LASTNAME], SCB_RESNAME_WIDTH),
-                                  r[COL_CLUB]])
-        self.meet.scbwin = scbwin.scbtable(self.meet.scb,
-                       self.meet.racenamecat(self.event),
-                       SCB_RESULT_FMT, fmtplaces, delay=90)
+                pcount += 1
+                plstr = r[COL_PLACE].decode('utf-8')
+                if plstr.isdigit():
+                    if int(plstr) > 20:
+                        break
+                    plstr = plstr + u'.'
+                fmtplaces.append([plstr,
+                        r[COL_BIB].decode('utf-8'),
+                        strops.fitname(r[COL_FIRSTNAME].decode('utf-8'),
+                        r[COL_LASTNAME].decode('utf-8'), name_w),
+                        r[COL_CLUB].decode('utf-8')])
+        FMT = [(3,u'l'), (3,u'r'), u' ',(name_w, u'l'), (4,u'r')]
+        evtstatus = u'Standings'
+        if rcount > 0 and pcount == rcount:
+            evtstatus = u'Result'
+        self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb,
+                              head=self.meet.racenamecat(self.event),
+                              subhead=evtstatus,
+                              coldesc=FMT,
+                              rows=fmtplaces)
         self.meet.scbwin.reset()
 
     def todstr(self, col, cr, model, iter, data=None):
@@ -289,7 +294,7 @@ class flap(object):
         self.inomnium = strops.confopt_bool(cr.get('event', 'inomnium'))
         if self.inomnium:
             # SWITCH: Event is part of an omnium, make any req'd overrides
-            self.seedsrc = 3    # read seeding from omnium points standinds
+            self.seedsrc = 1    # read seeding from omnium ranking
 
         # re-load starters/results
         self.onestart = False
@@ -351,7 +356,7 @@ class flap(object):
         if eid and eid != EVENT_ID:
             self.log.error('Event configuration mismatch: '
                            + repr(eid) + ' != ' + repr(EVENT_ID))
-            self.readonly = True
+            #self.readonly = True
 
     def saveconfig(self):
         """Save race to disk."""
@@ -438,10 +443,14 @@ class flap(object):
             count = placeholders
         if placeholders == 0:
             for r in self.riders:
-                rno = r[COL_BIB]
-                rname = strops.resname(r[COL_FIRSTNAME], r[COL_LASTNAME],
-                                       r[COL_CLUB])
-                hlist.append([str(count)+'.1', rno, rname, None])
+                rno = r[COL_BIB].decode('utf-8')
+                rh = self.meet.newgetrider(rno, self.series)
+                rname = u''
+                uci = None
+                if rh is not None:
+                    rname = rh[u'namestr']
+                    if rh[u'ucicode']: uci = rh[u'ucicode']
+                hlist.append([str(count)+'.1', rno, rname, uci])
                                  # all heats are one up
                 count -= 1
         else:
@@ -471,20 +480,31 @@ class flap(object):
             ret.append(rec)
         return ret
 
-    def startlist_report(self):
+    def startlist_report(self, program=False):
         """Return a startlist report."""
         ret = []
         cnt = 0
         sec = printing.dual_ittt_startlist()
         sec.set_single()
 
-        sec.heading = ' '.join([
-                      'Event', self.evno, ':',
-                  self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                  self.meet.edb.getvalue(self.event, eventdb.COL_INFO),
-                      '- Start List'])
-        sec.lines = self.get_heats()
-        return sec
+        headvec = [u'Event', self.evno, u':', self.event[u'pref'],
+                         self.event[u'info']]
+        if not program:
+            headvec.append(u'- Start List')
+        sec.heading = u' '.join(headvec)
+        lapstring = strops.lapstring(self.event[u'laps'])
+        substr = u' '.join([lapstring, self.event[u'dist'],
+                             self.event[u'prog']]).strip()
+        if substr:
+            sec.subheading = substr
+        if self.event[u'reco']:
+            sec.footer = self.event[u'reco']
+        if self.event[u'plac']:
+            sec.lines = self.get_heats(placeholders = self.event[u'plac'])
+        else:
+            sec.lines = self.get_heats()
+        ret.append(sec)
+        return ret
 
     def get_startlist(self):
         """Return a list of bibs in the rider model."""
@@ -498,9 +518,8 @@ class flap(object):
         if self.winopen:
             self.meet.announce.clrall()
             self.meet.ann_title(' '.join([
-                  'Event', self.evno, ':',
-                  self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                  self.meet.edb.getvalue(self.event, eventdb.COL_INFO)]))
+                  u'Event', self.evno, u':',
+                  self.event[u'pref'], self.event[u'info']]))
 
             self.meet.announce.linefill(1, '_')
             self.meet.announce.linefill(7, '_')
@@ -617,7 +636,7 @@ class flap(object):
             ns = se.get_text()
             if ns != self.series:
                 self.series = ns
-                self.meet.edb.editevent(self.event, series=ns)
+                self.event[u'seri'] = ns
 
             # xfer starters if not empty
             slist = strops.reformat_riderlist(
@@ -656,23 +675,32 @@ class flap(object):
 
             yield [bib, rank, time, info]
 
-    def result_report(self):
+    def result_report(self, recurse=False):
         """Return a list of printing sections containing the race result."""
         ret = []
         sec = printing.section()
-        sec.heading = 'Event ' + self.evno + ': ' + ' '.join([
-                       self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                       self.meet.edb.getvalue(self.event, eventdb.COL_INFO)
+        sec.heading = u'Event ' + self.evno + u': ' + u' '.join([
+                        self.event[u'pref'], self.event[u'info']
                               ]).strip()
         sec.lines = []
         ftime = None
+        rcount = 0
+        pcount = 0
         for r in self.riders:
-            rno = r[COL_BIB]
+            rcount += 1		# count all rders in model
+            rno = r[COL_BIB].decode('utf-8')
+            rh = self.meet.newgetrider(rno, self.series)
             rank = None
-            rname = strops.resname(r[COL_FIRSTNAME], r[COL_LASTNAME],
-                                      r[COL_CLUB])
+            rname = u''
+            if rh is not None:
+                rname = rh[u'namestr']
             rtime = None
             rcat = None
+            if self.event[u'cate']:
+                if rh is not None and rh[u'cat']:
+                    rcat = rh[u'cat']
+            if rh is not None and rh[u'ucicode']:
+                rcat = rh[u'ucicode']	# overwrite by force
             info = None
             dtime = None
             if self.onestart:
@@ -681,6 +709,7 @@ class flap(object):
                         rank = r[COL_PLACE] + '.'
                     else:
                         rank = r[COL_PLACE]
+                    pcount += 1
                 if r[COL_FINISH] is not None:
                     time = (r[COL_FINISH]-r[COL_START]).truncate(3)
                     if ftime is None:
@@ -689,62 +718,31 @@ class flap(object):
                         dtime = '+' + (time - ftime).rawtime(2)
                     rtime = time.rawtime(3)
 
-            sec.lines.append([rank, rno, rname, None, rtime, dtime])
+            sec.lines.append([rank, rno, rname, rcat, rtime, dtime])
+
+        # section subheading
+        lapstring = strops.lapstring(self.event[u'laps'])
+        substr = u' '.join([lapstring, self.event[u'dist'],
+                             self.event[u'prog']]).strip()
+        if self.onestart:
+            sec.subheading = substr
+            if rcount > 0 and pcount < rcount:
+                sec.subheading += u' - STANDINGS'
+            else:
+                sec.subheading += u' - Result'
+        else:
+            if substr:
+                sec.subheading = substr
+        
         ret.append(sec)
         return ret
 
-    def result_export(self, f):
-        """Export results to supplied file handle."""
-        cr = csv.writer(f)
-        df = '250m'	# fixed in this case
-        header = ['Event ' + self.evno, '', ' '.join([
-                       self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                       self.meet.edb.getvalue(self.event, eventdb.COL_INFO)
-                              ]).strip(), '',
-                  df, '50m', 'last 200']
-        cr.writerow(header)
-        for r in self.riders:
-            rno = r[COL_BIB]
-            rname = strops.resname(r[COL_FIRSTNAME], r[COL_LASTNAME],
-                                      r[COL_CLUB])
-            plstr = r[COL_PLACE]
-            if plstr != '':
-                plstr = "'" + plstr
-                if r[COL_PLACE].isdigit():
-                    plstr += '.'
-            if r[COL_PLACE] != '' or r[COL_COMMENT] != '':
-                if r[COL_START] is None:
-                    r[COL_START] = tod.tod(0)
-                elap = ''
-                if r[COL_FINISH] is not None:
-                    elap = (r[COL_FINISH]-r[COL_START]).rawtime(3)
-                cmt = r[COL_COMMENT]
-                #if cmt == '':
-                    #dbr = self.meet.rdb.getrider(r[COL_BIB], self.series)
-                    #if dbr is not None:
-                        #cmt = self.meet.rdb.getvalue(dbr,
-                                                   #riderdb.COL_CAT)
-                resrow = [plstr,rno, rname, cmt, "'" + elap]
-                lt = r[COL_START]
-                if r[COL_200] is not None:
-                    resrow.append("'"
-                             + (r[COL_200]-lt).rawtime(3)) # f50
-                    if r[COL_FINISH] is not None:
-                        resrow.append("'"
-                        + (r[COL_FINISH] - r[COL_200]).rawtime(3)) # l200
-                else:
-                    resrow.append('')	# f50
-                    resrow.append('')	# l200
-                cr.writerow(resrow)
-            else:
-                cr.writerow([plstr,rno,rname,'',''])
-
     def editent_cb(self, entry, col):
         """Shared event entry update callback."""
-        if col == eventdb.COL_PREFIX:
-            self.meet.edb.editevent(self.event, prefix=entry.get_text())
-        elif col == eventdb.COL_INFO:
-            self.meet.edb.editevent(self.event, info=entry.get_text())
+        if col == u'pref':
+            self.event[u'pref'] = entry.get_text().decode('utf-8', 'replace')
+        elif col == u'info':
+            self.event[u'info'] = entry.get_text().decode('utf-8', 'replace')
         self.update_expander_lbl_cb()
 
     def update_expander_lbl_cb(self):
@@ -809,6 +807,13 @@ class flap(object):
         glib.idle_add(self.delayed_announce)
         self.meet.delayed_export()
 
+    def rftimercb(self, e):
+        """Handle rftimer event."""
+        if e.refid == '':       # got a trigger
+            #return self.starttrig(e)
+            return False
+        return False
+
     def timercb(self, e):
         """Handle a timer event."""
         chan = timy.chan2id(e.chan)
@@ -898,7 +903,8 @@ class flap(object):
                     nr[i] = self.meet.rdb.getvalue(dbr, i)
             return self.riders.append(nr)
         else:           # patch 'seed' on an existing record
-            ri[COL_SEED] = istr
+            if not ri[COL_SEED] and istr:
+                ri[COL_SEED] = istr
             return True         # not sure about this one?
 
     def editcol_db(self, cell, path, new_text, col):
@@ -1040,17 +1046,24 @@ class flap(object):
 
     def fmtridername(self, tp):
         """Prepare rider name for display on scoreboard."""
+        name_w = self.meet.scb.linelen - 9
         bib = tp.getrider().strip()
         if bib != '':
             name = '[New Rider]'
             r = self.getrider(bib)
             if r is not None and r[COL_BIB] != '':
-                name = strops.fitname(r[COL_FIRSTNAME], r[COL_LASTNAME],
-                                      SCB_NAME_WIDTH)
+                first = r[COL_FIRSTNAME].decode('utf-8')
+                last = r[COL_LASTNAME].decode('utf-8')
+                club = r[COL_CLUB].decode('utf-8')
+                name = strops.fitname(first,
+                                      last,
+                                      name_w)
+            tp.namevec = [bib, strops.resname(first, last, club), '']
             return ' '.join([strops.truncpad(r[COL_BIB], 3, 'r'),
-                             strops.truncpad(name, SCB_NAME_WIDTH),
-                             strops.truncpad(r[COL_CLUB], 4, 'r')])
+                             strops.truncpad(name, name_w),
+                             strops.truncpad(club, 4, 'r')])
         else:
+            tp.namevec = None
             return ''
         
     def showtimerwin(self):
@@ -1308,15 +1321,11 @@ class flap(object):
         b.get_object('race_info_evno').set_text(self.evno)
         self.showev = b.get_object('race_info_evno_show')
         self.prefix_ent = b.get_object('race_info_prefix')
-        self.prefix_ent.connect('changed', self.editent_cb,
-                                 eventdb.COL_PREFIX)
-        self.prefix_ent.set_text(self.meet.edb.getvalue(
-                   self.event, eventdb.COL_PREFIX))
+        self.prefix_ent.connect('changed', self.editent_cb,u'pref')
+        self.prefix_ent.set_text(self.event[u'pref'])
         self.info_ent = b.get_object('race_info_title')
-        self.info_ent.connect('changed', self.editent_cb,
-                               eventdb.COL_INFO)
-        self.info_ent.set_text(self.meet.edb.getvalue(
-                   self.event, eventdb.COL_INFO))
+        self.info_ent.connect('changed', self.editent_cb,u'info')
+        self.info_ent.set_text(self.event[u'info'])
         b.get_object('race_type').set_text('Flying Lap')
         
 
