@@ -58,30 +58,57 @@ ETX=0x03
 LF=0x0a
 GFXHEAD = bytearray([EOT, EOT, 0x30, 0x47])
 BRIGHTVALS = [0x31, 0x32, 0x33]
+ELAPOFT = tod.tod('0.1')
 
 DCONF_HDR = bytearray([0x82, 0x81, 0x20, 0x30,  # 2x1 displays, horizontal
                        0x85, 0x94, 0xb2,        # PWM levels 1, 2, 3 -0x80
                        0x20,    # spare...
                        0x20, 0x20, 0x20, 0x20,
                        0x20, 0x20, 0x20, 0x20])
-DPANEL_HDR = bytearray([0x31,                   # 'small' size
-                        0x7f, 0x7f, 0x7f, 0x7f, # line addresses
+DPANEL_HDR1 = bytearray([0x31,                  # 0x31=small/0x32=big
+                        0x81, 0x82, 0x85, 0x86, # line addresses
                         0x80, 0x30, 0x31,       # disable CP mode
                         0x20, 0x20, 0x20, 0x20, # spare...
                         0x20, 0x20, 0x20, 0x20])
-MRPANEL_LAYOUT = [ [0x81, 0x82, 0x85, 0x86],
+DPANEL_HDR2 = bytearray([0x31,
+                        0x83, 0x84, 0x87, 0x88, # line addresses
+                        0x80, 0x30, 0x31,       # disable CP mode
+                        0x20, 0x20, 0x20, 0x20, # spare...
+                        0x20, 0x20, 0x20, 0x20])
+MRPANEL_SINGLE = [ [0x81, 0x82, 0x85, 0x86],
                    [0x83, 0x84, 0x87, 0x88] ]
+MRPANEL_DBL = [ [0x81, 0x7f, 0x7f, 0x7f],
+                [0x85, 0x7f, 0x7f, 0x7f] ]
 
-def mk_config_buf():
+## Panel layouts:
+
+# 'single': 32 chars, 2 lines
+#
+#   ----------------------------------------
+# 	0x81	0x82	|	0x83	0x84
+#	0x85	0x86	|	0x87	0x88
+#   ----------------------------------------
+#
+# 'double': 16 chars, 1 line double sized
+#
+#   ----------------------------------------
+#       0x81		|	0x83
+#
+#   ----------------------------------------
+
+def mk_config_buf(single=True):
     ret = bytearray()
     # add header and raw display blocks
     ret.extend(DCONF_HDR)
-    ret.extend(DPANEL_HDR)
-    ret.extend(DPANEL_HDR)
-    # fill in line offsets
+    ret.extend(DPANEL_HDR1)
+    ret.extend(DPANEL_HDR2)
+    # fill in panel layout
+    layout = MRPANEL_SINGLE
+    if not single:
+        layout = MRPANEL_DBL
     for i in range(0,4):
-       ret[16+1+i] = MRPANEL_LAYOUT[0][i]
-       ret[32+1+i] = MRPANEL_LAYOUT[1][i]
+       ret[16+1+i] = layout[0][i]
+       ret[32+1+i] = layout[1][i]
     return ret
 
 def iblrc(byteStr):
@@ -114,6 +141,43 @@ def i2b16(val):
 def i2b24(val):
     return bytearray([(val>>16)&0xff, (val>>8)&0xff, val&0xff])
 
+def vc2dc(vcol, vrow):
+    """Translate a vertical coordinate to display space."""
+    ret = None
+    if vcol >= 0 and vcol < 16:
+        ret = (vrow+96, 15-vcol)	# reflected on 2nd panel
+    elif vcol < 32:
+        ret = (vrow, 31-vcol)	# reflected on 1st panel
+    return ret
+
+def cset(buf,vcol,vrow,csrc):
+    """Draw the given character at the specified vcol and vrow."""
+    for i in range(0,6):
+        for j in range(0,8):
+            bset(buf, i+vcol, j+vrow, csrc[i]&(1<<j))            
+
+def nset(buf,vcol,vrow,csrc):
+    """Draw the given supersize number at vcol, vrow."""
+    for i in range(0,16):
+        for j in range(0,22):
+            bset(buf, i+vcol, j+vrow, csrc[i]&(1<<j))            
+
+def bset(buf,vcol,vrow,val):
+    """Set/clear bit in vertical buf with no checks."""
+    (dcol, drow) = vc2dc(vcol, vrow)
+    i=24*drow+dcol//8
+    j=7-(dcol%8)
+    if val:	# set
+        buf[i] |= (1<<j)
+    else:	# clear
+        buf[i] &= ~(1<<j)
+
+def logo2pbm(src):
+    """Write out pbm."""
+    with open(u'test.pbm', 'wb') as f:
+        f.write('P4\n')
+        f.write('192 16\n')
+        f.write(src)
 
 class hl970:
     """HL970 control console application."""
@@ -208,7 +272,7 @@ class hl970:
                             dstr = u'+' + (self.tod - self.elapfin).rawtime(0)
                 else:
                     # race in progress, show run time and distance or lap
-                    elap = self.tod - (self.elapstart+tod.tod('0.2'))
+                    elap = self.tod - (self.elapstart+ELAPOFT)
                     if elap > tod.MAXELAP:
                         elap = tod.ZERO
                     estr = elap.rawtime(0)
@@ -256,14 +320,15 @@ class hl970:
                     lstr = self.laplbl.rstrip()
                 if self.lapstart is not None:	# try to first write lap time
                     if self.lapfin is not None:
-                        # lap down time overwrites dist, but only if valid
                         laptm = self.lapfin - self.lapstart
-                        if laptm < self.maxlaptime:
-                            tstr = laptm.rawtime(0)
+                        # completed lap time
+                        tstr = laptm.rawtime(0)	# show lap even if > maxtm
                     elif self.timerstat != u'finished':
-                        laptm = self.tod - (self.lapstart +tod.tod('0.5'))
-                        # shorten lap time by one half interval
-                        if laptm < self.maxlaptime:
+                        # rolling lap time
+                        if (self.elapstart is not None
+                            and self.lapstart != self.elapstart):
+                            laptm = self.tod - (self.lapstart+ELAPOFT)
+                            # shorten lap time for upcnt
                             tstr = laptm.rawtime(0)
                 curtmmod = int(self.tod.timeval/3)%2
                 if self.laptype and curtmmod:      # but overwrite if type
@@ -327,6 +392,76 @@ class hl970:
 
     ## UI callbacks
 
+    def load_vfonts(self):
+        """Load the vertical emulation fonts."""
+        try:
+            # Start with rotated board charset
+            filename=metarace.default_file(u'ThFont-ISO8859-15.bmp')
+            pb = gtk.gdk.pixbuf_new_from_file(filename)
+            w = pb.get_width()
+            h = pb.get_height()
+            if h == 8 and w == 1344:
+                self.vfont = []
+                px = pb.get_pixels()
+                pxcount = 3	# RGB
+                if pb.get_has_alpha():
+                    pxcount = 4	# RGBA
+                self.log.debug(u'Total pixels: ' + repr(len(px)/pxcount))
+
+                # read in each char
+                for i in range(0,224):
+                    thischar = bytearray(6)
+                    charof = pxcount*6*i
+                    for j in range(0,8):
+                        rowstart = j*pxcount*1344 + charof
+                        for k in range(0,6):
+                            srcpos = rowstart + pxcount*k
+                            mval = 0
+                            for p in range(0,3):
+                                mval += ord(px[srcpos+p])
+                            if mval < 384:
+                                thischar[k] |= (1<<j)
+                    self.vfont.append(thischar)
+                self.log.debug(u'Loaded vertical emulation font.')
+            else:
+                self.log.warn(u'Invalid font dimension: ' + unicode(w) 
+                                 + u'x' + unicode(h))
+            # Then load the xtra large vertical font
+            filename=metarace.default_file(u'ThFont-Numeric-16x22.bmp')
+            pb = gtk.gdk.pixbuf_new_from_file(filename)
+            w = pb.get_width()
+            h = pb.get_height()
+            if h == 22 and w == 192:
+                self.vlfont = []
+                px = pb.get_pixels()
+                pxcount = 3	# RGB
+                if pb.get_has_alpha():
+                    pxcount = 4	# RGBA
+                self.log.debug(u'Total pixels: ' + repr(len(px)/pxcount))
+
+                # read in each char
+                for i in range(0,12):	# 12 chars
+                    thischar = []
+                    for l in range(0,16): thischar.append(0x00000000)
+                    charof = pxcount*16*i	# of width 16
+                    for j in range(0,22):	# and height 22
+                        rowstart = j*pxcount*192 + charof # rows of 192px
+                        for k in range(0,16):	# width 16 px
+                            srcpos = rowstart + pxcount*k
+                            mval = 0
+                            for p in range(0,3):
+                                mval += ord(px[srcpos+p])
+                            if mval < 384:
+                                thischar[k] |= (1<<j)
+                    self.vlfont.append(thischar)
+                self.log.debug(u'Loaded large vertical numeric font.')
+            else:
+                self.log.warn(u'Invalid font dimension: ' + unicode(w) 
+                                 + u'x' + unicode(h))
+        except Exception as e:
+            self.log.error(u'Loading emulation font: ' + repr(e))
+            self.vfont = None
+
     def bmp2dimg(self, filename):
         """Try and load the supplied image as dimg array."""
         try:
@@ -368,11 +503,19 @@ class hl970:
             self.logo = data
         except Exception as e:
             self.log.error(u'Loading logo file: ' + repr(e))
+            self.logo = mrlogobmp.METARACE
 
     def but_config_clicked_cb(self, button, data=None):
         """Start config."""
         self.log.debug(u'App config cb.')
         self.dosave = True
+        # re-write config to display
+        conf = mk_config_buf()
+        self.log.debug(u'Send Config: ' + repr(conf))
+        self.stdcmd(0x91, ord('S'), conf)
+        self.log.debug(u'Clear')
+        self.serialwrite(chr(STX) + chr(0x90) + '111234567890' + chr(LF))
+        self.serialwrite(chr(STX) + chr(0xA0) + '112345678' + chr(LF))
 
     def but_man_clicked_cb(self, button, data=None):
         """Manual display set."""
@@ -450,6 +593,7 @@ class hl970:
 
     def but_clear_clicked_cb(self, button, data=None):
         """Request clear on display."""
+        self.reset()		# reset all but finpanel runtime vars
         self.clearlines()	# empty local output buffer
         self.clearall()
 
@@ -471,8 +615,6 @@ class hl970:
         #self.ttname = None
         #self.ttcat = None
         #self.tttime = None
-        #self.clearlines()	# empty local output buffer
-        #self.clearall()
 
     def serialwrite(self, cmd):
         """Output command blocking."""
@@ -539,6 +681,9 @@ class hl970:
             self.elapstart = tod.str2tod(cmd.text)
         elif cmd.header == u'finish':
             self.elapfin = tod.str2tod(cmd.text)
+        elif cmd.header == u'redraw' and cmd.text == u'timer':
+            # fake a timeout in the cb
+            self.process_timeout()
         elif cmd.header == u'lapstart':
             self.lapstart = tod.str2tod(cmd.text)
         elif cmd.header == u'lapfin':
@@ -591,10 +736,106 @@ class hl970:
             self.timerstat = cmd.text
         elif cmd.header == u'brightness':
             self.set_brightness(cmd.text)
+        elif cmd.header == u'image':
+            if cmd.text:
+                glib.idle_add(self.set_logo, cmd.text, True)
+            else:
+                self.logofile = None
+                self.logo = mrlogobmp.METARACE
+                self.but_clear_clicked_cb(None)
+        elif cmd.header == u'clearall':
+            self.but_clear_clicked_cb(None)
+        elif cmd.header == u'leaderboard':
+            lvec = cmd.text.split(unichr(unt4.US))
+            glib.idle_add(self.draw_vpage, lvec)
+        elif cmd.header == u'eliminated':
+            glib.idle_add(self.draw_vnum, cmd.text)
         elif cmd.erp:	# general clearing
             self.reset()
         return False
 
+    def draw_vnum(self, num):
+        """Draw the supplied number to the top of the vpanel supersized."""
+        # based on geometry: 32x96, chars of 16x22, 1 line of 2 chars
+        if self.logo is None:
+            self.logo = bytearray(384)	# 192x16/8
+        num = num.translate(strops.INTEGER_UTRANS)
+        nl = 10
+        nh = 10
+        if len(num) > 0 and num[0].isdigit():
+            nl = int(num[0])
+        if len(num) > 1 and num[1].isdigit():
+            nh = int(num[1])
+
+        if self.vlfont is not None:
+            oft = 0 #offset down panel
+            nset(self.logo, 0, 0, self.vlfont[nl])
+            nset(self.logo, 16,0, self.vlfont[nh])
+            oft += 22
+            while oft < 96:
+                # erase remainder
+                for i in range(0,32):
+                    bset(self.logo, i, oft, 0)
+                oft += 1
+            self.but_logo_clicked_cb(None)
+        #logo2pbm(self.logo)
+        return False
+
+    def draw_vpage(self, page):
+        """Draw the supplied vertical page spec into dimg and display."""
+        # based on geometry: 32x96, chars of 6x8, lines of 5 chars
+        # NOT OPTIMISED IN ANY WAY!
+        if self.logo is None:
+            self.logo = bytearray(384)	# 192x16/8
+        linesrc = [0,0,0,1, 1,1,1,1,
+                   1,1,1,1, 1,1,1,1,
+                   1,1,1,1, 1,1,1,1,
+                   1,1,1,1, 1,0,0,0]
+        if self.vfont is not None:
+            oft = 0 #offset down panel
+            for line in page:
+                if oft > 88:
+                    break	# terminate at end of board
+                if line == u'-':
+                    #ll draw lap separator
+                    for i in range(0,32):
+                        bset(self.logo, i, oft, linesrc[i])
+                        bset(self.logo, i, oft+1, 0)
+                    oft += 2
+                else:
+                    line = strops.truncpad(
+                              line.translate(strops.PLACELIST_UTRANS),
+                              5).encode('ascii','ignore')
+                    cset(self.logo, 0, oft, self.vfont[ord(line[0])-0x20])
+                    cset(self.logo, 6, oft, self.vfont[ord(line[1])-0x20])
+                    for i in range(0,8):
+                        bset(self.logo, 12,oft+i, 0)
+                        bset(self.logo, 13,oft+i, 0)
+                    cset(self.logo, 14, oft, self.vfont[ord(line[2])-0x20])
+                    cset(self.logo, 20, oft, self.vfont[ord(line[3])-0x20])
+                    cset(self.logo, 26, oft, self.vfont[ord(line[4])-0x20])
+                    oft += 8
+            while oft < 96:
+                # erase remainder
+                for i in range(0,32):
+                    bset(self.logo, i, oft, 0)
+                oft += 1
+            self.but_logo_clicked_cb(None)
+        #logo2pbm(self.logo)
+        return False
+
+    def set_logo(self, filename, show=False):
+        self.logofile = None
+        if filename:
+            self.logofile = filename
+            bmpfilename = metarace.default_file(self.logofile)
+            self.bmp2dimg(bmpfilename)
+            if show:
+                self.but_logo_clicked_cb(None)
+        else:
+            self.logo = mrlogobmp.METARACE
+        return False
+
     def mirror_lines(self):
         """Copy line buffers back to UI."""
         self.line0.set_text(self.obuf[0]+self.obuf[1])
@@ -735,6 +976,9 @@ class hl970:
             bmpfilename = metarace.default_file(lgfile)
             self.bmp2dimg(bmpfilename)
 
+        # load vertical emulation fonts
+        self.load_vfonts()
+
         # check the maximum lap time field
         mlap = tod.str2tod(cr.get(u'hl970',u'maxlaptime'))
         if mlap is not None:
@@ -794,7 +1038,8 @@ class hl970:
         self.scrolltext.grab_focus()
         self.but_remote = b.get_object('but_remote')
         for l in [self.line0, self.line1]:
-            l.modify_font(pango.FontDescription('monospace bold 24'))
+            l.modify_font(pango.FontDescription('unifont 24'))
+            #l.modify_font(pango.FontDescription('monospace bold 24'))
         self.context = self.status.get_context_id('metarace hl970')
         b.connect_signals(self)
         self.set_remote_enable()
@@ -802,6 +1047,8 @@ class hl970:
         # run state
         self.running = True
         self.started = False
+        self.vfont = None
+        self.vlfont = None
         self.logo = mrlogobmp.METARACE
         self.logofile = None
         self.tod = tod.tod(u'now').truncate(0)
