@@ -25,29 +25,27 @@ import pango
 import threading
 import random
 import irclib
-import ConfigParser
 import time
 import os
 import sys
+import logging
 
 import metarace
 from metarace import unt4
 from metarace import tod
 from metarace import uiutil
 from metarace import strops
+from metarace import telegraph
+from metarace import jsonconfig
 
 # Global Defaults
-USCBSRV_HOST='localhost'
-USCBSRV_PORT=6667
-USCBSRV_CHANNEL='#announce'
-USCBSRV_SRVNICK='uscbsrv'
-USCBSRV_CLTNICK='rtan_'+str(random.randint(100,999))
 FONTFACE='Nimbus Sans L Bold Condensed'
 FONTSIZE=20	# font size in pixels
 MOTD=''		# Message of the day
 
 # Config filename
-CONFIGFILE='road_announce.ini'
+CONFIGFILE=u'road_announce.json'
+LOGFILE=u'road_announce.log'
 
 # Bunches colourmap
 COLOURMAP=[['#ffa0a0','red',1.0,0.1,0.1],
@@ -63,6 +61,7 @@ STARTTIME=80	# in seconds
 #MAPWIDTH=STARTTIME*TIMETICK
 MAPHMARGIN=8
 MAPVMARGIN=6
+LOGHANDLER_LEVEL=logging.DEBUG
 
 def roundedrecMoonlight(cr,x,y,w,h,radius_x=4,radius_y=4):
     """Draw a rounded rectangle."""
@@ -92,153 +91,6 @@ def roundedrecMoonlight(cr,x,y,w,h,radius_x=4,radius_y=4):
     cr.rel_curve_to (0.0, -c2, radius_x - c1, -radius_y, radius_x, -radius_y)
     cr.close_path ()
 
-class uscbio(threading.Thread):
-    def __init__(self):
-        threading.Thread.__init__(self)
-        self.running = False
-        self.cb = None
-        self.ih = irclib.IRC()
-        self.ic = self.ih.server()
-        self.np = tod.tod('now')+tod.tod('30')
-        self.hasconnnected = False
-        self.rdbuf = ''
-        self.doreconnect = False
-        self.chanstatus = False
-        self.host = USCBSRV_HOST
-        self.port = USCBSRV_PORT
-        self.channel = USCBSRV_CHANNEL
-        self.cltnick = USCBSRV_CLTNICK
-        self.srvnick = USCBSRV_SRVNICK
-        self._curpace = 0.0
-
-    def setcb(self, cb):
-        """Register the message callback function."""
-        self.cb = cb
-
-    def exit(self, msg=None):
-        """Request thread termination."""
-        self.running = False
-
-    def irc_event_cb(self, c, e):
-        """Collect and log all IRC events (debug)."""
-        print('EVENT: ' + repr(e.source()) + '/' + repr(e.eventtype()) + '/' 
-               + repr(e.target()) + ' :: ' + '/'.join(map(str, e.arguments())))
-
-    def nicknameinuse_cb(self, c, e):
-        """Handle nickname collision."""
-        self.cltnick = 'roan_'+str(random.randint(100,999))
-        self.doreconnect = True
-
-    def channel_join_cb(self, c, e):
-        """Register channel join."""
-        tg = e.target().lower()
-        if tg == self.channel:
-            self.chanstatus = True
-        
-    def channel_part_cb(self, c, e):
-        """Register channel part."""
-        tg = e.target().lower()
-        if tg == self.channel:
-            self.chanstatus = False
-
-    def unt_msg_cb(self, c, e):
-        """Handle a message packet."""
-        su = irclib.nm_to_n(e.source()).lower()
-        tg = e.target().lower()
-        if su == self.srvnick and tg == self.channel:
-            self._pacing(0.0)
-            # Have a 'broadcast' packet ... append and then search
-            self.rdbuf += unt4.decode(''.join(e.arguments()))
-            idx = self.rdbuf.find(chr(unt4.EOT))
-            if idx >= 0:
-                msgtxt = self.rdbuf[0:idx+1]
-                self.rdbuf = self.rdbuf[idx+1:]
-                if self.cb is not None:
-                    glib.idle_add(self.cb, unt4.unt4(unt4str=msgtxt))
-
-    def set_port(self, host=None, port=None, channel=None,
-                       cltnick=None, srvnick=None):
-        """Request change in irc connection."""
-        reconnect = False
-        if host is not None and host != self.host:
-            self.host = host
-            reconnect = True
-        if port is not None and port != self.port:
-            self.port = port
-            reconnect = True
-        if cltnick is not None and cltnick != self.port:
-            self.cltnick = cltnick
-            reconnect = True        
-        if channel is not None and channel != self.channel:
-            self.channel = channel
-            reconnect = True
-        if srvnick is not None:
-            self.srvnick = srvnick.lower()
-        if reconnect:
-            self.doreconnect = True
-
-    def _reconnect(self):
-        self.ic.connect(self.host, self.port, self.cltnick)
-
-    def in_channel(self):
-        return self.chanstatus
-
-    def _rejoin(self):
-        self.ic.join(self.channel)
-
-    def _pacing(self, delay=None):
-        """Adjust internal pacing delay."""
-        if delay is None:
-            self._curpace += 0.1
-            if self._curpace > 1.5:
-                self._curpace = 1.5
-        else:
-            self._curpace = delay
-        return self._curpace
-
-    def run(self):
-        """Called via threading.Thread.start()."""
-        self.running = True
-        self.ic.add_global_handler('pubmsg', self.unt_msg_cb, -10)
-        self.ic.add_global_handler('pubnotice', self.unt_msg_cb, -10)
-        self.ic.add_global_handler('join', self.channel_join_cb, -10)
-        self.ic.add_global_handler('part', self.channel_part_cb, -10)
-        self.ic.add_global_handler('kick', self.channel_part_cb, -10)
-        self.ic.add_global_handler('nicknameinuse', self.nicknameinuse_cb, -10)
-        #self.ic.add_global_handler('all_events', self.irc_event_cb, 0)
-        while self.running:
-            try:
-                self.ih.process_once(0)
-                if not self.ic.is_connected() or self.doreconnect:
-                    self.doreconnect = False
-                    self.chanstatus = False
-                    time.sleep(2)
-                    self._reconnect()
-                    # skip over preliminary connect chatter ready to 
-                    # poll channel status - avoids double channel join
-                    if self.hasconnnected:
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(1)
-                    else:
-                        self.hasconnnected = True
-                if not self.in_channel():
-                    time.sleep(2)
-                    self._rejoin()
-                now = tod.tod('now')
-                if now > self.np:
-                    self.ic.ctcp('PING', self.cltnick, str(int(time.time())))
-                    self.np = now + tod.tod('30')
-                time.sleep(self._pacing())
-            except Exception as e:
-                # TODO : FIX HERE?
-                print ('Exception from uscbio: ' + str(e))
-
 class rr_announce(object):
     """Road race announcer application."""
  
@@ -256,16 +108,28 @@ class rr_announce(object):
 
     def shutdown(self):
         """Cleanly shutdown."""
-        self.io.exit()
-        self.io.join()
         self.started = False
+        self.hide()
+        self.io.exit()
+        print(u'Waiting for remote to exit...')
+        self.io.join()
+
+    def app_destroy_handler(self):
+        if self.started:
+            #self.saveconfig()
+            self.shutdown()     # threads are joined in shutdown
+        # close event and remove main log handler
+        if self.loghandler is not None:
+            self.log.removeHandler(self.loghandler)
+        self.running = False
+        # flag quit in main loop
+        gtk.main_quit()
+        return False
 
     def window_destroy_cb(self, window):
         """Handle destroy signal."""
-        if self.started:
-            self.shutdown()
-        self.running = False
-        gtk.main_quit()
+        self.hide()
+        glib.idle_add(self.app_destroy_handler)
     
     #def map_area_expose_event_cb(self, widget, event):
         #"""Update desired portion of drawing area."""
@@ -370,6 +234,7 @@ class rr_announce(object):
         self.lbl_header.set_text(self.motd)
         self.elap_lbl.set_text('')
         self.riders.clear()
+        self.inters.clear()
         #self.map_redraw()		# update src map
         #self.map_area.queue_draw()	# queue copy to screen
         
@@ -390,6 +255,30 @@ class rr_announce(object):
                 #self.fin_time.set_text('')
                 #self.fin_box.hide()
 
+    def inter_line(self, msg):
+        sr = msg.split(chr(unt4.US))
+        if len(sr) == 5:
+            if sr[1] != '':	# got a rider
+                et = tod.str2tod(sr[4])
+                ets = '' 
+                if et is not None:
+                    if '.' in sr[4]:
+                        ets = et.rawtime(2)
+                    else:
+                        ets = et.rawtime(0) + '   '
+                    self.int_time.set_text(ets)
+                rs = sr[0]
+                if rs != '' and rs.isdigit():
+                    rs += '.'
+                self.int_rank.set_text(rs)
+                self.int_rider.set_text(sr[1] + ' ' + sr[2] + ' ' + sr[3])
+                #self.int_box.show()
+            else:		# clearit
+                self.int_rank.set_text('')
+                self.int_rider.set_text('')
+                self.int_time.set_text('')
+                #self.int_box.hide()
+
     def finish_line(self, msg):
         sr = msg.split(chr(unt4.US))
         if len(sr) == 5:
@@ -441,13 +330,38 @@ class rr_announce(object):
             #self.map_redraw()		# update src map
             #self.map_area.queue_draw()	# queue copy to screen
 
-    def msg_cb(self, m):
+    def append_arrival(self, msg):
+        sr = msg.split(chr(unt4.US))
+        if len(sr) == 6:
+            rs = sr[0]
+            rftime = tod.str2tod(sr[4])
+            if rftime is not None:
+                    self.last_time = rftime
+                    nr=[rs,sr[1],sr[2],sr[3],rftime.rawtime(2),
+                        sr[5],None]
+            else: 
+                # Informative non-timeline record
+                nr=[sr[0],sr[1],sr[2],sr[3],
+                        '', '', None]
+                
+            self.inters.append(nr)
+            #self.map_redraw()		# update src map
+            #self.map_area.queue_draw()	# queue copy to screen
+
+    def msg_cb(self, m, nick, chan):
         """Handle message packet in main thread."""
         redraw = False
+        if self.remoteuser and self.remoteuser.lower() != nick.lower():
+            #self.log.debug(u'Ignoring command from ' + repr(nick))
+            return False
         if m.header == 'rider':
             self.append_rider(m.text)
+        if m.header == 'arrivalrow':
+            self.append_arrival(m.text)
         elif m.header == 'finish':
             self.finish_line(m.text)
+        elif m.header == 'ttsplit':
+            self.inter_line(m.text)
         elif m.header == 'time':
             self.elap_lbl.set_text(m.text)
         elif m.header == 'title':
@@ -460,53 +374,94 @@ class rr_announce(object):
         
     def loadconfig(self):
         """Load config from disk."""
-        cr = ConfigParser.ConfigParser({'host':USCBSRV_HOST,
-                                        'port':str(USCBSRV_PORT),
-                                        'channel':USCBSRV_CHANNEL,
-                                        'srvnick':USCBSRV_SRVNICK,
-                                        'cltnick':USCBSRV_CLTNICK,
-                                        'fontsize':str(FONTSIZE),
-                                        'fullscreen':'Yes',
-                                        'motd':MOTD})
-        cr.add_section('uscbsrv')
-        cr.add_section('announce')
+        cr = jsonconfig.config({u'road_announce':{
+                                    u'remoteport':u'',
+                                    u'remoteuser':u'',
+                                    u'remotechan':u'#agora',
+                                    u'autoscroll':False,
+                                    u'timetick':12,
+                                    u'fontsize':20,
+                                    u'fullscreen':False,
+                                    u'groupcol':True,
+                                    u'catcol':True,
+                                    u'bunchmap':True,
+                                    u'search':False,
+                                    u'maxlaptime':'2:00',
+                                    u'motd':MOTD}})
+        cr.add_section(u'road_announce')
+        cwfilename = metarace.default_file(CONFIGFILE)
+        # read in sysdefaults before checking for config file
+        cr.merge(metarace.sysconf, u'road_announce')
+
+        # re-set log file
+        if self.loghandler is not None:
+            self.log.removeHandler(self.loghandler)
+            self.loghandler.close()
+            self.loghandler = None
+        self.loghandler = logging.FileHandler(
+                             os.path.join(self.configpath, LOGFILE))
+        self.loghandler.setLevel(LOGHANDLER_LEVEL)
+        self.loghandler.setFormatter(logging.Formatter(
+                       '%(asctime)s %(levelname)s:%(name)s: %(message)s'))
+        self.log.addHandler(self.loghandler)
+
         # check for config file
         try:
-            a = len(cr.read(CONFIGFILE))
-            if a == 0:
-                print('No config file found - loading default values.')
-        except e:
-            print('Error reading config: ' + str(e))
+            with open(cwfilename, 'rb') as f:
+                cr.read(f)
+        except Exception as e:
+            self.log.error(u'Reading app config: ' + repr(e))
+
+        self.fontsize = strops.confopt_posint(cr.get(u'road_announce',
+                                              u'fontsize'), FONTSIZE)
 
-        self.fontsize = strops.confopt_posint(cr.get('announce', 'fontsize'),
-                                              FONTSIZE)
         fnszstr = str(self.fontsize)+'px'
         self.font = pango.FontDescription(FONTFACE + ' ' + fnszstr)
-        self.motd = cr.get('announce', 'motd')
-        if strops.confopt_bool(cr.get('announce', 'fullscreen')):
+
+        self.motd = cr.get(u'road_announce', u'motd')
+        if strops.confopt_bool(cr.get(u'road_announce', u'fullscreen')):
             self.window.fullscreen()
 
-        nhost = cr.get('uscbsrv', 'host')
-        nport = strops.confopt_posint(cr.get('uscbsrv', 'port'),
-                                      USCBSRV_PORT)
-        nchannel = cr.get('uscbsrv', 'channel')
-        ncltnick =cr.get('uscbsrv', 'cltnick')
-        nsrvnick = cr.get('uscbsrv', 'srvnick')
-        self.io.set_port(nhost, nport, nchannel, ncltnick, nsrvnick)
+        self.remoteport = cr.get(u'road_announce', u'remoteport')
+        self.remotechan = cr.get(u'road_announce', u'remotechan')
+        self.remoteuser = cr.get(u'road_announce', u'remoteuser')
+        self.io.set_portstr(portstr=self.remoteport,
+                             channel=self.remotechan)
+        if self.remoteuser:
+            self.log.info(u'Enabled remote control by: '
+                          + repr(self.remoteuser))
+        else:
+            self.log.info(u'Promiscuous remote control enabled.')
 
         self.lbl_header.modify_font(self.font)
         self.elap_lbl.modify_font(self.font)
         self.search_lbl.modify_font(self.font)
         self.search_entry.modify_font(self.font)
         self.view.modify_font(self.font)
+        self.iview.modify_font(self.font)
         self.fin_rank.modify_font(self.font)
         self.fin_label.modify_font(self.font)
         self.fin_rider.modify_font(self.font)
         self.fin_time.modify_font(self.font)
+        self.int_rank.modify_font(self.font)
+        self.int_label.modify_font(self.font)
+        self.int_rider.modify_font(self.font)
+        self.int_time.modify_font(self.font)
 
-    def __init__(self):
-        self.io = uscbio()
-        self.io.setcb(self.msg_cb)
+    def __init__(self, configpath=None):
+        # logger and log handler
+        self.log = logging.getLogger()
+        self.log.setLevel(logging.DEBUG)
+        self.loghandler = None  # set in loadconfig to meet dir
+
+        # meet configuration path and options
+        if configpath is None:
+            configpath = u'.'   # None assumes 'current dir'
+        self.configpath = configpath
+        self.loglevel = logging.INFO    # UI log window
+
+        self.io = telegraph.telegraph()
+        self.io.set_pub_cb(self.msg_cb)
         self.started = False
         self.running = True
 
@@ -535,6 +490,16 @@ class rr_announce(object):
         self.fin_time = b.get_object('fin_time_lbl')
         self.fin_time.modify_font(self.font)
 
+        b.get_object('int_pfx_lbl').modify_font(self.font)
+        self.int_label = b.get_object('int_pfx_lbl')
+        self.int_label.modify_font(self.font)
+        self.int_rank = b.get_object('int_rank_lbl')
+        self.int_rank.modify_font(self.font)
+        self.int_rider = b.get_object('int_rider_lbl')
+        self.int_rider.modify_font(self.font)
+        self.int_time = b.get_object('int_time_lbl')
+        self.int_time.modify_font(self.font)
+
         #self.fin_box = b.get_object('fin_box')
         #self.fin_box.hide()
 
@@ -560,10 +525,17 @@ class rr_announce(object):
                                     gobject.TYPE_STRING,  # downtime
                                     gobject.TYPE_PYOBJECT) # rftod
 
+        self.inters = gtk.ListStore(gobject.TYPE_STRING,  # rank
+                                    gobject.TYPE_STRING,  # no.
+                                    gobject.TYPE_STRING,  # namestr
+                                    gobject.TYPE_STRING,  # cat/com
+                                    gobject.TYPE_STRING,  # timestr
+                                    gobject.TYPE_STRING,  # downtime
+                                    gobject.TYPE_PYOBJECT) # rftod
         t = gtk.TreeView(self.riders)
         self.view = t
         t.set_reorderable(False)
-        t.set_rules_hint(False)
+        t.set_rules_hint(True)
         t.set_headers_visible(False)
         self.search_lbl = b.get_object('search_lbl')
         self.search_lbl.modify_font(self.font)
@@ -580,9 +552,24 @@ class rr_announce(object):
         uiutil.mkviewcoltxt(t, 'Gap', 5,calign=1.0,width=90)
         # "down" time?
         #uiutil.mkviewcoltxt(t, 'Bunch', 5,width=50,bgcol=6,calign=0.5)
+        b.get_object('text_scroll').add(t)
         t.show()
         t.grab_focus()
-        b.get_object('text_scroll').add(t)
+
+        t = gtk.TreeView(self.inters)
+        self.iview = t
+        t.set_reorderable(False)
+        t.set_rules_hint(True)
+        t.set_headers_visible(False)
+        t.modify_font(self.font)
+        uiutil.mkviewcoltxt(t, '', 0,width=40)
+        uiutil.mkviewcoltxt(t, 'No.', 1,calign=1.0,width=40)
+        uiutil.mkviewcoltxt(t, 'Rider', 2,expand=True,fixed=True)
+        uiutil.mkviewcoltxt(t, 'Rank', 3,calign=0.0)
+        uiutil.mkviewcoltxt(t, 'Time', 4,calign=1.0,width=100)
+        uiutil.mkviewcoltxt(t, 'Avg', 5,calign=1.0,width=90)
+        t.show()
+        b.get_object('arrival_scroll').add(t)
         b.connect_signals(self)
 
 def main():
