@@ -50,7 +50,7 @@ import sys
 import metarace
 from metarace import tod
 from metarace import timy
-from metarace import uscbsrv
+from metarace import telegraph
 from metarace import strops
 from metarace import ucsv
 from metarace import jsonconfig
@@ -64,7 +64,8 @@ LOGFILE=u'irtt_start.log'
 TIMEFONT='Nimbus Sans L Condensed Bold '
 COUNTERFONT='Nimbus Sans L Condensed Bold '
 NOHDR = [u'Start', u'start', u'time', u'Time', u'']
-AUDIOSYNCTHRESH = 50000000L	# Tolerate no more than 50ms audio desync
+DEFAUDIOSYNCTHRESH = 100000000L	# Tolerate no more than 100ms audio desync
+				# note: on Pi/ARM, 200ms reqd due to proc lag
 
 # draw text centred
 def text_cent(cr, pr, oh, ow, msg, ch, fn):
@@ -220,6 +221,9 @@ class irtt_start(object):
         cr.add_section(u'irtt_start')
         # check for config file
         cfile = metarace.default_file(CONFIGFILE)
+        # read in sysdefaults before checking for config file
+        cr.merge(metarace.sysconf, u'irtt_start')
+
         try:
             self.log.debug(u'Reading config from: ' +repr(cfile))
             with open(cfile, 'rb') as f:
@@ -429,7 +433,7 @@ class irtt_start(object):
                 self.bulb = 'green'
                 self.countdown = 0
                 self.log.debug(u'GO: ' + self.riderlogstr(self.currider))
-            if cdn == -5:	# load sets minimum gap-> ~25sec
+            if cdn == -4:	# load sets minimum gap-> ~25sec
                 self.remote_msg(u'CLEAR: ' + self.riderlogstr(self.currider))
                 self.clear()	# also does dearm
                 self.currider = self.ridermap[self.currider][4]
@@ -470,7 +474,7 @@ class irtt_start(object):
 
         # require one timy and one uscbsrv
         self.timer = timy.timy()
-        self.scb = uscbsrv.uscbsrv()
+        self.scb = telegraph.telegraph()
 
         self.started = False
         self.running = True
