@@ -19,11 +19,11 @@
 
 This module provides a class 'ittt' which implements the 'race'
 interface and manages data, timing and scoreboard for generic
-individual track time trial events:
+individual track time trial like events:
 
  - TT
- - Flying 200m/lap
- - Pursuit (Qualify)
+ - Pursuit
+ - Team Sprint
 
 """
 
@@ -49,7 +49,7 @@ from metarace import timerpane
 from metarace import printing
 
 # config version string
-EVENT_ID = 'tracktimetrial-1.3'
+EVENT_ID = 'tracktimetrial-1.4'
 
 # startlist model columns
 COL_BIB = 0
@@ -63,17 +63,6 @@ COL_START = 7
 COL_FINISH = 8
 COL_SPLITS = 9
 
-# scb consts
-SCB_NAME_WIDTH = metarace.SCB_LINELEN - 9
-SCB_RESNAME_WIDTH = metarace.SCB_LINELEN - 11
-SCB_RESULT_FMT = [(3, u'l'), (3, u'r'), u' ',
-                  (SCB_RESNAME_WIDTH, u'l'), u' ', (3, u'r')]
-
-# team name specialisations
-SCB_TEAMNAME_WIDTH = metarace.SCB_LINELEN
-SCB_RESTEAM_WIDTH = metarace.SCB_LINELEN - 4
-SCB_TEAM_FMT = [(4, u'l'), (SCB_RESTEAM_WIDTH,u'l')]
-
 # scb function key mappings
 key_reannounce = 'F4'		     # (+CTRL) calls into delayed announce
 key_startlist = 'F6'                 # re-display running time (startlist)
@@ -160,30 +149,44 @@ class ittt(object):
         self.meet.scbwin = None
         self.timerwin = False     # TODO: bib width enhancement
         fmtplaces = []
+        name_w = self.meet.scb.linelen - 12
+        rcount = 0
+        pcount = 0
         for r in self.riders:
+            rcount += 1
             if r[COL_PLACE] is not None and r[COL_PLACE] != '':
+                pcount += 1	#"placed"
                 plstr = r[COL_PLACE].decode('utf-8')
                 if plstr.isdigit():
-                    if int(plstr) > 20:
-                        break
+                    #if int(plstr) > 20:
+                        #break
                     plstr = plstr + u'.'
                 if not self.teamnames:
+                    name_w = self.meet.scb.linelen - 12
+                    name = strops.fitname(r[COL_FIRSTNAME].decode('utf-8'),
+                                          r[COL_LASTNAME].decode('utf-8'),
+                                          name_w, trunc=True)
+                    club = r[COL_CLUB].decode('utf-8')
                     fmtplaces.append([plstr,
                          r[COL_BIB].decode('utf-8'),
-                         strops.fitname(r[COL_FIRSTNAME].decode('utf-8'),
-                         r[COL_LASTNAME].decode('utf-8'), SCB_RESNAME_WIDTH),
-                         r[COL_CLUB].decode('utf-8')])
+                         name,
+                         strops.truncpad(club, 4,'r')])
                 else:
-                    fmtplaces.append([plstr,
-                         strops.truncpad(r[COL_FIRSTNAME].decode('utf-8'),
-                                         SCB_RESTEAM_WIDTH, 'l')])
-        evtstatus = u'STANDINGS'	# TODO: detect result/virtual/prov
-        fmt = SCB_RESULT_FMT
+                    name = r[COL_FIRSTNAME].decode('utf-8')
+                    name_w = self.meet.scb.linelen - 8
+                    club = r[COL_CLUB].decode('utf-8')
+                    fmtplaces.append([plstr, name, club])
+        evtstatus = u'Standings'
+        if rcount > 0 and pcount == rcount:
+            evtstatus = u'Result'
+
+        fmt = [(3,u'l'), (4,u'r'), u' ',(name_w, u'l'), (4,u'r')]
         if self.teamnames:
-            fmt = SCB_TEAM_FMT
-        self.meet.scbwin = scbwin.scbtable(self.meet.scb,
-                       self.meet.racenamecat(self.event, tail=evtstatus),
-                       fmt, fmtplaces, delay=50)
+            fmt = [(3,u'l'), u' ',(name_w, u'l'), (4,u'r')]
+        self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb,
+                              head=self.meet.racenamecat(self.event),
+                              subhead=evtstatus.upper(),
+                              coldesc=fmt, rows=fmtplaces)
         self.meet.scbwin.reset()
 
     def todstr(self, col, cr, model, iter, data=None):
@@ -269,6 +272,10 @@ class ittt(object):
         self.set_timetype(cr.get('event', 'timetype'))
         self.distance = strops.confopt_dist(cr.get('event', 'distance'))
         self.units = strops.confopt_distunits(cr.get('event', 'distunits'))
+        if self.event[u'laps']:
+            self.units = 'laps'
+            self.distance = strops.confopt_posint(self.event[u'laps'],
+                                                  self.distance)
         self.chan_S = strops.confopt_chan(cr.get('event', 'chan_S'), defchans)
         self.chan_A = strops.confopt_chan(cr.get('event', 'chan_A'), defchana)
         self.chan_B = strops.confopt_chan(cr.get('event', 'chan_B'), defchanb)
@@ -279,7 +286,7 @@ class ittt(object):
         self.inomnium = strops.confopt_bool(cr.get('event', 'inomnium'))
         if self.inomnium:
             # SWITCH: Event is part of an omnium, make any req'd overrides
-            self.seedsrc = 3	# read seeding from omnium points standinds
+            self.seedsrc = 1	# read seeding from omnium points standinds
 
         # re-load starters/results
         self.onestart = False
@@ -353,7 +360,7 @@ class ittt(object):
         if eid and eid != EVENT_ID:
             self.log.error(u'Event configuration mismatch: '
                            + repr(eid) + u' != ' + repr(EVENT_ID))
-            self.readonly = True
+            #self.readonly = True
 
     def saveconfig(self):
         """Save race to disk."""
@@ -428,8 +435,11 @@ class ittt(object):
         if substr:
             sec.subheading = substr
         if self.event[u'reco']:
-            sec.recordstr = self.event[u'reco']
-        sec.lines = self.get_heats()
+            sec.footer = self.event[u'reco']
+        if self.event[u'plac']: 
+            sec.lines = self.get_heats(placeholders = self.event[u'plac'])
+        else:
+            sec.lines = self.get_heats()
         ret.append(sec)
         return ret
 
@@ -483,14 +493,17 @@ class ittt(object):
             blanknames = True
         if self.timetype == 'single':
             for r in self.riders:
-                rno = r[COL_BIB]
+                rno = r[COL_BIB].decode('utf-8')
                 if u't' in self.series:	# Team no hack
                     rno = u' '	# force name
-                rname = strops.resname(r[COL_FIRSTNAME], r[COL_LASTNAME],
-                                       r[COL_CLUB])
                 info = None
                 if cats and rno in cats:
                     info = cats[rno]
+                rh = self.meet.newgetrider(rno, self.series)
+                rname = u''
+                if rh is not None:
+                     rname = rh[u'namestr']
+                     if rh[u'ucicode']: info = rh[u'ucicode']
                 hlist.append([unicode(count)+'.1', rno, rname, info])
                                  # all heats are one up
                 count -= 1
@@ -498,15 +511,18 @@ class ittt(object):
             hno = int(math.ceil(0.5*count))
             lane = 1
             for r in self.riders:
-                rno = r[COL_BIB]
+                rno = r[COL_BIB].decode('utf-8')
+                rh = self.meet.newgetrider(rno, self.series)
+                rname = u''
                 if u't' in self.series:	# Team no hack
                     rno = u' '	# force name
-                rname = strops.resname(r[COL_FIRSTNAME], r[COL_LASTNAME],
-                                   r[COL_CLUB])
                 heat = unicode(hno) + u'.' + unicode(lane)
                 info = None
                 if cats and rno in cats:
                     info = cats[rno]
+                if rh is not None:
+                    rname = rh[u'namestr']
+                    if rh[u'ucicode']: info = rh[u'ucicode']
                 hlist.append([heat, rno, rname, info])
                 lane += 1
                 if lane > 2:
@@ -528,7 +544,7 @@ class ittt(object):
             heat = str(h)
             if self.difftime:	# override heat if 'final'
                 heat = u'-'
-            if blanknames:
+            if blanknames and len(r[1]) > 3:	# HACK for miss
                 r[1] = u''
                 r[2] = u''
                 r[3] = None
@@ -717,7 +733,7 @@ class ittt(object):
             ns = se.get_text()
             if ns != self.series:
                 self.series = ns
-                self.meet.edb.editevent(self.event, series=ns)
+                self.event[u'seri'] = ns
 
             # xfer starters if not empty
             slist = strops.reformat_riderlist(
@@ -756,7 +772,7 @@ class ittt(object):
 
             yield [bib, rank, time, info]
 
-    def result_report(self):
+    def result_report(self, recurse=False):
         """Return a list of printing sections containing the race result."""
         self.placexfer()
         ret = []
@@ -764,30 +780,32 @@ class ittt(object):
         sec.heading = u'Event ' + self.evno + u': ' + u' '.join([
                        self.event[u'pref'], self.event[u'info']
                               ]).strip()
-        lapstring = strops.lapstring(self.event[u'laps'])
-        substr = u' '.join([lapstring, self.event[u'dist'],
-                             self.event[u'prog']]).strip()
-        if substr:
-            sec.subheading = substr
         sec.lines = []
         ftime = None
+        rcount = 0
+        pcount = 0
         for r in self.riders:
-            rno = r[COL_BIB]
+            rcount += 1
+            rno = r[COL_BIB].decode('utf-8')
+            rh = self.meet.newgetrider(rno, self.series)
             rank = None
             rname = u''
             if not self.teamnames:
-                rname = strops.resname(r[COL_FIRSTNAME].decode('utf-8'),
-                                       r[COL_LASTNAME].decode('utf-8'),
-                                       r[COL_CLUB].decode('utf-8'))
+                if rh is not None:
+                    rname = rh[u'namestr']
             else:	# Team no hack
                 rno = u' '	# force name
-                rname = r[COL_FIRSTNAME].decode('utf-8')
+                if rh is not None:
+                    rname = rh[u'first']
             rtime = None
             rcat = None
-            if self.showcats:
-                dbr = self.meet.rdb.getrider(rno, self.series)
-                if dbr is not None:
-                    rcat = self.meet.rdb.getvalue(dbr, riderdb.COL_CAT)
+            if self.event[u'cate']:
+                if rh is not None:
+                    if rh[u'cat']:
+                        rcat = rh[u'cat']
+            if rh is not None:
+                if rh[u'ucicode']:
+                    rcat = rh[u'ucicode']   # overwrite by force
             info = None
             dtime = None
             if self.onestart:
@@ -796,6 +814,7 @@ class ittt(object):
                         rank = r[COL_PLACE] + '.'
                     else:
                         rank = r[COL_PLACE]
+                    pcount += 1
                 if r[COL_FINISH] is not None:
                     time = (r[COL_FINISH]-r[COL_START]).truncate(3)
                     if ftime is None: 
@@ -810,6 +829,22 @@ class ittt(object):
                     rtime = str(r[COL_COMMENT])
 
             sec.lines.append([rank, rno, rname, rcat, rtime, dtime])
+        
+        lapstring = strops.lapstring(self.event[u'laps'])
+        substr = u' '.join([lapstring, self.event[u'dist'],
+                             self.event[u'prog']]).strip()
+
+        ## Should be a method 
+        if self.onestart:
+            sec.subheading = substr
+            if rcount > 0 and pcount < rcount:
+                sec.subheading += u' - STANDINGS'
+            else:
+                sec.subheading += u' - Result'
+        else:
+            if substr:
+                sec.subheading = substr
+
         ret.append(sec)
         return ret
 
@@ -883,9 +918,9 @@ class ittt(object):
             place = self.riders.get_value(ri, COL_PLACE)
             if sp is self.fs:
                 elap = t - self.curstart
-                self.meet.gemini.rtick(elap)	## DANGER -> TEST
                 self.meet.scbwin.setr1('(' + place + ')')
                 self.meet.scbwin.sett1(self.fs.ck.get_text())
+                self.meet.gemini.set_time(self.fs.ck.get_text(),0)
                 if self.timetype == 'single': # Speed/TTB is hack mode
                     dist = self.meet.get_distance(self.distance, self.units)
                     if dist is not None:
@@ -900,15 +935,24 @@ class ittt(object):
             else:
                 self.meet.scbwin.setr2('(' + place + ')')
                 self.meet.scbwin.sett2(self.bs.ck.get_text())
+                self.meet.gemini.set_time(self.bs.ck.get_text(),1)
+            self.meet.gemini.show_dual()
         # call for a delayed announce...
         glib.idle_add(self.delayed_announce)
         # AND THEN, if other lane not armed, export result
         if self.t_other(sp).getstatus() != 'armfin':
             self.meet.delayed_export()
 
+    def rftimercb(self, e):
+        """Handle rftimer event."""
+        if e.refid == '':       # got a trigger
+            #return self.starttrig(e)
+            return False
+        return False
+
     def timercb(self, e):
         """Handle a timer event."""
-        chan = timy.chan2id(e.chan)
+        chan = strops.chan2id(e.chan)
         if self.timerstat == 'armstart':
             if chan == self.chan_S:
                 self.torunning(e)
@@ -933,15 +977,18 @@ class ittt(object):
             return False
         now = tod.tod('now')
         if self.fs.status in ['running', 'armint', 'armfin']:
-            elap = now - self.lstart
-            self.fs.runtime(elap)
+            self.fs.runtime(now - self.lstart)
             if self.timerwin and type(self.meet.scbwin) is scbwin.scbtt:
-                self.meet.scbwin.sett1(self.fs.ck.get_text())
-                self.meet.gemini.rtick(elap, 1)
+                elapstr = self.fs.ck.get_text()
+                self.meet.scbwin.sett1(elapstr)
+                self.meet.gemini.set_time(elapstr[0:12],lane=0)
         if self.bs.status in ['running', 'armint', 'armfin']:
             self.bs.runtime(now - self.lstart)
             if self.timerwin and type(self.meet.scbwin) is scbwin.scbtt:
-                self.meet.scbwin.sett2(self.bs.ck.get_text())
+                elapstr = self.bs.ck.get_text()
+                self.meet.scbwin.sett2(elapstr)
+                self.meet.gemini.set_time(elapstr[0:12],lane=1)
+        self.meet.gemini.show_dual()
         return True
 
     def show_200_ttb(self, scb):
@@ -1006,7 +1053,8 @@ class ittt(object):
                     nr[i] = self.meet.rdb.getvalue(dbr, i)
             return self.riders.append(nr)
         else:		# patch 'seed' on an existing record
-            ri[COL_SEED] = istr
+            if not ri[COL_SEED] and istr:
+                ri[COL_SEED] = istr
             return True		# not sure about this one?
 
     def editcol_db(self, cell, path, new_text, col):
@@ -1029,6 +1077,10 @@ class ittt(object):
                 if t > tod.FAKETIMES['max']:
                     if t == tod.FAKETIMES['dsq']:
                         place = 'dsq'
+                    elif t == tod.FAKETIMES['dns']:
+                        place = 'dns'
+                    elif t == tod.FAKETIMES['dnf']:
+                        place = 'dnf'
                     #else:
                         #place = 'dnf' # leave to naturally sort
             i = self.getiter(t.refid)
@@ -1112,7 +1164,7 @@ class ittt(object):
     def armlap(self, sp, cid):
         """Arm timer for a lap split."""
         if self.timerstat == 'running':
-            if sp.getstatus() == 'running':
+            if sp.getstatus() in ['caught', 'running']:
                 sp.toarmint()
                 self.meet.timer.arm(cid)
             elif sp.getstatus() == 'armint':
@@ -1128,7 +1180,7 @@ class ittt(object):
         
     def abortrider(self, sp):
         """Abort the selected lane."""
-        if sp.getstatus() not in ['idle', 'finish']:
+        if sp.getstatus() not in ['idle', 'caught', 'finish']:
             bib = sp.getrider()
             ri = self.getiter(bib)
             if ri is not None:
@@ -1152,13 +1204,15 @@ class ittt(object):
                 if ri is not None:
                     self.settimes(ri, st=self.curstart,
                                       splits=op.splits, comment='caught')
-                op.tofinish()
+                op.tofinish('caught')
                 self.meet.timer_log_msg(bib, '- Caught -')
                 if self.timerwin and type(self.meet.scbwin) is scbwin.scbtt:
                     if op is self.fs:
                         self.meet.scbwin.sett1(' [caught]     ')
+                        self.meet.gemini.set_time(u'    -:--.-  ',0)
                     else:
                         self.meet.scbwin.sett2(' [caught]     ')
+                        self.meet.gemini.set_time(u'    -:--.-  ',1)
             if sp.getstatus() not in ['idle', 'finish']:
                 bib = sp.getrider()
                 ri = self.getiter(bib)
@@ -1174,14 +1228,14 @@ class ittt(object):
     def falsestart(self):
         """Register false start."""
         if self.timerstat == 'running':
-            if self.fs.getstatus() not in ['idle', 'finish']:
+            if self.fs.getstatus() not in ['idle', 'caught', 'finish']:
                 self.fs.toload()
                 self.meet.timer_log_msg(self.fs.getrider(),
                                         '- False start -')
                 if self.timerwin and type(self.meet.scbwin) is scbwin.scbtt:
                     self.meet.scbwin.setr1('False')
                     self.meet.scbwin.sett1('Start')
-            if self.bs.getstatus() not in ['idle', 'finish']:
+            if self.bs.getstatus() not in ['idle', 'caught', 'finish']:
                 self.bs.toload()
                 self.meet.timer_log_msg(self.bs.getrider(),
                                         '- False start -')
@@ -1199,7 +1253,7 @@ class ittt(object):
     def armfinish(self, sp, cid):
         """Arm timer for finish trigger."""
         if self.timerstat == 'running':
-            if sp.getstatus() in ['running', 'finish']:
+            if sp.getstatus() in ['running', 'caught', 'finish']:
                 if sp.getstatus() == 'finish':
                      self.meet.timer_log_msg(sp.getrider(),
                                              '- False finish -')
@@ -1228,20 +1282,24 @@ class ittt(object):
             ret = name
             if r is not None and r[COL_BIB] != '':
                 if self.teamnames:
-                    ret = strops.truncpad(r[COL_FIRSTNAME].decode('utf-8'),
-                                           SCB_TEAMNAME_WIDTH, 'l')
-                    tp.namevec = [bib, ret, '']
+                    club = r[COL_CLUB].decode('utf-8')
+                    name = r[COL_FIRSTNAME].decode('utf-8')
+                    name_w = self.meet.scb.linelen-5	# w=4 + space
+                    ret = ' '.join([strops.truncpad(name, name_w, 'l'),
+                                    strops.truncpad(club, 4, 'r')])
+                    tp.namevec = [bib, ret, club]
                 else:
+                    name_w = self.meet.scb.linelen - 9
                     first = r[COL_FIRSTNAME].decode('utf-8')
                     last = r[COL_LASTNAME].decode('utf-8')
                     club = r[COL_CLUB].decode('utf-8')
                     name = strops.fitname(first,
                                           last,
-                                          SCB_NAME_WIDTH)
+                                          name_w)
                     tp.namevec = [bib, strops.resname(first, last, club), '']
                     ret = ' '.join([strops.truncpad(r[COL_BIB], 3, 'r'),
-                                 strops.truncpad(name, SCB_NAME_WIDTH),
-                                 strops.truncpad(club, 3, 'r')])
+                                 strops.truncpad(name, name_w),
+                                 strops.truncpad(club, 4, 'r')])
             return ret
         else:
             tp.namevec = None
@@ -1260,6 +1318,8 @@ class ittt(object):
         self.meet.announce.gfx_overlay(2)
         self.meet.announce.gfx_set_title(self.event[u'pref'] + u' '
                             + self.event[u'info'])
+        self.meet.gemini.set_bib(self.fs.getrider(),0)
+        self.meet.gemini.set_bib(self.bs.getrider(),1)
         if self.fs.namevec is not None:
             self.meet.announce.gfx_add_row(self.fs.namevec)
         if self.bs.namevec is not None:
@@ -1282,9 +1342,10 @@ class ittt(object):
             self.lstart = None
             self.meet.timer.arm(self.chan_S)
             self.showtimerwin()
+            self.meet.printimp(True)
             if not self.onestart:
                 self.meet.timer.printline(self.meet.racenamecat(self.event))
-            self.meet.printimp(True)
+            self.meet.delayimp('0.01')
             if self.fs.status == 'armstart':
                 self.meet.scbwin.sett1('       0.0     ')
                 nstr = self.fs.biblbl.get_text()
@@ -1305,6 +1366,7 @@ class ittt(object):
             self.bs.toidle()
         self.timerstat = 'idle'
         self.meet.printimp(False)
+        self.meet.delayimp('1.00')
         self.curstart = None
         self.lstart = None
         self.diffstart = None
@@ -1471,7 +1533,7 @@ class ittt(object):
             self.bs.frame.hide()
             self.bs.hide_laps()
             self.fs.frame.set_label('Timer')
-            self.fs.hide_laps()
+            self.fs.show_laps()
         else:
             self.bs.frame.show()
             self.bs.show_laps()
@@ -1570,8 +1632,11 @@ class ittt(object):
         # Timer Panes
         mf = b.get_object('race_timer_pane')
         self.fs = timerpane.timerpane('Front Straight')
+        #self.fs.uport = self.meet.udptimer
         self.fs.bibent.connect('activate', self.bibent_cb, self.fs)
         self.bs = timerpane.timerpane('Back Straight')
+        #self.bs.uport = self.meet.udptimer
+        self.bs.urow = 6	# scb row for timer messages
         self.bs.bibent.connect('activate', self.bibent_cb, self.bs)
         mf.pack_start(self.fs.frame)
         mf.pack_start(self.bs.frame)
