@@ -88,6 +88,13 @@ class namebank(object):
             self.__ind = None
         self.__open = False
 
+    def raw_search(self, skey):
+        """Return an exact match from the index."""
+        ret = None
+        if skey in self.__ind:
+            ret = [a for a in self.__ind[skey]]
+        return ret
+
     def search(self, first=u'', last=u''):
         """Return a set of matching rider ids from the namebank."""
 
@@ -121,7 +128,12 @@ class namebank(object):
             lset = cset
 
         # return intersection of fset and lset
-        return(fset.intersection(lset))
+        return fset.intersection(lset)
+
+    def __iter__(self):
+        """Iterate over all namebank records."""
+        for key in self.__nb:
+            yield(self.__nb[key])
 
     def __len__(self):
         """Called to implement the built-in function len()."""
