@@ -59,7 +59,7 @@ COL_TIME = 6
 COL_PLACE = 7
 COL_POINTS = 8		# array of points for display
 
-PENALTY = 10		# fixed dnf penalty for mass start, needs work
+PENALTY = 10		# fixed fallback penalty for mass start
 
 # scb function key mappings
 key_reannounce = 'F4'	# (+CTRL)
@@ -67,14 +67,6 @@ key_abort = 'F5'	# (+CTRL)
 key_startlist = 'F3'
 key_results = 'F4'
 
-# SCB constants
-SCB_STARTERS_NW = metarace.SCB_LINELEN - 9
-SCB_STARTERS_FMT = [(3, 'r'), ' ', (SCB_STARTERS_NW,'l'), ' ', (4,'r')]
-
-SCB_RESPOINTS_NW = metarace.SCB_LINELEN - 10
-SCB_RESPOINTS_FMT = [(3,'l'),(3,'r'),' ',
-                     (SCB_RESPOINTS_NW,'l'),(3,'r')]
-
 class omnium(object):
     def loadconfig(self):
         """Load race config from disk."""
@@ -83,6 +75,7 @@ class omnium(object):
                                         'id':EVENT_ID,
                                         'showinfo':'Yes',
                                         'events':'',
+                                        'autospec':'',
                                         'evnicks':''})
         cr.add_section('event')
         cr.add_section('riders') # no need fr omnium??
@@ -91,6 +84,7 @@ class omnium(object):
             self.log.debug('Attempting to read config from '
                                + repr(self.configpath))
             cr.read(self.configpath)
+        self.autospec = cr.get('event', 'autospec')
         for r in cr.get('event', 'startlist').split():
             self.addrider(r)
             ## TODO : load/save comment for rider
@@ -101,6 +95,9 @@ class omnium(object):
         self.events = strops.reformat_bibserlist(cr.get('event', 'events'))
         self.nicknames = cr.get('event', 'evnicks').split()
         self.recalculate()
+        if not self.onestart:
+           if self.autospec:
+                self.meet.autostart_riders(self, self.autospec)
 
         # After load complete - check config and report. This ensures
         # an error message is left on top of status stack. This is not
@@ -110,10 +107,11 @@ class omnium(object):
         if eid and eid != EVENT_ID:
             self.log.error('Event configuration mismatch: '
                            + repr(eid) + ' != ' + repr(EVENT_ID))
-            self.readonly = True
+            #self.readonly = True
 
     def startlist_report(self, program=False):
         """Return a startlist report."""
+        ret = []
         sec = printing.section()
         headvec = [u'Event', self.evno, u':', self.event[u'pref'],
                          self.event[u'info']]
@@ -126,7 +124,8 @@ class omnium(object):
             rname = strops.resname(r[COL_FIRST], r[COL_LAST],
                                        r[COL_CLUB])
             sec.lines.append([None, rno, rname, None, None, None])
-        return sec
+        ret.append(sec)
+        return ret
 
     def get_startlist(self):
         """Return a list of bibs in the rider model."""
@@ -146,6 +145,7 @@ class omnium(object):
         cw.set('event', 'events', self.events)
         cw.set('event', 'evnicks', ' '.join(self.nicknames))
         cw.set('event', 'showinfo', self.info_expand.get_expanded())
+        cw.set('event', 'autospec', self.autospec)
         cw.set('event', 'id', EVENT_ID)
         self.log.debug('Saving race config to: ' + self.configpath)
         with open(self.configpath, 'wb') as f:
@@ -169,22 +169,26 @@ class omnium(object):
 
             yield [bib, rank, time, info]
 
-    def result_report(self):
+    def result_report(self, recurse=True):
         """Return a list of printing sections containing the race result."""
         ret = []
-        sec = printing.section()
+        sec = printing.section(u'omnium')
         sec.heading = 'Event ' + self.evno + ': ' + ' '.join([
-                       self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                       self.meet.edb.getvalue(self.event, eventdb.COL_INFO)
-                              ]).strip()
+                         self.event[u'pref'], self.event[u'info']]).strip()
+        sec.subheading = self.standingstr()
         sec.lines = []
         aggtm = None
         prevline = None
 
         for r in self.riders:
-            rno = r[COL_BIB]
-            rname = strops.resname(r[COL_FIRST], r[COL_LAST],
-                                       r[COL_CLUB])
+            rno = r[COL_BIB].decode('utf-8')
+            rh = self.meet.newgetrider(rno, self.series)
+            rname = u''
+            if rh is not None:
+                rname = rh[u'namestr']
+            rcat = None
+            if rh is not None and rh[u'ucicode']:
+                rcat = rh[u'ucicode']   # overwrite by force
             rank = ''
             if r[COL_PLACE] != '':
                 rank = r[COL_PLACE]
@@ -196,7 +200,7 @@ class omnium(object):
             tmstr = ''
             if r[COL_TIME] != tod.ZERO:
                 tmstr = r[COL_TIME].rawtime(2)
-            nrow = [rank, rno, rname, None, ptsstr, None, tmstr]
+            nrow = [rank, rno, rname, rcat, ptsstr, None, tmstr]
             if prevline:
                 if rank and prevline[4] == ptsstr:	# same pts
                     prevline[5] = prevline[6]	# patch tmstr
@@ -210,41 +214,19 @@ class omnium(object):
             sec.lines.append(prevline)
         sec.colheader = [None, None, None, None, 'Points', aggtm]
         ret.append(sec)
+        if recurse:
+            for eno in self.events.split():
+                if eno != self.evno:
+                    lastev = eno
+                    r = self.meet.get_event(eno, False)
+                    if r is None:
+                        self.log.error('Invalid Omnium config: aborting.')
+                        return
+                    r.loadconfig()
+                    ret.extend(r.result_report())
+                    r.destroy()
         return ret
 
-    def result_export(self, f):
-        """Export results to supplied file handle."""
-
-        cr = csv.writer(f)
-        header = ['Event ' + self.evno, '', ' '.join([
-                       self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                       self.meet.edb.getvalue(self.event, eventdb.COL_INFO)
-                              ]).strip(),
-                     '', 'Pt', 'Time']
-        for c in self.nicknames:
-            header.append(c)
-        cr.writerow(header)
-
-        for r in self.riders:
-            rno = r[COL_BIB]
-            rname = strops.resname(r[COL_FIRST], r[COL_LAST],
-                                       r[COL_CLUB])
-            rank = ''
-            if r[COL_PLACE] != '':
-                rank = r[COL_PLACE]
-                if rank.isdigit():
-                    rank += '.'
-            ptsstr = ''
-            if r[COL_TOTAL] > 0:
-                ptsstr = str(r[COL_TOTAL])
-            tmstr = ''
-            if r[COL_TIME] != tod.ZERO:
-                tmstr = r[COL_TIME].rawtime(3)
-            orow = ["'"+rank, rno, rname,'',"'"+ptsstr, "'"+tmstr]
-            for c in r[COL_POINTS]:
-                orow.append("'"+c)
-            cr.writerow(orow)
-
     def addrider(self, bib='', info=None):
         """Add specified rider to race model."""
         nr=[bib, '', '', '', '', 0, tod.tod(0), '', []]
@@ -289,6 +271,31 @@ class omnium(object):
             r[COL_TIME] = tod.tod(0)
             r[COL_POINTS] = []
 
+    def standingstr(self):
+        """Return an event status string for reports and scb."""
+        ret = u''
+        if self.curevent is not None and self.lastevent is not None:
+            if self.curevent != self.lastevent:
+                ev = self.meet.edb[self.curevent]
+                if ev is not None:
+                    if self.curevent in self.eventstatusmap:
+                        if self.eventstatusmap[self.curevent] in ['running']:
+                            # force virtual for race in progress
+                            self.resultvirtual = True
+                    if self.resultvirtual:
+                        ret = u'Virtual Standings'
+                    else:
+                        ret = u'Standings after ' + ev[u'info']
+                else:
+                    ret = u'Standings'               
+            else:
+                # TODO check for completion
+                if self.resultvirtual:
+                    ret = u'Virtual Standings'
+                else:
+                    ret = u'Final Result'
+        return ret
+
     def sortomnium(self, x, y):
         """Sort results according to omnium rules."""
 
@@ -314,28 +321,41 @@ class omnium(object):
     def recalculate(self):
         """Update internal model."""
         self.clearplaces()
+        curev = None
+        lastev = None
 
         # Pass one: Fill in points for all events
+        self.resultvirtual = False # unless rescount differs for any riders
+        self.eventstatusmap = {}
         rescount = {}
+        penalty = len(self.riders)
+        if penalty == 0:
+            penalty = PENALTY
+        self.log.debug(u'Set dnf penalty to: ' + repr(penalty))
         ecnt = 0
         for eno in self.events.split():
             if eno != self.evno:
+                lastev = eno
                 r = self.meet.get_event(eno, False)
                 if r is None:
                     self.log.error('Invalid Omnium config: aborting.')
                     return
                 r.loadconfig()
+                self.eventstatusmap[eno] = r.timerstat
                 lastrank = 0	# used to track dnfs
                 for res in r.result_gen():
                     bib = res[0]
                     lr = self.getrider(bib)
                     if lr is not None:
                         while len(lr[COL_POINTS]) < ecnt:
-                            self.log.warn('Filling in rider ' + repr(bib)
+                            self.log.debug('Filling in rider ' + repr(bib)
                                           + ', missing from event ' + repr(eno))
                             lr[COL_POINTS].append('')
                         if len(lr[COL_POINTS]) == ecnt:
+                            self.log.debug('res[1] = ' + repr(res[1]))
                             if res[1] is not None and lr[COL_PLACE] != 'dnf':
+                                # track highest evt here
+                                curev = eno
                                 self.onestart = True
                                 if type(res[1]) is int:
                                     lr[COL_TOTAL] += res[1]
@@ -346,7 +366,7 @@ class omnium(object):
                                     lastrank = res[1]
                                 elif res[1] == 'dnf':	# DNF gets pos+penalty
                                     lastrank += 1
-                                    lr[COL_TOTAL] += lastrank + PENALTY
+                                    lr[COL_TOTAL] += lastrank + penalty
                                     if bib not in rescount:
                                         rescount[bib] = 1
                                     else:
@@ -359,15 +379,32 @@ class omnium(object):
                                     lr[COL_TIME] = lr[COL_TIME] + res[2]
                             else:
                                 lr[COL_POINTS].append('')
+                                # this rider is not yet placed
+                                if lastev == curev:
+                                    self.resultvirtual = True
+                                    # omnium race in progress
+                                else:
+                                    # race probably not yet started, ignore
+                                    pass
                         else:
                             self.log.error('Ignoring duplicate result for rider ' + repr(bib) + ' in event ' + repr(eno))
                     else:
                         self.log.warn('Result for rider not in aggregate: ' + repr(bib) + ' in event ' + repr(eno))
+                r.destroy()
             else:	# serious problem... but ignore for now
                 self.log.error('Ignoring self in list of aggregate events.')
             ecnt += 1
 
-        # Pass 2: Create aux map and sort model
+        # record event handles for current and last event
+        self.curevent = None
+        if curev:
+            self.curevent = curev
+        self.lastevent = None
+        if lastev:
+            self.lastevent = lastev
+
+        # Pass 2: Create aux map, check virtual and sort model
+        rescheck = None 
         auxtbl = []
         idx = 0
         for r in self.riders:
@@ -378,6 +415,10 @@ class omnium(object):
             dnf = False
             if r[COL_PLACE] != '':	# dnf
                 dnf = True
+            if not dnf:
+                if rescheck is not None and rcnt != rescheck:
+                    self.resultvirtual = True
+                rescheck = rcnt
             arow = [idx, bib, rcnt, dnf, r[COL_TOTAL], r[COL_TIME]]
             auxtbl.append(arow)
             idx += 1
@@ -427,9 +468,8 @@ class omnium(object):
 
             self.meet.ann_title(' '.join([
                   'Event', self.evno, ':',
-                  self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                  self.meet.edb.getvalue(self.event, eventdb.COL_INFO),
-                  '- Standings']))
+                  self.event[u'pref'], self.event[u'info'],
+                  self.standingstr()]))
             self.meet.announce.linefill(1, '_')
             ha = [ '   ', '  #', 'Rider'.ljust(25), ' Pt ']
             for n in self.nicknames:	# configurable error?
@@ -474,32 +514,44 @@ class omnium(object):
         self.meet.scbwin = None
         self.timerwin = False
         startlist = []
+        name_w = self.meet.scb.linelen - 8
+        fmt = [(3, u'r'), u' ', (name_w,u'l'), (4,u'r')]
         for r in self.riders:
             startlist.append([r[COL_BIB],
-                              strops.fitname(r[COL_FIRST],
-                                             r[COL_LAST],
-                                             SCB_STARTERS_NW),
+                              strops.fitname(r[COL_FIRST], r[COL_LAST],
+                                             name_w),
                                   r[COL_CLUB]])
-        self.meet.scbwin = scbwin.scbtable(self.meet.scb,
-                                           self.meet.racenamecat(self.event),
-                       SCB_STARTERS_FMT, startlist)
+        evtstatus = u'Startlist'
+        self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb,
+                                     head=self.meet.racenamecat(self.event),
+                                     subhead=evtstatus,
+                                     coldesc=fmt, rows=startlist)
         self.meet.scbwin.reset()
 
     def do_places(self):
         """Show race result on scoreboard."""
         resvec = []
-        hdr = self.meet.racenamecat(self.event,
-                        metarace.SCB_LINELEN - 3) + ' pt'
+        name_w = self.meet.scb.linelen - 10
+        fmt = [(3,'l'),(3,'r'),' ',(name_w,'l'),(3,'r')]
         for r in self.riders:
-            resvec.append([r[COL_PLACE], r[COL_BIB],
-                         strops.fitname(r[COL_FIRST], r[COL_LAST],
-                                        SCB_RESPOINTS_NW),
-                         str(r[COL_TOTAL])])
-        self.meet.scbwin = None
-        self.meet.scbwin = scbwin.scbtable(self.meet.scb,
-                                           hdr,
-                                SCB_RESPOINTS_FMT,
-                                resvec, delay=90, pagesz=5)
+            if r[COL_PLACE] is not None and r[COL_PLACE] != '':
+                plstr = r[COL_PLACE].decode('utf-8')
+                if plstr.isdigit():
+                    if int(plstr) > 20:
+                        break
+                    plstr = plstr + u'.'
+                resvec.append([plstr,
+                    r[COL_BIB].decode('utf-8'),
+                    strops.fitname(r[COL_FIRST].decode('utf-8'),
+                                   r[COL_LAST].decode('utf-8'),
+                                        name_w),
+                    str(r[COL_TOTAL])])
+        ## TODO: detect standings after nn/provisonal/final
+        evtstatus = self.standingstr()
+        self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb,
+                                    head=self.meet.racenamecat(self.event),
+                                    subhead=evtstatus, coldesc=fmt,
+                                    rows=resvec)
         self.meet.scbwin.reset()
         return False
 
@@ -508,9 +560,12 @@ class omnium(object):
         self.log.debug('Race shutdown: ' + msg)
         if not self.readonly:
             self.saveconfig()
-        self.meet.edb.editevent(self.event, winopen=False)
         self.winopen = False
 
+    def rftimercb(self, e):
+        """Handle rftimer event."""
+        return False
+
     def timercb(self, e):
         """Handle a timer event."""
         return False
@@ -533,6 +588,8 @@ class omnium(object):
         re.set_text(self.events)
         ne = b.get_object('race_nicks_entry')
         ne.set_text(' '.join(self.nicknames))
+        as_e = b.get_object('auto_starters_entry')
+        as_e.set_text(self.autospec)
         response = dlg.run()
         if response == 1:       # id 1 set in glade for "Apply"
             self.log.debug('Updating race properties.')
@@ -544,7 +601,15 @@ class omnium(object):
             ns = se.get_text()
             if ns != self.series:
                 self.series = ns
-                self.meet.edb.editevent(self.event, series=ns)
+                self.event[u'seri'] = ns
+
+            # update auto startlist spec
+            nspec = as_e.get_text()
+            if nspec != self.autospec:
+                self.autospec = nspec
+                if not self.onestart:
+                    if self.autospec:
+                        self.meet.autostart_riders(self, self.autospec)
 
             # xfer starters if not empty
             slist = strops.reformat_riderlist(
@@ -584,10 +649,10 @@ class omnium(object):
 
     def editent_cb(self, entry, col):
         """Shared event entry update callback."""
-        if col == eventdb.COL_PREFIX:
-            self.meet.edb.editevent(self.event, prefix=entry.get_text())
-        elif col == eventdb.COL_INFO:
-            self.meet.edb.editevent(self.event, info=entry.get_text())
+        if col == u'pref':
+            self.event[u'pref'] = entry.get_text().decode('utf-8', 'replace')
+        elif col == u'info':
+            self.event[u'info'] = entry.get_text().decode('utf-8', 'replace')
         self.update_expander_lbl_cb()
 
     def todstr(self, col, cr, model, iter, data=None):
@@ -648,6 +713,10 @@ class omnium(object):
         self.log = logging.getLogger('omnium')
         self.log.setLevel(logging.DEBUG)
         self.log.debug('opening event: ' + str(self.evno))
+        self.curevent = None
+        self.lastevent = None
+        self.resultvirtual = False
+        self.eventstatusmap = {}
 
         # race run time attributes
         self.onestart = False
@@ -655,6 +724,7 @@ class omnium(object):
         self.winopen = True
         self.events = ''
         self.nicknames = []
+        self.autospec = ''      # automatic startlist
 
 
         self.riders = gtk.ListStore(gobject.TYPE_STRING, # 0 bib
@@ -678,15 +748,12 @@ class omnium(object):
         b.get_object('omnium_info_evno').set_text(self.evno)
         self.showev = b.get_object('omnium_info_evno_show')
         self.prefix_ent = b.get_object('omnium_info_prefix')
-        self.prefix_ent.set_text(self.meet.edb.getvalue(
-                   self.event, eventdb.COL_PREFIX))
-        self.prefix_ent.connect('changed', self.editent_cb,
-                                 eventdb.COL_PREFIX)
+        self.prefix_ent.connect('changed', self.editent_cb,u'pref')
+        self.prefix_ent.set_text(self.event[u'pref'])
         self.info_ent = b.get_object('omnium_info_title')
-        self.info_ent.set_text(self.meet.edb.getvalue(
-                   self.event, eventdb.COL_INFO))
-        self.info_ent.connect('changed', self.editent_cb,
-                               eventdb.COL_INFO)
+        self.info_ent.connect('changed', self.editent_cb,u'info')
+        self.info_ent.set_text(self.event[u'info'])
+
         self.update_expander_lbl_cb()
 
         # riders pane
@@ -719,5 +786,3 @@ class omnium(object):
             b.add_from_file(os.path.join(metarace.UI_PATH, 'tod_context.ui'))
             self.context_menu = b.get_object('tod_context')
             b.connect_signals(self)
-            # update properties in meet
-            self.meet.edb.editevent(event, winopen=True)
