@@ -30,13 +30,13 @@ import pango
 import os
 import logging
 import csv
-import decimal
 import ConfigParser
 
 import metarace
 from metarace import timy
 from metarace import scbwin
 from metarace import tod
+from metarace import unt4
 from metarace import uiutil
 from metarace import riderdb
 from metarace import strops
@@ -67,18 +67,7 @@ RES_COL_INFO = 10
 RES_COL_STPTS = 11
 
 # scb consts
-SCB_INTSPRINT_NW = metarace.SCB_LINELEN - 8
-SCB_INTSPRINT_FMT = [(2, u'l'), (3, u'r'), u' ',
-                  (SCB_INTSPRINT_NW, u'l'), u' ', (1, u'r')]
 
-SCB_STARTERS_NW = metarace.SCB_LINELEN - 9
-SCB_STARTERS_FMT = [(3, u'r'), u' ', (SCB_STARTERS_NW,u'l'), u' ', (4,u'r')]
-SCB_RESMADISON_NW = metarace.SCB_LINELEN - 8
-SCB_RESMADISON_FMT = [(2,u'r'),u' ',(SCB_RESMADISON_NW,u'l'),
-                      (2,u'r'),(3,u'r')]
-SCB_RESPOINTS_NW = metarace.SCB_LINELEN - 9
-SCB_RESPOINTS_FMT = [(2,u'l'),(3,u'r'),u' ',
-                     (SCB_RESPOINTS_NW,u'l'),(3,u'r')]	# no club
 SPRINT_PLACE_DELAY = 3		# 3 seconds per place
 SPRINT_PLACE_DELAY_MAX = 11	# to a maximum of 11
 
@@ -118,6 +107,7 @@ class ps(object):
                                         'distance':'',
                                         'distunits':'laps',
                                         'masterslaps':defmasterslaps,
+                                        'inomnium':'No',
                                         'showinfo':'Yes',
                                         'autospec':'',
                                         'scoring':defscoretype})
@@ -131,6 +121,10 @@ class ps(object):
                             + repr(self.configpath))
             cr.read(self.configpath)
 
+        self.inomnium = strops.confopt_bool(cr.get('event', 'inomnium'))
+        if self.inomnium:
+            self.seedsrc = 1    # fetch start list seeding from omnium
+
         for r in cr.get('event', 'startlist').split():
             nr=[r, '', '', '', True, 0, 0, 0, '', -1, '', 0]
             if cr.has_option('points', r):
@@ -205,7 +199,7 @@ class ps(object):
 
         # after load, add auto if required
         if not self.onestart and self.autospec:
-            self.meet.autostart_riders(self, self.autospec)
+            self.meet.autostart_riders(self, self.autospec, self.seedsrc)
 
         # After load complete - check config and report. This ensures
         # an error message is left on top of status stack. This is not
@@ -215,7 +209,7 @@ class ps(object):
         if eid and eid != EVENT_ID:
             self.log.error('Event configuration mismatch: '
                            + repr(eid) + ' != ' + repr(EVENT_ID))
-            self.readonly = True
+            #self.readonly = True
 
     def get_startlist(self):
         """Return a list of bibs in the rider model."""
@@ -250,6 +244,7 @@ class ps(object):
         else:
             cw.set('event', 'masterslaps', 'No')
         cw.set('event', 'autospec', self.autospec)
+        cw.set('event', 'inomnium', self.inomnium)
         cw.set('event', 'sprintlaps', self.sprintlaps)
 
         cw.add_section('sprintplaces')
@@ -297,7 +292,10 @@ class ps(object):
                     if r[RES_COL_PLACE] is not None and r[RES_COL_PLACE] != '':
                         rank = int(r[RES_COL_PLACE])
                 else:
-                    rank = 'dnf'	# ps only handle did not finish
+                    if r[RES_COL_INFO] in ['dns', 'dsq']:
+                        rank = r[RES_COL_INFO]
+                    else:
+                        rank = 'dnf'	# ps only handle did not finish
 
             if self.scoring == 'madison':
                 laps = r[RES_COL_LAPS]
@@ -313,8 +311,9 @@ class ps(object):
             time = None
             yield [bib, rank, time, info]
 
-    def result_report(self):
+    def result_report(self, recurse=False):
         """Return a list of printing sections containing the race result."""
+        self.recalculate()
         ret = []
         sec = printing.section()
         sec.heading = 'Event ' + self.evno + ': ' + ' '.join([
@@ -324,8 +323,6 @@ class ps(object):
         lapstring = strops.lapstring(self.event[u'laps'])
         substr = u' '.join([lapstring, self.event[u'dist'],
                              self.event[u'prog']]).strip()
-        if substr:
-            sec.subheading = substr
         sec.units = 'Pts'
         fs = ''
         if self.finish is not None:
@@ -333,12 +330,20 @@ class ps(object):
         fl = None
         ll = None
         for r in self.riders:
-            rno = r[RES_COL_BIB]
-            rname = strops.resname(r[RES_COL_FIRST].decode('utf-8'), 
-                                   r[RES_COL_LAST].decode('utf-8'),
-                                    r[RES_COL_CLUB].decode('utf-8'))
-            infstr = ''
+            rno = r[RES_COL_BIB].decode('utf-8')
+            rh = self.meet.newgetrider(rno, self.series)
+            rname = u''
+            if rh is not None:
+                rname = rh[u'namestr']
             plstr = ''
+            rcat = None
+            if self.event[u'cate']:
+                if rname is not None and rh[u'cat']:
+                    rcat = rh[u'cat']
+            if self.inomnium:
+                rcat = None
+            if rh is not None and rh[u'ucicode']:
+                rcat = rh[u'ucicode']   # overwrite by force
             if self.onestart and r[RES_COL_PLACE] is not None:
                 plstr = r[RES_COL_PLACE]
                 if r[RES_COL_PLACE].isdigit():
@@ -364,78 +369,21 @@ class ps(object):
                 if plstr or finplace or ptstr: # only output those with points
                                          # dnf  or 
 					# placed in final sprint
-                    sec.lines.append([plstr, rno, rname, None, fs, ptstr])
+                    sec.lines.append([plstr, rno, rname, rcat, fs, ptstr])
                 fs = ''
+
+        if self.onestart:
+            sec.subheading = substr + u' - ' + self.standingstr()
+        else:
+            if substr:
+                sec.subheading = substr
+
         ret.append(sec)
 
         # output intermediate sprints?
 
         return ret
 
-    def result_export(self, f):
-        """Export results to supplied file handle."""
-        cr = csv.writer(f)
-        df = ''
-        dist = self.meet.get_distance(self.distance, self.units)
-        if dist is not None:
-            df = str(dist) + 'm'
-        hdr = ['Event ' + self.evno, '',
-               ' '.join([
-                  self.event[u'pref'],
-                         self.event[u'info']
-                        ]).strip(),
-               '', 'Pts', df]
-        nopts = []
-        for s in self.sprints:
-            hdr.append("'" + s[SPRINT_COL_ID])
-            nopts.append('')
-        hdr.append('Fin')
-        cr.writerow(hdr)
-        fs = ''
-        if self.finish is not None:
-            fs = self.time_lbl.get_text().strip()
-        if True:
-            lapup = None
-            for r in self.riders:
-                rno = r[RES_COL_BIB]
-                rname =  strops.resname(r[RES_COL_FIRST].decode('utf-8'),
-                                   r[RES_COL_LAST].decode('utf-8'),
-                                    r[RES_COL_CLUB].decode('utf-8'))
-                infstr = ''
-                if r[RES_COL_INFO] is not None and r[RES_COL_INFO] != '':
-                    infstr = r[RES_COL_INFO]
-                if lapup is None:
-                    lapup = r[RES_COL_LAPS]
-                else:
-                    fs = ''
-                lapcnt = 0
-                if self.scoring == 'madison':
-                    lapcnt = r[RES_COL_LAPS] - lapup
-                else:
-                    lapcnt = r[RES_COL_LAPS]
-                lapstr = ''
-                if lapcnt != 0 and self.scoring == 'madison':
-                    lapstr = '{0:+}'.format(lapcnt)
-                finplace = 'u/p'
-                if r[RES_COL_FINAL] >= 0:
-                   finplace = str(r[RES_COL_FINAL] + 1)
-                plstr = ''
-                if self.onestart and r[RES_COL_PLACE] is not None:
-                    plstr = "'" + r[RES_COL_PLACE]
-                    if r[RES_COL_PLACE].isdigit():
-                        plstr += '.'
-                ptstr = '-'
-                if r[RES_COL_TOTAL] != 0:
-                    ptstr = str(r[RES_COL_TOTAL])
-                resrow = [plstr, rno, rname, "'"+infstr, "'"+ptstr, "'"+fs]
-
-                if r[RES_COL_BIB] in self.auxmap:
-                    resrow.extend(self.auxmap[r[RES_COL_BIB]])
-                else:
-                    resrow.extend(nopts)
-                resrow.append(finplace)
-                cr.writerow(resrow)
-
     def getrider(self, bib):
         """Return temporary reference to model row."""
         ret = None
@@ -457,13 +405,21 @@ class ps(object):
     def addrider(self, bib='', info=None):
         """Add specified rider to race model."""
         nr = [bib, '', '', '', True, 0, 0, 0, '', -1, '', 0]
-        if bib == '' or self.getrider(bib) is None:
+        er = self.getrider(bib)
+        if bib == '' or er is None:
             dbr = self.meet.rdb.getrider(bib, self.series)
             if dbr is not None:
                 for i in range(1,4):
                     nr[i] = self.meet.rdb.getvalue(dbr, i)
+                if self.inomnium:
+                    if info:
+                        nr[RES_COL_INFO] = info
             return self.riders.append(nr)
         else:
+            if er is not None:
+                self.log.debug('onestart is: ' + repr(self.onestart))
+                if self.inomnium and not self.onestart:
+                    er[RES_COL_INFO] = info
             return None
 
     def delrider(self, bib):
@@ -515,25 +471,40 @@ class ps(object):
             uiutil.buttonchg(self.stat_but, uiutil.bg_none, 'Running')
             self.meet.timer.dearm(timy.CHAN_FINISH)
 
+    def sort_handicap(self, x, y):
+        """Sort function for handicap marks."""
+        if x[2] != y[2]:
+            if x[2] is None:    # y sorts first
+                return 1
+            elif y[2] is None:  # x sorts first
+                return -1
+            else:       # Both should be ints here
+                return cmp(x[2], y[2])
+        else:   # Defer to rider number
+            if x[1].isdigit() and y[1].isdigit():
+                return cmp(int(x[1]), int(y[1]))
+            else:
+                return cmp(x[1], y[1])
+
     def reorder_riderno(self):
         """Sort the rider list by rider number."""
         if len(self.riders) > 1:
             auxmap = []
             cnt = 0
+            intmark = 0
+            if self.inomnium:
+                intmark = strops.mark2int(r[RES_COL_INFO].decode('utf-8','replace'))
             for r in self.riders:
-                auxmap.append([cnt, r[RES_COL_BIB]])
+                auxmap.append([cnt,
+                   r[RES_COL_BIB].decode('utf-8','replace'), intmark])
                 cnt += 1
-            auxmap.sort(self.sort_riderno)
+            auxmap.sort(self.sort_handicap)
             self.riders.reorder([a[0] for a in auxmap])
 
     def startlist_report(self, program=False):
         """Return a startlist report."""
         ret = []
-        sec = None
-        if program: # and etype not in [u'keirin', u'sprint']:
-            sec = printing.twocol_startlist()
-        else:
-            sec = printing.section()
+        sec = printing.twocol_startlist()
         headvec = [u'Event', self.evno, u':', self.event[u'pref'],
                          self.event[u'info']]
         if not program:
@@ -546,16 +517,34 @@ class ps(object):
             sec.subheading = substr
         self.reorder_riderno()
         sec.lines = []
+        col2 = []
         cnt = 0
+        if self.inomnium:
+            sec.lines.append([u' ',u' ',u'The Fence', None, None, None])
+            col2.append([u' ',u' ',u'Sprinters Lane', None, None, None])
         for r in self.riders:
-            rno = r[RES_COL_BIB].decode('utf-8')
-            inf = r[RES_COL_INFO].decode('utf-8')
-            first = r[RES_COL_FIRST].decode('utf-8')
-            last = r[RES_COL_LAST].decode('utf-8')
-            club = r[RES_COL_CLUB].decode('utf-8')
-            rname = strops.resname(first, last, club)
-            sec.lines.append([None, rno, rname, inf, None, None])
             cnt += 1
+            rno = r[RES_COL_BIB].decode('utf-8')
+            rh = self.meet.newgetrider(rno, self.series)
+            inf = r[RES_COL_INFO].decode('utf-8','replace')
+            if self.inomnium:
+                # inf holds seed, ignore for now
+                inf = None
+            # layout needs adhjustment
+            #if rh[u'ucicode']:
+                #inf = rh[u'ucicode']   # overwrite by force
+            rname = u''
+            if rh is not None:
+                rname = rh[u'namestr']
+            if self.inomnium:
+                if cnt%2 == 1:
+                    sec.lines.append([None, rno, rname, inf, None, None])
+                else:
+                    col2.append([None, rno, rname, inf, None, None])
+            else:
+                sec.lines.append([None, rno, rname, inf, None, None])
+        for i in col2:
+            sec.lines.append(i)
         ret.append(sec)
         if cnt > 0 and not program:
             sec = printing.bullet_text()
@@ -574,6 +563,7 @@ class ps(object):
                             + self.event[u'pref'] + u' '
                             + self.event[u'info'])
 
+        name_w = self.meet.scb.linelen - 8
         for r in self.riders:
             self.meet.announce.gfx_add_row([r[0],
                                strops.resname(r[RES_COL_FIRST].decode('utf-8'),
@@ -582,14 +572,23 @@ class ps(object):
                                                 ''])
 
             if r[RES_COL_INRACE]:
+                club = r[RES_COL_CLUB].decode('utf-8','replace')
+                if len(club) > 3:
+                    # look it up?
+                    if self.series in self.meet.ridermap:
+                        rh = self.meet.ridermap[self.series][r[RES_COL_BIB]]
+                        if rh is not None:
+                            club = rh['note']
                 startlist.append([r[RES_COL_BIB],
-                                  strops.fitname(r[RES_COL_FIRST],
-                                                 r[RES_COL_LAST],
-                                                 SCB_STARTERS_NW),
-                                  r[RES_COL_CLUB]])
-        self.meet.scbwin = scbwin.scbtable(self.meet.scb,
-                                           self.meet.racenamecat(self.event),
-                       SCB_STARTERS_FMT, startlist)
+                              strops.fitname(r[RES_COL_FIRST].decode('utf-8'),
+                                             r[RES_COL_LAST].decode('utf-8'),
+                                             name_w),
+                                  club])
+        FMT = [(3, u'r'), u' ', (name_w,u'l'), u' ', (3,u'r')]
+        self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb,
+                                    head=self.meet.racenamecat(self.event),
+                                    subhead=u'STARTLIST',
+                                    coldesc=FMT, rows=startlist)
         self.meet.scbwin.reset()
 
     def key_event(self, widget, event):
@@ -643,12 +642,16 @@ class ps(object):
                     self.meet.announce.setline(3, sinfo + ':')
                     sid = int(self.sprints.get_string_from_iter(i))
                     cnt = 0
+                    unitshown = False
                     for r in self.sprintresults[sid]:
                         pstr = ''
                         if r[3] != '':
-                            pstr = r[3] + 'pt'
+                            pstr = r[3]
+                            if not unitshown:
+                                pstr += 'pts'
+                                unitshown = True
                         self.meet.announce.postxt(4+cnt,0, ' '.join([
-                            strops.truncpad(r[0] + '.', 3),
+                            strops.truncpad(r[0], 3),
                             strops.truncpad(r[1], 3, 'r'),
                             strops.truncpad(r[2], 20),
                             pstr]))
@@ -677,6 +680,7 @@ class ps(object):
             elif sidstart > len(self.sprints)-10:
                 sidstart = len(self.sprints)-10
             if self.scoring == 'madison':
+                leaderboard = []
                 rtype = 'Team '
                 if self.evtype != 'madison':
                     rtype = 'Rider'
@@ -692,9 +696,11 @@ class ps(object):
                 self.meet.announce.setline(10, hdr)
                 curline = 11
                 ldrlap = None
+                curlap = None
                 for r in self.riders:                     
                     if ldrlap is None:
                         ldrlap = r[RES_COL_LAPS]
+                        curlap = ldrlap
                     lapdwn = r[RES_COL_LAPS] - ldrlap
                     lapstr = '  '
                     if lapdwn != 0:
@@ -739,7 +745,17 @@ class ps(object):
                           placestr, bibstr, namestr, lapstr, ptstr,
                           spstr, finstr]))
                     curline += 1
-                
+                    if r[RES_COL_INRACE]:
+                        if curlap > r[RES_COL_LAPS]:
+                            while curlap != r[RES_COL_LAPS]:
+                                curlap -= 1
+                                if curlap < -15:
+                                    break
+                                leaderboard.append(u'-')
+                        leaderboard.append(r[RES_COL_BIB].rjust(2)
+                                           + psrc.rjust(3))
+                self.meet.announce.send_cmd(u'leaderboard',
+                         unichr(unt4.US).join(leaderboard))
             else:
                 # use scratch race style layout for up to 26 riders
                 count = 0       
@@ -784,7 +800,7 @@ class ps(object):
         """Show race result on scoreboard."""
 
         thesec = self.result_report()
-        placestype = u'Leaders: '
+        placestype = u'Standings: '
         if len(thesec) > 0:
             if self.finished: 
                 placestype = u'Result: '
@@ -802,38 +818,53 @@ class ps(object):
         fmt = ''
         hdr = ''
         if self.scoring == 'madison':
-            fmt = SCB_RESMADISON_FMT
-            hdr = ' # team           lap pt'
-            llap = None
+            name_w = self.meet.scb.linelen - 8
+            fmt = [(2,u'r'),u' ',(name_w,u'l'),
+                      (2,u'r'),(3,u'r')]
+            # does this require special consideration?
+            leaderboard = []
+            hdr = u' # team' + ((self.meet.scb.linelen-13) * u' ') + u'lap pt'
+            llap = None		# leader's lap
             for r in self.riders:
                 if r[RES_COL_INRACE]:
+                    bstr = r[RES_COL_BIB].decode('utf-8','ignore')
                     if llap is None:
                         llap = r[RES_COL_LAPS]
                     lstr = str(r[RES_COL_LAPS] - llap)
                     if lstr == '0': lstr = ''
                     pstr = str(r[RES_COL_TOTAL])
                     if pstr == '0': pstr = '-'
-                    resvec.append([r[RES_COL_BIB],
+                    resvec.append([bstr,
                          strops.fitname('',
-                            r[RES_COL_LAST].upper(), SCB_RESMADISON_NW),
+                                     r[RES_COL_LAST].decode('utf-8',
+                                                 'ignore').upper(), name_w),
                          lstr, pstr])
         else:
-            fmt = SCB_RESPOINTS_FMT
-            hdr = self.meet.racenamecat(self.event,
-                        metarace.SCB_LINELEN - 3) + ' pt'
+            name_w = self.meet.scb.linelen-10
+            fmt = [(3,u'l'),(3,u'r'),u' ',
+                     (name_w,u'l'),(3,u'r')]
+                        #self.meet.scb.linelen - 3) + ' pt'
             for r in self.riders:
                 if r[RES_COL_INRACE]:
+                    plstr = r[RES_COL_PLACE] + u'.'
+                    bstr = r[RES_COL_BIB]
                     pstr = str(r[RES_COL_TOTAL])
                     if pstr == '0': pstr = '-'
-                    resvec.append([r[RES_COL_PLACE], r[RES_COL_BIB],
+                    resvec.append([plstr, bstr,
                          strops.fitname(r[RES_COL_FIRST], r[RES_COL_LAST],
-                                        SCB_RESPOINTS_NW),
+                                        name_w),
                          pstr])
             # cols are: rank, bib, name, pts
+        hdr = self.meet.racenamecat(self.event)
         self.meet.scbwin = None
         self.timerwin = False
-        self.meet.scbwin = scbwin.scbtable(self.meet.scb,
-                                           hdr, fmt, resvec, delay=60, pagesz=7)
+        evtstatus = self.standingstr(width=self.meet.scb.linelen).upper()
+        #evtstatus=u'STANDINGS'
+        #if self.finished: 
+            #evtstatus=u'RESULT'
+        self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb, head=hdr,
+                                        subhead=evtstatus, coldesc=fmt,
+                                        rows=resvec)
         self.meet.scbwin.reset()
         return False
 
@@ -890,8 +921,9 @@ class ps(object):
             # and do it on the scoreboard too ?!
             self.meet.scbwin = scbwin.scbintsprint(self.meet.scb,
                                    self.meet.racenamecat(self.event),
-                                   u'Gaining a Lap',
-                                   [(3,u'r'), u' ', (metarace.SCB_LINELEN-4,u'l')],
+                                   u'Gaining a Lap'.upper(),
+                                   [(3,u'r'), u' ',
+                                    (self.meet.scb.linelen-4,u'l')],
                                    srlines)
             self.meet.scbwin.reset()
             self.next_sprint_counter += 1
@@ -935,10 +967,10 @@ class ps(object):
     def showtimer(self):
         """Show race timer on scoreboard."""
         tp = 'Time:'
-        self.meet.scbwin = scbwin.scbtimer(self.meet.scb,
-                                       self.event[u'pref'],
-                         self.event[u'info'],
-                               tp)
+        self.meet.scbwin = scbwin.scbtimer(scb=self.meet.scb,
+                                   line1=self.meet.racenamecat(self.event),
+                                   line2=u'',
+                                   timepfx=tp)
         self.timerwin = True
         self.meet.scbwin.reset()
         if self.timerstat == 'finished':
@@ -947,7 +979,16 @@ class ps(object):
             dist = self.meet.get_distance(self.distance, self.units)
             if dist:
                 self.meet.scbwin.setavg(elap.speedstr(dist))
-            self.meet.scbwin.update()
+                ## UDP hack
+                #spt=elap.rawspeed(dist).rjust(5) + u'   km/h'
+                #msg = unt4.unt4( header=unichr(unt4.DC3) + u'S9',
+                                #xx=1,
+                                #yy=4,
+                                #text=spt)
+                #self.meet.udptimer.sendto(msg.pack(),
+                                          #(self.meet.udpaddr,6789))
+
+                self.meet.scbwin.update()
 
     def shutdown(self, win=None, msg='Exiting'):
         """Terminate race object."""
@@ -987,6 +1028,13 @@ class ps(object):
                     self.meet.scbwin.setavg(elap)
             self.sprintstart = None
 
+    def rftimercb(self, e):
+        """Handle rftimer event."""
+        if e.refid == '':       # got a trigger
+            #return self.starttrig(e)
+            return False
+        return False
+
     def timercb(self, e):
         """Handle a timer event."""
         chan = timy.chan2id(e.chan)
@@ -1077,7 +1125,7 @@ class ps(object):
                 self.autospec = nspec
                 if not self.onestart:
                     if self.autospec:
-                        self.meet.autostart_riders(self, self.autospec)
+                        self.meet.autostart_riders(self, self.autospec, self.seedsrc)
 
             # xfer starters if not empty
             slist = strops.reformat_riderlist(
@@ -1140,9 +1188,27 @@ class ps(object):
     def set_elapsed(self):
         """Update elapsed race time."""
         if self.start is not None and self.finish is not None:
-            self.time_lbl.set_text((self.finish - self.start).timestr(2))
+            et = self.finish - self.start
+            ## UDP hack
+            #msg = unt4.unt4(
+                            #header=unichr(unt4.DC3) + u'N F$',
+                            #xx=0,
+                            #yy=0,
+                            #text=et.timestr(2)[0:12])
+            #self.meet.udptimer.sendto(msg.pack(),
+                                      #(self.meet.udpaddr,6789))
+            self.time_lbl.set_text(et.timestr(2))
         elif self.start is not None:    # Note: uses 'local start' for RT
-            self.time_lbl.set_text((tod.tod('now') - self.lstart).timestr(1))
+            runtm  = (tod.tod('now') - self.lstart).timestr(1)
+            ## UDP hack
+            #msg = unt4.unt4(
+                            #header=unichr(unt4.DC3) + u'R F$',
+                            #xx=0,
+                            #yy=0,
+                            #text=runtm[0:12])
+            #self.meet.udptimer.sendto(msg.pack(),
+                                      #(self.meet.udpaddr,6789))
+            self.time_lbl.set_text(runtm)
         elif self.timerstat == 'armstart':
             self.time_lbl.set_text(tod.tod(0).timestr(1))
         else:
@@ -1200,7 +1266,7 @@ class ps(object):
                 if self.start is not None and self.finish is not None:
                     self.log_elapsed()
                 self.log.info('Updated race times.')
-            except (decimal.InvalidOperation, ValueError), v:
+            except Exception as v:
                 self.log.error('Error updating times: ' + str(v))
         else:
             self.log.info('Edit race times cancelled.')
@@ -1216,6 +1282,36 @@ class ps(object):
             self.ctrl_places.set_text('')
         self.ctrl_places.grab_focus()
 
+    def standingstr(self, width=None):
+        """Return an event status string for reports and scb."""
+        ret = u'Standings'
+        totsprints = len(self.sprints)
+        lastsprint = None
+        cur = 1
+        for s in self.sprints:
+            self.log.debug(u'cur: ' + repr(cur) + u' val: ' + repr(s[SPRINT_COL_PLACES]))
+            if s[SPRINT_COL_PLACES]:
+                lastsprint = cur
+            cur += 1
+        if lastsprint >= totsprints:
+            ret = u'Result'
+            # check for all places in final sprint?
+            for r in self.riders:
+                if r[RES_COL_INRACE] and r[RES_COL_FINAL] < 0:
+                    # not placed at finish
+                    ret = u'Provisional Result'
+                    break
+        else:
+            ret = u'Standings'
+            if lastsprint:
+                if width is not None and width < 25:
+                    ret += u' - Sprint {0}/{1}'.format(lastsprint,
+                                                          totsprints)
+                else:
+                    ret += u' After Sprint {0} of {1}'.format(lastsprint,
+                                                             totsprints)
+        return ret
+
     def delayed_result(self):
         """Roll the places entry on to the next sprint."""
         ## TODO : Find correct re-entrant method for handling delayed
@@ -1225,11 +1321,15 @@ class ps(object):
         elif self.next_sprint_counter == 1:
             self.next_sprint_counter = 0
             self.do_places()
-            if self.oktochangecombo:
-                i = self.ctrl_place_combo.get_active_iter()
-                i = self.ctrl_place_combo.get_model().iter_next(i)
-                if i is not None:
-                    self.ctrl_place_combo.set_active_iter(i)
+            if self.ctrl_places.get_property('has-focus'):
+                if self.oktochangecombo:
+                    i = self.ctrl_place_combo.get_active_iter()
+                    i = self.ctrl_place_combo.get_model().iter_next(i)
+                    if i is not None:
+                        self.ctrl_place_combo.set_active_iter(i)
+            else:
+                # input widget lost focus, don't auto advance
+                self.oktochangecombo = False
         else:
             self.next_sprint_counter = 0        # clamp negatives
         return False
@@ -1274,10 +1374,12 @@ class ps(object):
             self.timerwin = False
             self.log.info(sinfo + ': ' + places)
             if prevplaces == '':
+                FMT = [(2, u'l'), (3, u'r'), u' ',
+                  (self.meet.scb.linelen-8, u'l'), u' ', (1, u'r')]
                 self.meet.scbwin = scbwin.scbintsprint(self.meet.scb,
-                                   self.meet.racenamecat(self.event), sinfo,
-                                   SCB_INTSPRINT_FMT,
-                                   self.sprintresults[sid][0:4])
+                                   self.meet.racenamecat(self.event),
+                                   sinfo.upper(),
+                                   FMT, self.sprintresults[sid][0:4])
                 self.meet.scbwin.reset()
                 self.next_sprint_counter += 1
                 delay = SPRINT_PLACE_DELAY * len(self.sprintresults[sid]) or 1
@@ -1388,6 +1490,7 @@ class ps(object):
         self.sprintresults[index] = []
         place = 0
         count = 0
+        name_w = self.meet.scb.linelen - 8
         for placegroup in placestr.split():
             for bib in placegroup.split('-'):
                 if bib not in placeset:
@@ -1411,9 +1514,15 @@ class ps(object):
                     fname = r[RES_COL_FIRST].decode('utf-8')
                     lname = r[RES_COL_LAST].decode('utf-8')
                     club = r[RES_COL_CLUB].decode('utf-8')
+                    if len(club) > 3:
+                        # look it up?
+                        if self.series in self.meet.ridermap:
+                            rh = self.meet.ridermap[self.series][bib]
+                            if rh is not None:
+                                club = rh['note']
                     self.sprintresults[index].append([plstr,
                            r[RES_COL_BIB],
-                           strops.fitname(fname, lname, SCB_INTSPRINT_NW),
+                           strops.fitname(fname, lname, name_w),
                                 ptsstr, strops.resname(fname, lname, club)])
                     count += 1
                 else:
@@ -1646,6 +1755,8 @@ class ps(object):
         self.oktochangecombo = False
         self.autospec = ''
         self.finished = False
+        self.inomnium = False
+        self.seedsrc = None
 
         # data models
         self.sprints = gtk.ListStore(gobject.TYPE_STRING,   # ID = 0
