@@ -36,7 +36,6 @@ The generic 'race' model requires the following interface:
     race.destroy()                 - send destroy signal to event window
     race.show()                    - show event window
     race.hide()                    - hide event window
-    race.result_export(f)          - write event results to stream 'f'
     race.addrider(bib)             - add a new starter with given bib
     race.delrider(bib)             - remove starter with given bib
     race.key_event(widget, event)  - race key handler
@@ -48,7 +47,6 @@ import gtk
 import glib
 import gobject
 import pango
-import decimal
 import logging
 import ConfigParser
 import csv
@@ -56,6 +54,7 @@ import os
 
 import metarace
 from metarace import tod
+from metarace import unt4
 from metarace import timy
 from metarace import riderdb
 from metarace import scbwin
@@ -75,15 +74,6 @@ COL_INFO = 4
 COL_DNF = 5
 COL_PLACE = 6
 
-SCB_RESNAME_WIDTH = 24 - 11
-SCB_RESULT_FMT = [(2, u'l'), (3, u'r'), u' ',
-                  (SCB_RESNAME_WIDTH, u'l'), u' ', (4, u'r')]
-
-##! ! ! fix the hardcoded name formats
-SCB_STARTNAME_WIDTH = 24 - 9
-SCB_STARTERS_FMT = [(3, u'r'), u' ', (SCB_STARTNAME_WIDTH,u'l'),
-                   u' ', (4,u'r')]
-
 # scb function key mappings
 key_startlist = 'F3'			# show starters in table
 key_results = 'F4'			# recalc/show result window
@@ -150,9 +140,10 @@ class race(object):
                  glib.timeout_add_seconds(1, self.delayed_reorder)
             return self.riders.append(nr)
         else:
-            if er is not None:			# rider in model, repatch?
+            if er is not None:
+                # Rider already in the model, set the info if
+                # event type is handicap or event is part of omnium
                 if self.evtype == u'handicap' or self.inomnium:
-                    # TODO: check inomnium logic
                     if not er[COL_INFO] and info: # don't overwrite if set
                         er[COL_INFO] = info
             return None
@@ -181,6 +172,7 @@ class race(object):
         placeset = set()
         place = 1
         count = 0
+        resname_w = self.meet.scb.linelen - 11
 
         for placegroup in placestr.split():
             for bib in placegroup.split(u'-'):
@@ -193,10 +185,18 @@ class race(object):
                         r = self.getrider(bib)
                     rank = place + self.startplace
                     r[COL_PLACE] = rank
-                    self.results.append([unicode(rank), r[COL_BIB].decode('utf-8','replace'),
-                       strops.fitname(r[COL_FIRSTNAME].decode('utf-8','replace'),
-                       r[COL_LASTNAME].decode('utf-8','replace'), SCB_RESNAME_WIDTH),
-                       r[COL_CLUB].decode('utf-8','replace')])
+                    club = r[COL_CLUB].decode('utf-8','replace')
+                    if len(club) > 3:
+                        # look it up?
+                        if self.series in self.meet.ridermap:
+                            rh = self.meet.ridermap[self.series][bib]
+                            if rh is not None:
+                                club = rh['note']
+                    self.results.append([unicode(rank)+u'.',
+                     r[COL_BIB].decode('utf-8','replace'),
+                     strops.fitname(r[COL_FIRSTNAME].decode('utf-8','replace'),
+                       r[COL_LASTNAME].decode('utf-8','replace'),
+                       resname_w), club])
                     i = self.getiter(bib)
                     self.riders.swap(self.riders.get_iter(count), i)
                     count += 1
@@ -227,6 +227,7 @@ class race(object):
                                         'ctrl_places':'',
                                         'start':'',
                                         'lstart':'',
+					'comments':'',
                                         'finish':'',
 					'distance':defdistance,
 					'distunits':defdistunits,
@@ -261,10 +262,19 @@ class race(object):
             self.riders.append(nr)
 
         # race infos
+        self.comments = []
+        nc = cr.get('event', 'comments')
+        if nc:
+            self.comments.append(nc)
         self.startplace = strops.confopt_posint(cr.get('event', 'startplace'))
         self.set_timetype(cr.get('event', 'timetype'))
         self.distance = strops.confopt_dist(cr.get('event', 'distance'))
         self.units = strops.confopt_distunits(cr.get('event', 'distunits'))
+        if self.timetype != '200m' and self.event[u'laps']:
+            # use event program to override
+            self.units = 'laps'
+            self.distance = strops.confopt_posint(self.event[u'laps'],
+                                                  self.distance)
         self.info_expand.set_expanded(strops.confopt_bool(
                                        cr.get('event', 'showinfo')))
         self.set_start(cr.get('event', 'start'), cr.get('event', 'lstart'))
@@ -291,7 +301,12 @@ class race(object):
         if eid and eid != EVENT_ID:
             self.log.error('Event configuration mismatch: '
                            + repr(eid) + ' != ' + repr(EVENT_ID))
-            self.readonly = True
+            #self.readonly = True
+
+    def sort_riderno(self, x, y):
+        """Sort riders by rider no."""
+        return cmp(strops.riderno_key(x[1]),
+                    strops.riderno_key(y[1]))
 
     def sort_handicap(self, x, y):
         """Sort function for handicap marks."""
@@ -303,10 +318,8 @@ class race(object):
             else:	# Both should be ints here
                 return cmp(x[2], y[2])
         else:	# Defer to rider number
-            if x[1].isdigit() and y[1].isdigit():
-                return cmp(int(x[1]), int(y[1]))
-            else:
-                return cmp(x[1], y[1])
+            return cmp(strops.riderno_key(x[1]),
+                        strops.riderno_key(y[1]))
 
     def reorder_handicap(self):
         """Reorder rider model according to the handicap marks."""
@@ -316,7 +329,10 @@ class race(object):
             for r in self.riders:
                 auxmap.append([cnt, r[COL_BIB].decode('utf-8','replace'), strops.mark2int(r[COL_INFO].decode('utf-8','replace'))])
                 cnt += 1
-            auxmap.sort(self.sort_handicap)
+            if self.evtype == u'handicap':
+                auxmap.sort(self.sort_handicap)
+            else:
+                auxmap.sort(self.sort_riderno)
             self.riders.reorder([a[0] for a in auxmap])
 
     def set_timetype(self, data=None):
@@ -376,10 +392,26 @@ class race(object):
         """Update elapsed time in race ui and announcer."""
         if self.start is not None and self.finish is not None:
             et = self.finish - self.start
+            ## UDP hack
+            #msg = unt4.unt4(
+                            #header=unichr(unt4.DC3) + u'N F$',
+                            #xx=0,
+                            #yy=0,
+                            #text=et.timestr(2)[0:12])
+            #self.meet.udptimer.sendto(msg.pack(),
+                                      #(self.meet.udpaddr,6789))
             self.time_lbl.set_text(et.timestr(2))
         elif self.start is not None:	# Note: uses 'local start' for RT
-            self.time_lbl.set_text((tod.tod('now')
-                                      - self.lstart).timestr(1))
+            runtm  = (tod.tod('now') - self.lstart).timestr(1)
+            ## UDP hack
+            #msg = unt4.unt4(
+                            #header=unichr(unt4.DC3) + u'R F$',
+                            #xx=0,
+                            #yy=0,
+                            #text=runtm[0:12])
+            #self.meet.udptimer.sendto(msg.pack(),
+                                      #(self.meet.udpaddr,6789))
+            self.time_lbl.set_text(runtm)
         elif self.timerstat == 'armstart':
             self.time_lbl.set_text(tod.tod(0).timestr(1))
         else:
@@ -446,10 +478,12 @@ class race(object):
         ret = []
         sec = None
         etype = self.event[u'type']
-        if program: # and etype not in [u'keirin', u'sprint']:
-            sec = printing.twocol_startlist()
-        else:
+        twocol = True
+        #if not self.inomnium:
+        if False: #etype in [u'handicap']: # HACK - reqstartlist,u'keirin',u'sprint']:
             sec = printing.section()
+        else:
+            sec = printing.twocol_startlist()
         headvec = []
         if etype != 'break':
             headvec.extend([u'Event', self.evno, u':'])
@@ -467,20 +501,41 @@ class race(object):
         self.reorder_handicap()
         sec.lines = []
         cnt = 0
+        col2 = []
+        if self.inomnium:
+            sec.lines.append([u' ',u' ',u'The Fence', None, None, None])
+            col2.append([u' ',u' ',u'Sprinters Lane', None, None, None])
         for r in self.riders:
-            rno = r[COL_BIB].decode('utf-8','replace')
+            cnt += 1
+            rno = r[COL_BIB].decode('utf-8')
+            rh = self.meet.newgetrider(rno, self.series)
             inf = r[COL_INFO].decode('utf-8','replace')
             if self.evtype in [u'keirin', u'sprint']: # encirc draw no
                 inf = strops.drawno_encirc(inf)
-            rname = strops.resname(r[COL_FIRSTNAME].decode('utf-8','replace'), r[COL_LASTNAME].decode('utf-8','replace'),
-                                       r[COL_CLUB].decode('utf-8','replace'))
-            sec.lines.append([None, rno, rname, inf, None, None])
-            cnt += 1
+            if self.inomnium:
+                # inf holds seed, ignore for now
+                inf = None
+            # layout needs adjust
+            #if rh[u'ucicode']:
+                #inf = rh[u'ucicode']   # overwrite by force
+            rname = u''
+            if rh is not None:
+                rname = rh[u'namestr']
+            if self.inomnium:
+                if cnt%2 == 1:
+                    sec.lines.append([None, rno, rname, inf, None, None])
+                else:
+                    col2.append([None, rno, rname, inf, None, None])
+            else:
+                sec.lines.append([None, rno, rname, inf, None, None])
+        for i in col2:
+            sec.lines.append(i)
         if self.event[u'plac']:
             while cnt < self.event[u'plac']:
                 sec.lines.append([None, None, None, None, None, None])
                 cnt += 1
         ret.append(sec)
+
         if cnt > 0 and not program:
             sec = printing.bullet_text()
             sec.lines.append([None, 'Total riders: ' + str(cnt)])
@@ -515,6 +570,10 @@ class race(object):
         cw.set('event', 'timetype', self.timetype)
         cw.set('event', 'autospec', self.autospec)
         cw.set('event', 'inomnium', self.inomnium)
+        thecom = u''
+        if len(self.comments) > 0:
+           thecom = self.comments[0]
+        cw.set('event', 'comments', thecom)
         cw.set('event', 'startplace', self.startplace)
 
         cw.add_section('riders')
@@ -668,8 +727,11 @@ class race(object):
         tp = 'Time:'
         if self.timetype == '200m':
             tp = '200m:'
-        self.meet.scbwin = scbwin.scbtimer(self.meet.scb, self.event[u'pref'],
-                                           self.event[u'info'], tp)
+        self.meet.announce.send_cmd(u'eliminated',u' ')
+        self.meet.scbwin = scbwin.scbtimer(scb=self.meet.scb,
+                                     line1=self.meet.racenamecat(self.event),
+                                     line2=u'',
+                                     timepfx=tp)
         self.timerwin = True
         self.meet.scbwin.reset()
         if self.timerstat == 'finished':
@@ -679,6 +741,15 @@ class race(object):
                 dist = self.meet.get_distance(self.distance, self.units)
                 if dist:
                     self.meet.scbwin.setavg(elap.speedstr(dist))
+                    ## UDP hack
+                    #spt=elap.rawspeed(dist).rjust(5) + u'   km/h'
+                    #msg = unt4.unt4( header=unichr(unt4.DC3) + u'S9',
+                                    #xx=1,
+                                    #yy=4,
+                                    #text=spt)
+                    #self.log.info(u'sending: ' + msg.text)
+                    #self.meet.udptimer.sendto(msg.pack(),
+                                              #(self.meet.udpaddr,6789))
             self.meet.scbwin.update()
 
     def key_event(self, widget, event):
@@ -726,10 +797,17 @@ class race(object):
         if self.start is not None and self.finish is not None:
             ts = (self.finish - self.start).timestr(2)
         if self.doscbplaces:
-            self.meet.scbwin = scbwin.scbtable(self.meet.scb,
-                                  self.meet.racenamecat(self.event),
-                                  SCB_RESULT_FMT, self.results[0:4],
-                                  timepfx=tp, timestr=ts,pagesz=5)
+            FMT = [(3, u'l'), (3,u'r'),u' ',
+                  (self.meet.scb.linelen-11, u'l'),(4,u'r')]
+
+            ## TODO: provisional/final?
+            evtstatus = u'Result'
+            self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb,
+                                  head=self.meet.racenamecat(self.event),
+                                  subhead=evtstatus,
+                                  coldesc=FMT, rows=self.results,
+                                  #coldesc=FMT, rows=self.results[0:4],
+                                  timepfx=tp, timestr=ts)
             self.meet.scbwin.reset()
             self.doscbplaces = False
             self.meet.announce.gfx_overlay(1)
@@ -752,13 +830,22 @@ class race(object):
         self.meet.announce.gfx_set_title(u'Startlist: ' 
                             + self.event[u'pref'] + u' '
                             + self.event[u'info'])
+        name_w = self.meet.scb.linelen-8
         for r in self.riders:
             if not r[5]:
                 nfo = r[4]			# Try info field
+                if self.evtype in [u'sprint']: # add asterisk
+                    nfo = nfo + r[3].rjust(3)	# fall back on club/affil
                 if nfo is None or nfo == '':
-                    nfo = r[3]			# fall back on club/affil
+                    nfo = r[3]
+                    if len(nfo) > 3:
+                        # look it up?
+                        if self.series in self.meet.ridermap:
+                            rh = self.meet.ridermap[self.series][r[0]]
+                            if rh is not None:
+                                nfo = rh['note']
                 startlist.append([r[0], strops.fitname(r[1], r[2],
-                                 SCB_STARTNAME_WIDTH), nfo])
+                                 name_w), nfo])
                 inf = r[COL_INFO].decode('utf-8','replace').strip()
                 if self.evtype in [u'keirin', u'sprint']: # encirc draw no
                     inf = strops.drawno_encirc(inf)
@@ -767,9 +854,12 @@ class race(object):
                                                 r[2].decode('utf-8'),
                                                 r[3].decode('utf-8')),
                                                 inf])
-        self.meet.scbwin = scbwin.scbtable(self.meet.scb,
-                             self.meet.racenamecat(self.event),
-                             SCB_STARTERS_FMT, startlist)
+        FMT = [(3, u'r'), u' ', (name_w,u'l'),
+                   (4,u'r')]
+        self.meet.scbwin = scbwin.scbtable(scb=self.meet.scb,
+                             head=self.meet.racenamecat(self.event),
+                             subhead=u'STARTLIST',
+                             coldesc=FMT, rows=startlist)
         self.meet.scbwin.reset()
 
     def stat_but_cb(self, button):
@@ -863,17 +953,21 @@ class race(object):
                     fname = r[COL_FIRSTNAME].decode('utf-8','replace')
                     lname = r[COL_LASTNAME].decode('utf-8','replace')
                     club = r[COL_CLUB].decode('utf-8','replace')
-                    rstr = r[COL_BIB] + u' ' + strops.fitname(
-                                fname, lname, metarace.SCB_LINELEN-6-len(r[COL_BIB].decode('utf-8','replace')))
-                    self.meet.scbwin = scbwin.scbintsprint(self.meet.scb,
-                          self.event[u'pref'], self.event[u'info'],
-                                   [u'Out: ', (metarace.SCB_LINELEN-5, u'l')],
-                                   [ [rstr] ])
+                    rno = r[COL_BIB].decode('utf-8','replace')
+                    rstr = (rno + u' '
+                            + strops.fitname(fname, lname,
+                                          self.meet.scb.linelen-3-len(rno)))
+                    self.meet.scbwin = scbwin.scbintsprint(scb=self.meet.scb,
+                                 line1=self.meet.racenamecat(self.event),
+                                 line2=u'Rider Eliminated',
+                            coldesc=[ u' ',(self.meet.scb.linelen-1, u'l')],
+                            rows=[ [rstr] ])
                     self.meet.scbwin.reset()
                     self.meet.gemini.reset_fields()
                     self.meet.gemini.set_bib(bib)
                     self.meet.gemini.show_brt()
 
+                    self.meet.announce.send_cmd(u'eliminated', bib.rjust(2))
                     self.meet.announce.gfx_overlay(2)
                     self.meet.announce.gfx_set_title(
                                         self.event[u'pref'] + u' '
@@ -960,6 +1054,13 @@ class race(object):
                     #self.meet.gemini.rtick(self.finish - self.start, 2)
             glib.idle_add(self.delayed_announce)
 
+    def rftimercb(self, e):
+        """Handle rftimer event."""
+        if e.refid == '':       # got a trigger
+            #return self.starttrig(e)
+            return False
+        return False
+
     def timercb(self, e):
         """Handle a timer event."""
         chan = timy.chan2id(e.chan)
@@ -1009,7 +1110,7 @@ class race(object):
                 if self.start is not None and self.finish is not None:
                     self.log_elapsed()
                 self.log.info('Updated race times.')
-            except (decimal.InvalidOperation, ValueError), v:
+            except Exception as v:
                 self.log.error('Error updating times: ' + str(v))
 
             glib.idle_add(self.delayed_announce)
@@ -1031,14 +1132,17 @@ class race(object):
                     if r[COL_PLACE] is not None and r[COL_PLACE] != '':
                         rank = int(r[COL_PLACE].decode('utf-8','replace'))
                 else:
-                    rank = 'dnf'    # only handle did not finish for now
+                    if r[COL_INFO] in ['dns', 'dsq']:
+                        rank = r[COL_INFO]
+                    else:
+                        rank = 'dnf'
             time = None
             if self.finish is not None and ft is None:
                 time = (self.finish - self.start).rawtime(2)
                 ft = True
             yield [bib, rank, time, info]
 
-    def result_report(self):
+    def result_report(self, recurse=False):
         """Return a list of printing sections containing the race result."""
         self.placexfer(self.ctrl_places.get_text())
         ret = []
@@ -1049,43 +1153,62 @@ class race(object):
         lapstring = strops.lapstring(self.event[u'laps'])
         substr = u' '.join([lapstring, self.event[u'dist'],
                              self.event[u'prog']]).strip()
-        if substr:
+        first = True
+        fs = ''
+        if self.finish is not None:
+            fs = self.time_lbl.get_text().strip()
+        rcount = 0
+        pcount = 0
+        for r in self.riders:
+            rcount += 1
+            rno = r[COL_BIB].decode('utf-8')
+            rh = self.meet.newgetrider(rno, self.series)
+            rname = u''
+            if rh is not None:
+                rname = rh[u'namestr']
+            inf = r[COL_INFO].decode('utf-8','replace').strip()
+            if self.evtype in [u'keirin', u'sprint']: # encirc draw no
+                inf = strops.drawno_encirc(inf)
+            # but suppress inf if within an omnium
+            if self.inomnium:
+                inf = None
+            if rh is not None and rh[u'ucicode']:
+                inf = rh[u'ucicode']   # overwrite by force
+            plstr = u''
+            if r[COL_DNF]:
+                pcount += 1
+                if r[COL_INFO] in ['dns', 'dsq']:
+                    plstr = r[COL_INFO]
+                    inf = None
+                else:
+                    plstr = u'dnf'
+            elif self.onestart and r[COL_PLACE] != u'':
+                plstr = r[COL_PLACE].decode('utf-8','replace') + u'.'
+                pcount += 1
+            if plstr:	# don't emit a row for unplaced riders
+                if not first:
+                    sec.lines.append([plstr,rno,rname,inf, None, None])
+                else:
+                    sec.lines.append([plstr,rno,rname,inf, fs, None])
+                    first = False
+        if self.onestart:
             sec.subheading = substr
-
-        if self.timerstat != 'finished':
-            # emit a 'start list' -> tempo!
-            for r in self.riders:
-                rno = r[COL_BIB].decode('utf-8','replace')
-                rname = strops.resname(r[COL_FIRSTNAME].decode('utf-8','replace'), r[COL_LASTNAME].decode('utf-8','replace'),
-                                       r[COL_CLUB].decode('utf-8','replace'))
-                inf = r[COL_INFO].decode('utf-8','replace').strip()
-                sec.lines.append([None, rno, rname, None, None, inf])
+            if rcount > 0 and pcount < rcount:
+                sec.subheading += u' - Provisional Result'
+            else:
+                sec.subheading += u' - Result'
         else:
-            first = True
-            fs = ''
-            if self.finish is not None:
-                fs = self.time_lbl.get_text().strip()
-            for r in self.riders:
-                rno = r[COL_BIB].decode('utf-8','replace')
-                rname = strops.resname(r[COL_FIRSTNAME].decode('utf-8','replace'), r[COL_LASTNAME].decode('utf-8','replace'),
-                                       r[COL_CLUB].decode('utf-8','replace'))
-                inf = r[COL_INFO].decode('utf-8','replace').strip()
-                if self.evtype in [u'keirin', u'sprint']: # encirc draw no
-                    inf = strops.drawno_encirc(inf)
-                plstr = ''
-                if r[COL_DNF]:
-                    plstr = 'dnf'
-                elif self.onestart and r[COL_PLACE] != '':
-                    plstr = r[COL_PLACE].decode('utf-8','replace') + '.'
-                if plstr:	# don't emit a row for unplaced riders
-                    if not first:
-                        # ( this is problematic - handicap is really a propert of the rider
-                        sec.lines.append([plstr,rno,rname,inf, None])
-                    else:
-                        sec.lines.append([plstr,rno,rname,inf, fs, None])
-                        first = False
+            if substr:
+                sec.subheading = substr
 
         ret.append(sec)
+
+        if len(self.comments) > 0:
+            sec = printing.bullet_text()
+            sec.subheading = u'Decisions of the commisaires panel'
+            for c in self.comments:
+                sec.lines.append([None, c])
+            ret.append(sec)
         return ret
 
     def destroy(self):
@@ -1123,6 +1246,7 @@ class race(object):
         self.results = []
 
         self.readonly = not ui
+        self.comments = []
         self.onestart = False
         self.start = None
         self.lstart = None
