@@ -15,30 +15,49 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Rider 'database' container object with model and view.
+"""Rider 'database' utilities
 
-Manage a rider db with the following data columns (all string):
+Manage a rider db with the columns described below. Input columns
+may be arranged in any order if the column header is present. Riders
+are uniquely identified by their series and number, which may be specified
+explicitly as a tuple: (no,series) or as a combined string 'no.series'. 
 
-  Bib		Bib 'number' lowercase alphanumeric string, only chars a-z0-9
-  First Name	Rider's full first name
-  Last Name	Rider's full last name
-  Club		Rider's club or affiliation for the meet
-  Category	Riders racing category FOR THE MEET
-  Series	A non-overlapping series of bib numbers
-		may be the empty string '' (default)
-  Refid		A unique reference string for the rider - used for RFID
- 
-"Primary key" is (Bib, Series)
+Numbers and series keys are matched case-insensitively, and must only 
+contain alphanumeric characters 0-9, A-Z, a-z. All other values associated
+with a rider are unicode strings.
+
+	no	"No"
+		Rider number
+	seri	"Series"
+		Rider's number series (optional)
+	firs	"First Name"
+		Rider full first name
+	last	"Last Name"
+		Rider full last name
+	club	"Club"
+		Club/team/state/nation field
+	cate	"Category"
+		A space separated list of rider category keys for this rider
+	uci	"UCI Code"
+		Rider's UCI code
+	lice	"License No."
+		Rider's License no or namebank key
+	rfid	"RFID No"
+		Rider's RFID, or Transponder no (if relevant)
+	note	"Notes"
+		Additional notes on this rider
+
+Reserved number series:
+
+	team	Extended team information (for road tours)
+	cat	Extended category information
 
-When Series is not present the empty series is assumed: ''
-A bib number conflict should raise error on import, this will force
-explicit series confirmation before import into meet succeeds.
 
 """
 
 import gtk
-import glib
 import gobject
+import glib
 import logging
 import os
 
@@ -56,19 +75,49 @@ COL_CAT = 4
 COL_SERIES = 5
 COL_REFID = 6
 COL_UCICODE = 7
+COL_NOTE = 8
 
 # wheeltime refid prefix
 TAGPREFIX = '058001'
 
 # Reserved rider numbers.
-RESERVED_NOS = [u'bib', u'no.', u'no', u'all', u'number', u'num', u'#']
+RESERVED_NOS = [u'bib', u'no.', u'no', u'all', u'number', u'num']
+
+# Default rider values if not empty string
+RIDER_DEFAULTS = {
+	u'no':None,
+}
+
+# rider db column heading and key mappings
+RIDER_COLUMNS = {
+	u'no':		u"No",
+	u'seri':	u"Series",
+	u'firs':	u"First Name",
+	u'last':	u"Last Name",
+	u'club':	u"Club",
+	u'cate':	u"Category",
+	u'uci':		u"UCI Code",
+	u'lice':	u"License No.",
+	u'rfid':	u"RFID No",
+	u'note':	u"Notes"
+}
+
+RIDER_COLUMN_CONVERTERS= {
+	u'no':strops.confopt_riderno,
+	u'seri':strops.confopt_riderno
+}
+
+
+def colkey(colstr=u''):
+    return colstr[0:4].lower().strip()
+
 
 class riderdb(object):
     """Rider database."""
     def addempty(self, bib=u'', series=u''):
         """Add a new empty row in the rider model."""
         i = self.model.append([bib.lower(), u'', u'', u'', u'',
-                                series, u'', u''])
+                                series, u'', u'', u''])
         ref = gtk.TreeRowReference(self.model, self.model.get_path(i))
         if self.view is not None:
             self.postedit = None
@@ -124,8 +173,8 @@ class riderdb(object):
                     if len(ir) > 0 and ir[0] != u'Bib' and ir[0] != u'No.':
                         bib = ir[COL_BIB].strip().lower()
                         if bib.isalnum() and bib not in RESERVED_NOS:
-                            nr = [bib, u'', u'', u'', u'', u'', u'', u'']
-                            for i in range(1,8):
+                            nr = [bib, u'', u'', u'', u'', u'', u'', u'', u'']
+                            for i in range(1,9):
                                 if len(ir) > i:
                                     nr[i] = ir[i].strip()
 
@@ -241,7 +290,8 @@ class riderdb(object):
         with open(csvfile, 'wb') as f:
             cr = ucsv.UnicodeWriter(f)
             cr.writerow([u'No.',u'First Name',u'Last Name',u'Club',
-                         u'Category',u'Series(optional)',u'Refid',u'UCIcode'])
+                         u'Category',u'Series(optional)',u'Refid',
+                          u'UCIcode',u'Note'])
             cr.writerows(self)	# check the unicode out-in-out-in
 
     def gotorow(self, ref=None):
@@ -287,6 +337,7 @@ class riderdb(object):
                 cat = r[COL_CAT].decode('utf-8')
                 refid = r[COL_REFID].decode('utf-8')
                 ucicode = r[COL_UCICODE].decode('utf-8')
+                note = r[COL_NOTE].decode('utf-8')
                 rmap = {}
                 rmap[u'bib']=key
                 rmap[u'first']=first
@@ -295,6 +346,7 @@ class riderdb(object):
                 rmap[u'cat']=cat
                 rmap[u'refid']=refid
                 rmap[u'ucicode']=ucicode
+                rmap[u'note']=note
                 rmap[u'namestr']=strops.resname(first, last, club)
                 rmap[u'name']=strops.resname(first, last)
                 ret[key]=rmap
@@ -451,14 +503,15 @@ class riderdb(object):
         return ret
 
     def editrider(self, ref=None, first=None, last=None,
-                  club=None, cat=None, refid=None, ucicode=None, bib=None):
+                  club=None, cat=None, refid=None, ucicode=None, 
+                  note=None, bib=None):
         """Create or update the rider with supplied parameters."""
         i = None
         if ref is None:
             if bib is None:
                 bib = self.nextriderno()
             ## ERROR: num not ever defined?
-            i = self.model.append([bib, u'', u'', u'', u'', u'', u'', u''])
+            i = self.model.append([bib, u'', u'', u'', u'', u'', u'', u'', u''])
             ref = gtk.TreeRowReference(self.model,
                                            self.model.get_path(i))
         if ref.valid():
@@ -475,6 +528,8 @@ class riderdb(object):
                 self.model.set_value(i, COL_REFID, refid)
             if ucicode is not None:
                 self.model.set_value(i, COL_UCICODE, ucicode)
+            if note is not None:
+                self.model.set_value(i, COL_NOTE, note)
         return ref
 
     def filter_nonempty(self, col=None):
@@ -655,10 +710,20 @@ class riderdb(object):
                                    gobject.TYPE_STRING, # 4 category
                                    gobject.TYPE_STRING, # 5 series
                                    gobject.TYPE_STRING, # 6 refid
-                                   gobject.TYPE_STRING) # 7 ucicode
+                                   gobject.TYPE_STRING, # 7 ucicode
+                                   gobject.TYPE_STRING) # 8 note
         self.tagmap = {}
         self.maptag = {}
         self.view = None
         self.colvec = []
         self.postedit = None
         self.editwasempty = False
+
+class listofentry(object):
+    """Load and manage a MR style list of entries."""
+    def __init__(self):
+        __model = {}
+        __event_name = None
+       
+    
+
