@@ -53,6 +53,7 @@ LOGHANDLER_LEVEL = logging.DEBUG
 ROADRACE_TYPES = {u'irtt':u'Road Time Trial',
                   u'rms':u'Road Race',
                   u'trtt':u'Team Road Time Trial',
+                  u'crit':u'Criterium/Kermesse',
                   u'rhcp':u'Handicap',
                   u'sportif':u'Sportif Ride',
                   u'cross':u'Cyclocross Race'}
@@ -270,7 +271,7 @@ class fakemeet:
             with open(cwfilename, 'rb') as f:
                 cr.read(f)
         except Exception as e:
-            self.log.error(u'Reading meet config: ' + unicode(e))
+            print(u'Error reading meet config: ' + unicode(e))
         # set meet meta, and then copy into text entries
         self.shortname = cr.get(u'roadmeet', u'shortname')
         self.title_str = cr.get(u'roadmeet', u'title')
@@ -310,9 +311,12 @@ class roadmeet:
         response = dlg.run()
         if response == gtk.RESPONSE_OK:
             self.configpath = dlg.get_filename()
-            self.loadconfig()
-            self.log.info(u'Meet data loaded from'
-                           + repr(self.configpath) + u'.')
+            try:
+                self.loadconfig()
+                self.log.info(u'Meet data loaded from'
+                               + repr(self.configpath) + u'.')
+            except Exception as e:
+                self.log.error(u'Open event: ' + repr(e))
         else:
             self.log.debug(u'Load new meet cancelled.')
         dlg.destroy()
@@ -345,11 +349,11 @@ class roadmeet:
         dotype = False
         if self.etype in ROADRACE_TYPES:
             tlbl = ROADRACE_TYPES[self.etype]
-        if self.etype in [u'rms', u'rhcp']:
+        if self.etype in [u'rms', u'rhcp', u'crit']:
             # make the combo so switcheroo is possible
             dotype = True
             cnt = 0
-            for t in [u'rms', u'rhcp']:
+            for t in [u'rms', u'rhcp', u'crit']:
                 tmodel.append([t, ROADRACE_TYPES[t]])
                 if t == self.etype:
                     tcombo.set_active(cnt)
@@ -380,14 +384,9 @@ class roadmeet:
         o_ent.set_text(self.organiser_str)
         c_ent = b.get_object('commissaire_entry')
         c_ent.set_text(self.commissaire_str)
-        l_ent = b.get_object('logo_entry')
-        l_ent.set_text(self.logo)
         di_ent = b.get_object('distance_entry')
         if self.distance is not None:
             di_ent.set_text(str(self.distance))
-        n_opt = b.get_object('resultno_opt')
-        n_opt.set_active(self.bibs_in_results)
-        
         upe = b.get_object('uscbsrv_host_entry')
         upe.set_text(self.uscbport)
         uce = b.get_object('uscbsrv_channel_entry')
@@ -406,7 +405,7 @@ class roadmeet:
         ate.set_text(self.alttimer_port)
         atb = b.get_object('timing_alt_dfl')
         atb.connect('clicked',
-                    lambda b: ate.set_text(timy.DEFPORT))
+                    lambda b: ate.set_text(timy.BACKUPPORT))
         response = dlg.run()
         if response == 1:	# id 1 set in glade for "Apply"
             self.log.debug(u'Updating meet properties.')
@@ -416,10 +415,8 @@ class roadmeet:
             self.date_str = d_ent.get_text().decode('utf-8')
             self.organiser_str = o_ent.get_text().decode('utf-8')
             self.commissaire_str = c_ent.get_text().decode('utf-8')
-            self.logo = l_ent.get_text().decode('utf-8')
             self.distance = strops.confopt_float(
                                 di_ent.get_text().decode('utf-8'))
-            self.bibs_in_results = n_opt.get_active()
             self.remote_enable = uoc.get_active()
             self.set_title()
 
@@ -551,6 +548,38 @@ class roadmeet:
             self.curevent.readonly = True
         self.close_event()
 
+    def menu_race_armstart_activate_cb(self, menuitem, data=None):
+        """Default armstart handler."""
+        self.log.info(u'Arm Start')
+        try:
+            self.curevent.armstart()
+        except Exception as e:
+            self.log.error('Arm start - ' + repr(e))
+
+    def menu_race_armlap_activate_cb(self, menuitem, data=None):
+        """Default armlap handler."""
+        self.log.info(u'Arm Lap')
+        try:
+            self.curevent.armlap()
+        except Exception as e:
+            self.log.error('Arm lap - ' + repr(e))
+
+    def menu_race_armfin_activate_cb(self, menuitem, data=None):
+        """Default armfin handler."""
+        self.log.info(u'Arm Finish')
+        try:
+            self.curevent.armfinish()
+        except Exception as e:
+            self.log.error('Arm finish - ' + repr(e))
+
+    def menu_race_finished_activate_cb(self, menuitem, data=None):
+        """Default finished handler."""
+        self.log.info(u'Finished')
+        try:
+            self.curevent.set_finished()
+        except Exception as e:
+            self.log.error('Set finished - ' + repr(e))
+
     def open_event(self, eventhdl=None):
         """Open provided event handle."""
         if eventhdl is not None:
@@ -613,6 +642,9 @@ class roadmeet:
                 self.print_report(sections)
             else:
                 self.log.info('Startlist - Nothing to print.')
+            if not self.curevent.readonly:
+                # emit starters to announcer
+                self.scb.send_cmd(u'startlist', self.curevent.get_startlist())
 
     def menu_reports_signon_activate_cb(self, menuitem, data=None):
         """Generate a sign on sheet."""
@@ -703,6 +735,37 @@ class roadmeet:
         else:
             self.log.debug(u'Replay RF data cancelled.')
 
+    def menu_import_lif_activate_cb(self, menuitem, data=None):
+        """Import a LIF file."""
+        if self.curevent is None:
+            self.log.info(u'No event open to import to.')
+            return
+        sfile = uiutil.loadcsvdlg(u'Select LIF file to import',
+                                  self.window)
+        if os.path.isfile(sfile):
+            with open(sfile, 'rb') as f:
+                cr = ucsv.UnicodeReader(f)
+                ltime = None
+                count = 0
+                for r in cr:
+                    if len(r) > 8:
+                        if r[2] == u'':	# rider record
+                            if r[1].isalnum() and r[1] != u'No.':
+                                bib = r[1].strip().lower()
+                                rank = strops.confopt_posint(r[0], None)
+                                if bib and rank is not None:
+                                    rtime = tod.str2tod(r[6])
+                                    dtime = tod.str2tod(r[8])
+                                    if dtime is not None:
+                                        # this is a new bunch
+                                        ltime = rtime
+                                    self.curevent.setriderval(bib, rank, ltime)
+                                    count += 1
+                self.log.info(u'Loaded ' + unicode(count) + u' riders from '
+                                    + repr(sfile))
+        else:
+            self.log.debug(u'Import startlist cancelled.')
+
     ## TODO: repair the start time import for all event types
     ##       fix the series ambiguity in all event types
     def menu_import_startlist_activate_cb(self, menuitem, data=None):
@@ -802,13 +865,7 @@ class roadmeet:
             tfile = None
         return tfile
 
-    def menu_data_results_cb(self, menuitem, data=None):
-        """Create live result report and export to MR"""
-        self.saveconfig()
-        self.log.info(u'Meet data saved to ' + repr(self.configpath) + u'.')
-        if self.curevent is None:
-            return
-
+    def export_result_maker(self):
         # 1: make report with meta from event and meet
         tfile = self.default_template()
         rep = printing.printrep(template=tfile, path=self.configpath)
@@ -834,6 +891,8 @@ class roadmeet:
         # 3: set provisional status	# TODO: other tests for prov flag?
         if self.curevent.timerstat != 'finished':
             rep.set_provisional(True)
+        else:
+            rep.reportstatus = u'final'	# TODO: write in other phases
 
         # 4: call into curevent to get result sections
         ## TODO: if status is 'new' write out a startlist
@@ -866,6 +925,10 @@ class roadmeet:
             #filebase += u'_result'
 
         ofile = os.path.join(exportpath, filebase+u'.pdf')
+        # HACK: metarace site should be in config
+        lb = os.path.join(u'http://metarace.com.au/site',
+                            self.mirrorpath, filebase)
+        rep.canonical = u'.'.join([lb, u'pdf'])
         with open(ofile, 'wb') as f:
             rep.output_pdf(f)
         ofile = os.path.join(exportpath, filebase+u'.xls')
@@ -874,13 +937,22 @@ class roadmeet:
         ofile = os.path.join(exportpath, filebase+u'.json')
         with open(ofile, 'wb') as f:
             rep.output_json(f)
-        lb = os.path.join(u'/site', self.mirrorpath, filebase)
         lt = [u'pdf', u'xls', u'txt', u'json']
         ofile = os.path.join(exportpath, filebase+u'.txt')
         with open(ofile, 'wb') as f:
             rep.output_text(f, linkbase=lb, linktypes=lt)
 
-        # 6: if not in progress, create and trigger rsync proc
+    def menu_data_results_cb(self, menuitem, data=None):
+        """Create live result report and export to MR"""
+        self.saveconfig()
+        self.log.info(u'Meet data saved to ' + repr(self.configpath) + u'.')
+        if self.curevent is None:
+            return
+
+        # only build reports if the race is really stand-alone
+        if self.etype in [u'sportif'] or self.mirrorcmd == u'rsync':
+            self.export_result_maker()
+
         glib.idle_add(self.mirror_start)
         self.log.info(u'Race info export.')
 
@@ -945,12 +1017,20 @@ class roadmeet:
     def menu_timing_reconnect_activate_cb(self, menuitem, data=None):
         """Reconnect timer and re-start."""
         (cdev, cport) = rfid_device(self.timer_port)
+        self.log.debug(u'REPORT')
         self.timer.setport(cport)	# force reconnect
+        self.log.debug(u'STOP')
         self.timer.stop_session()
+        self.log.debug(u'SANE')
         self.timer.sane()
+        self.log.debug(u'START')
         self.timer.start_session()
         self.alttimer.setport(self.alttimer_port)
         self.alttimer.sane()
+        ## TODO: move this and the others into method, and include
+        ## all road-ish events: 'rhcp', 'rms', None
+        if self.etype == 'rms':	# road race finish requires 1.0s delay
+            self.alttimer.write('DTF01.00')
         self.log.info(u'Re-connect/re-start attached timer.')
 
     ## Help menu callbacks
@@ -1029,6 +1109,7 @@ class roadmeet:
         self.log.removeHandler(self.sh)
         self.log.removeHandler(self.lh)
         self.window.hide()
+        metarace.unlockpath(self.configpath)
         self.log.info(u'Meet destroyed. ' + msg)
         glib.idle_add(self.meet_destroy_handler)
 
@@ -1157,6 +1238,11 @@ class roadmeet:
 
     def loadconfig(self):
         """Load meet config from disk."""
+        if not metarace.lockpath(self.configpath):
+            self.log.error(u'Unable to get lock on configpath.')
+            raise Exception(u'Unable to get lock on configpath.')
+            # but this causes LSP drama
+
         cr = jsonconfig.config({u'roadmeet':{
                u'shortname':None,
                u'title':u'',
@@ -1169,7 +1255,7 @@ class roadmeet:
                u'distance':u'',
                u'docindex':u'0',
                u'timer':u'',
-               u'rfidlvl':[u'70', u'70'],
+               u'rfidlvl':[u'30', u'30'],
                u'alttimer':u'',
                u'resultnos':u'Yes',
                u'uscbport':u'',
@@ -1192,8 +1278,7 @@ class roadmeet:
         self.loghandler = logging.FileHandler(
                              os.path.join(self.configpath, LOGFILE))
         self.loghandler.setLevel(LOGHANDLER_LEVEL)
-        self.loghandler.setFormatter(logging.Formatter(
-                       '%(asctime)s %(levelname)s:%(name)s: %(message)s'))
+        self.loghandler.setFormatter(logging.Formatter(metarace.LOGFORMAT))
         self.log.addHandler(self.loghandler)
 
         # check for config file
@@ -1235,6 +1320,8 @@ class roadmeet:
             self.alttimer_port = nport
             self.alttimer.setport(nport)
             self.alttimer.sane() # sane prod here is probably good idea
+            if self.etype == 'rms':	# road race finish requires 1.0s delay
+                self.alttimer.write('DTF01.00')
 
         # set meet meta, and then copy into text entries
         self.shortname = cr.get(u'roadmeet', u'shortname')
@@ -1268,6 +1355,10 @@ class roadmeet:
         self.open_event(event) # always open on load if posible
         self.set_title()
 
+        # timerconfig post event load
+        if self.etype == 'rms':	# road race finish requires 1.0s delay
+            self.alttimer.write('DTF01.00')
+
         # restore printer preferences
         psfilename = os.path.join(self.configpath, u'print.prf')
         if os.path.isfile(psfilename):
@@ -1291,11 +1382,18 @@ class roadmeet:
         """Return race distance in km."""
         return self.distance
 
-    def announce_timer(self, evt):
+    def announce_timer(self, evt, timer=None):
         """Send message into announce for remote control."""
         if not self.remote_enable:
-            tvec = [unicode(1), self.timer.unitno, evt.chan,
-                    evt.refid, evt.rawtime()]
+            if timer is None:
+                timer = self.timer
+            prec = 4
+            if timer is self.timer:
+                prec = 2	# transponders only read to hundredths
+            elif u'M' in evt.chan:
+                prec = 2
+            tvec = [evt.index, timer.unitno, evt.chan,
+                    evt.refid, evt.rawtime(prec)]
             self.scb.send_cmd(u'timer',unichr(unt4.US).join(tvec),
                                        self.timerchan)
 
@@ -1400,18 +1498,27 @@ class roadmeet:
             tv = cmd.text.split(unichr(unt4.US)) # split into fields
             if len(tv) > 4:	# enough fields
                 try:
-                    index = int(tv[0])	# record index
+                    index = 0
+                    if tv[0].isdigit():
+                        index = int(tv[0])	# record index
                     ## TODO: handle discontinuity
                     srcid = tv[1]
                     self.log.debug(u'Remote Timer: ' + repr(srcid)
                                    + u' index:' + repr(index) + u' '
                                    + tv[2] + u'/' + tv[3] + u'@' + tv[4])
-                    self.timer.trig(timeval=tv[4], index='REM',
-                                    chan=tv[2], refid=tv[3])
+                    if u'timy' in srcid:
+                        self.alttimer.trig(timeval=tv[4], index=tv[0],
+                                    chan=tv[2], refid=tv[3], sourceid=srcid)
+                    else:
+                        self.timer.trig(timeval=tv[4], index='REM',
+                                    chan=tv[2], refid=tv[3], sourceid=srcid)
                 except Exception as e:
                     self.log.error('Reading timer msg: ' + repr(e))
         else:
-            self.log.debug(u'Unknown remote cmd ' + repr(nick) + u': '
+            if cmd.header == u'rfidstatus':
+                self.menu_rfustat_clicked_cb(None)
+            else:
+                self.log.debug(u'Unknown remote cmd ' + repr(nick) + u': '
                        + repr(cmd.header) + u'::' + repr(cmd.text))
 
     def __init__(self, configpath=None, etype=None):
@@ -1581,7 +1688,7 @@ def main(etype=None):
     if configpath is not None:
         os.chdir(configpath)
     app = roadmeet(configpath, etype)
-    app.loadconfig()
+    app.loadconfig()	# exception here ok
     app.window.show()
     app.start()
     try:
