@@ -162,6 +162,78 @@ class rr_announce(object):
             return
         cr = self.map_src.cairo_create()
 
+        # if there are riders in the map, do a bunches map
+        if len(self.riders) > 0:
+            self.draw_ridermap(cr)
+        else: # else draw the profile
+            self.draw_profile(cr)
+
+    def draw_profile(self, cr):
+        width = self.map_winsz
+        height = 80
+        cr.identity_matrix()
+
+        # bg filled
+        cr.set_source_rgb(0.85,0.85,0.9)
+        cr.paint()
+        if self.profile is None:
+             return
+
+        dw = width - (2 * MAPHMARGIN)
+        dh = height - (2 * MAPVMARGIN)
+
+        cr.set_line_width(1.0)
+
+        xmod = 1
+
+        xof = 0
+        cnt = 0
+        while xof < dw and float(cnt) < self.dmax:
+            lh = 4
+            if cnt % 10000 == 0:
+                lh = 12
+                cr.set_source_rgb(0.05,0.05,0.05)
+                cr.move_to(xof+MAPHMARGIN+1,
+                           MAPVMARGIN+dh-lh-2)
+                cr.show_text(unicode(cnt//1000)+'km')
+            elif cnt % 5000 == 0:
+                lh = 8
+            cr.set_source_rgb(0.05,0.05,0.05)
+            cr.move_to(xof+MAPHMARGIN, MAPVMARGIN+dh-lh)
+            cr.line_to(xof+MAPHMARGIN, MAPVMARGIN+dh)
+            cr.stroke()
+            if cnt % 10000 == 0:
+                cr.set_source_rgb(0.96,0.96,0.96)
+                cr.move_to(xof+MAPHMARGIN, MAPVMARGIN)
+                cr.line_to(xof+MAPHMARGIN, MAPVMARGIN+dh-lh-2)
+                cr.stroke()
+            cnt += 5000
+            xof =dw*(float(cnt)/self.dmax)
+
+        cr.set_source_rgb(0.05,0.05,0.95)
+        xo = MAPHMARGIN
+        zr = self.zmax - self.zmin
+        yo = dh*((self.profile[0][1]-self.zmin)/zr)
+        cr.move_to(xo, MAPVMARGIN+dh-yo)
+        for r in self.profile:
+            xp = xo + dw*r[0]
+            yz = dh*((r[1]-self.zmin)/zr)
+            cr.line_to(xp,MAPVMARGIN+dh-yz)
+
+        cr.stroke()
+        if self.distance is not None:
+           drem = 1000.0*self.distance
+           xof = (self.dmax-drem)/self.dmax
+           if xof < 1.0:
+               xp = MAPHMARGIN + dw*xof
+               cr.set_source_rgb(0.95,0.05,0.05)
+               cr.move_to(xp,MAPVMARGIN)
+               cr.line_to(xp,MAPVMARGIN+dh)
+               cr.stroke()
+        
+        
+
+    def draw_ridermap(self, cr):
         width = self.map_winsz
         height = 80
         cr.identity_matrix()
@@ -354,6 +426,7 @@ class rr_announce(object):
                 a = float(m.text)
                 if a > 0.1:
                     self.distance = a
+                    self.redraw_flag = True
             except:
                 self.log.debug(u'Invalid distance: ' + repr(m.text))
         elif m.header == u'timerstat':
@@ -373,6 +446,7 @@ class rr_announce(object):
                                     u'autoscroll':False,
                                     u'timetick':12,
                                     u'fontsize':20,
+                                    u'minelevrange':300.0,
                                     u'fullscreen':False,
                                     u'groupcol':True,
                                     u'catcol':True,
@@ -449,6 +523,58 @@ class rr_announce(object):
         self.search_entry.modify_font(pango.FontDescription(FONTFACE + ' ' + fnszstr))
         self.lbl_resultmsg.modify_font(pango.FontDescription(FONTFACE + ' ' + fnszstr))
         self.view.modify_font(pango.FontDescription(FONTFACE + ' '+fnszstr))
+        zrangemin = strops.confopt_float(cr.get(u'road_announce',
+                                                u'minelevrange'),200.0)
+        routefile = metarace.default_file(u'route.dat')
+        self.profile = None
+        self.zmin = None
+        self.zmax = None
+        self.dmax = None
+        if os.path.exists(routefile):
+            print(u'loading profile...')
+            self.profile = []
+            with open(routefile, 'rb') as f:
+                dmax = 0.0
+                xmin = -1.0
+                xmax = 1.0
+                ymin = -1.0
+                ymax = 1.0
+                zmin = 90000.0
+                zmax = -100.0
+                for l in f:
+                    r = l.split()
+                    t = strops.confopt_float(r[1])
+                    x = strops.confopt_float(r[2])
+                    y = strops.confopt_float(r[3])
+                    z = strops.confopt_float(r[4])
+                    d = strops.confopt_float(r[6])
+                    if z < zmin:
+                        zmin = z
+                    if z > zmax:
+                        zmax = z
+                    if x < xmin:
+                        xmin = x
+                    if x > xmax:
+                        xmax = x
+                    if y < ymin:
+                        ymin = y
+                    if y > ymax:
+                        ymax = y
+                    if d > dmax:
+                        dmax = d
+                    self.profile.append([t,z,d,x,y]) 
+                self.zmin = zmin
+                self.zmax = zmax
+                self.xmin = xmin
+                self.xmax = xmax
+                self.ymin = ymin
+                self.ymax = ymax
+                self.dmax = dmax
+                erange = zmax - zmin	# profile min elevation range
+                if erange < zrangemin:
+                    self.zmin -= 0.5*(zrangemin - erange)
+                    self.zmax = self.zmin + zrangemin
+                print(repr([zmin,zmax,dmax]))
 
     def doautoscroll(self):
         """Automatically handle smooth scroll in text view after delay."""
@@ -539,16 +665,17 @@ class rr_announce(object):
         self.elap_lbl.set_text(estr)
         self.gap_lbl.set_text(dstr)
 
-        if self.timerstat == 'running' and self.laplbl:
-            tmsg = u''
-            if self.finstr:
-                tmsg = self.finstr + u' '
-            # add lap and sprint info
-            tmsg += self.laplbl
-            if self.laptype:
-                tmsg += u' - ' + self.laptype
-            self.lbl_header.set_text(tmsg)
-        else:
+        #if self.timerstat == 'running' and self.laplbl:
+            #tmsg = u''
+            #if self.finstr:
+                #tmsg = self.finstr + u' '
+            ## add lap and sprint info
+            #tmsg += self.laplbl
+            #if self.laptype:
+                #tmsg += u' - ' + self.laptype
+            #self.lbl_header.set_text(tmsg)
+        #else:
+        if True:	# always show the title_str, it now has better info
             if self.title_str:
                 self.lbl_header.set_text(self.title_str)
 
@@ -628,6 +755,15 @@ class rr_announce(object):
         self.cur_bunchid = 0
         self.cur_bunchcnt = 0
 
+        self.profile = None
+        self.zmin = None
+        self.zmax = None
+        self.xmin = None
+        self.xmax = None
+        self.ymin = None
+        self.ymax = None
+        self.dmax = None
+
         self.riders = gtk.ListStore(gobject.TYPE_STRING,  # rank
                                     gobject.TYPE_STRING,  # no.
                                     gobject.TYPE_STRING,  # namestr
