@@ -57,10 +57,9 @@ from metarace import strops
 from metarace import unt4
 from metarace import tod
 
-PAGE_INIT=10		# count before table starts displaying
+PAGE_INIT=10		# delay before table data starts displaying
 PAGE_DELAY=50		# def tenths of sec to hold each page of table
-PAGE_ROWOFT=0		# first DHI row for table data
-SP_PAGE_ROWOFT=0	# first DHI row for table data
+PAGE_ROWOFT=2		# first DHI row for table data
 DATE_FMT=('%a %d/%m/%y')
 
 def get_dateline(width=32):
@@ -128,30 +127,23 @@ class scbclock(scbwin):
            CENTERED LINE 3
 
     """
-    def __init__(self, scb=None, line1=u'', line2=u'', line3=u''):
+    def __init__(self, scb=None, line1=u'', line2=u'', line3=u'',locstr=u''):
         scbwin.__init__(self, scb)
         self.line1 = line1
         self.line2 = line2
         self.line3 = line3
+        self.locstr = locstr
         self.bodyoft = 1 # body text offset
         if self.scb.pagelen > 4:
             self.bodyoft = 2
         self.header = get_dateline(self.scb.linelen)
-        self.tovr = unt4.OVERLAY_4LINE
-        if self.line3 == u'':
-            if self.line2 == u'':
-                if self.line1 == u'':
-                    self.tovr = unt4.OVERLAY_1LINE
-                else:
-                    self.tovr = unt4.OVERLAY_2LINE
-            else:
-                self.tovr = unt4.OVERLAY_3LINE
+
     def redraw(self):
-        for i in range(0,self.scb.pagelen):
-            self.scb.clrline(i)
-        #self.scb.setoverlay(self.tovr)
-        self.scb.setline(0, self.header)
-        #self.scb.flush()
+        self.scb.setline(0, self.header,chan=u'#anascb')
+        for i in range(1,self.scb.pagelen):
+            self.scb.clrline(i,chan=u'#anascb')
+        self.scb.setoverlay(unt4.OVERLAY_MATRIX,chan=u'#anascb')
+        self.scb.flush()
 
     def update(self):
         """Animate the clock window.
@@ -167,20 +159,24 @@ class scbclock(scbwin):
             flush = False
             if self.count == 14:
                 self.scb.setline(self.bodyoft,
-                                 self.line1.strip().center(self.scb.linelen))
+                                 self.line1.strip().center(self.scb.linelen),chan=u'#anascb')
                 flush = True
             if self.count == 16:
                 self.scb.setline(self.bodyoft+1,
-                                 self.line2.strip().center(self.scb.linelen))
+                                 self.line2.strip().center(self.scb.linelen),chan=u'#anascb')
                 flush = True
             if self.count == 18:
                 self.scb.setline(self.bodyoft+2,
-                                 self.line3.strip().center(self.scb.linelen))
+                                 self.line3.strip().center(self.scb.linelen),chan=u'#anascb')
+                flush = True
+            if self.count == 20:
+                self.scb.setline(self.bodyoft+4,
+                                 self.locstr.strip().center(self.scb.linelen),chan=u'#anascb')
                 flush = True
             if self.count % 2 == 0:
                 next = get_dateline(self.scb.linelen)
                 if next != self.header:
-                    self.scb.setline(0, next)
+                    self.scb.setline(0, next,chan=u'#anascb')
                     flush = True
                     self.header = next
             self.count += 1
@@ -226,12 +222,16 @@ class scbtt(scbwin):
         self.singleoft = 1
 
     def redraw(self):
-        for i in range(0,10):
-            self.scb.clrline(i)
-        #self.scb.setoverlay(unt4.OVERLAY_R1P4)
-        self.scb.setline(0, self.header)
-        self.scb.setline(1+self.singleoft, self.line1)
-        self.scb.setline(3, self.line2)
+        clrset = list(range(0,self.scb.pagelen))
+        self.scb.setline(0, self.header,chan=u'#anascb')
+        clrset.remove(0)
+        self.scb.setline(2+self.singleoft, self.line1,chan=u'#anascb')
+        clrset.remove(2+self.singleoft)
+        self.scb.setline(4+self.singleoft, self.line2,chan=u'#anascb')
+        clrset.remove(4+self.singleoft)
+        # only clear rows not already set above.
+        for i in clrset:
+            self.scb.clrline(i,chan=u'#anascb')
         self.nextline2 = self.line2
         self.curt1 = u''
         self.nextt1 = u''
@@ -241,6 +241,7 @@ class scbtt(scbwin):
         self.nextt2 = u''
         self.curr2 = u''
         self.nextr2 = u''
+        self.scb.setoverlay(unt4.OVERLAY_MATRIX,chan=u'#anascb')
         self.scb.flush()
 
     def setline2(self, line2str=u''):
@@ -268,23 +269,23 @@ class scbtt(scbwin):
         if not self.paused:
             flush = False
             if self.curr1 != self.nextr1 or self.curt1 != self.nextt1:
-                self.scb.setline(2+self.singleoft,
+                self.scb.setline(3+self.singleoft,
                       strops.truncpad(self.nextr1, self.scb.linelen - 13,
-                                      'r') + u' ' + self.nextt1)
+                                      'r') + u' ' + self.nextt1,chan=u'#anascb')
                 self.curr1 = self.nextr1
                 self.curt1 = self.nextt1
                 flush = True
             if self.curr2 != self.nextr2 or self.curt2 != self.nextt2:
-                self.scb.setline(4,
+                self.scb.setline(5,
                       strops.truncpad(self.nextr2, self.scb.linelen - 13,
-                                      'r') + u' ' + self.nextt2)
+                                      'r') + u' ' + self.nextt2,chan=u'#anascb')
                 self.curr2 = self.nextr2
                 self.curt2 = self.nextt2
                 flush = True
             if self.line2 != self.nextline2:
                 # A hack to allow display of speed string at single finish
                 self.line2 = self.nextline2
-                self.scb.setline(3, self.nextline2)
+                self.scb.setline(4, self.nextline2,chan=u'#anascb')
                 flush = True
             self.count += 1
             if flush:
@@ -320,17 +321,22 @@ class scbtimer(scbwin):
         self.nextavg = u''
 
     def redraw(self):
-        for i in range(0,10):
-            self.scb.clrline(i)
-        #self.scb.setoverlay(unt4.OVERLAY_R2P2)
-        self.scb.setline(0, self.line1.strip().center(self.scb.linelen))
-        self.scb.setline(1, self.line2.strip().center(self.scb.linelen))
-        self.scb.setline(3, strops.truncpad(self.timepfx, 
-                             self.scb.linelen - 13, 'r'))
+        clrset = list(range(0,self.scb.pagelen))
+        self.scb.setline(0, self.line1.strip().center(self.scb.linelen),chan=u'#anascb')
+        clrset.remove(0)
+        self.scb.setline(1, self.line2.strip().center(self.scb.linelen),chan=u'#anascb')
+        clrset.remove(1)
+        #self.scb.setline(3, strops.truncpad(self.timepfx, 
+                             #self.scb.linelen - 13, 'r'),chan=u'#anascb')
+        #clrset.remove(3)
+        for i in clrset:
+            self.scb.clrline(i,chan=u'#anascb')
+
         self.curtime = u''
         self.nexttime = u''
         self.curavg = u''
         self.nextavg = u''
+        self.scb.setoverlay(unt4.OVERLAY_MATRIX,chan=u'#anascb')
         self.scb.flush()
 
     def settime(self, timestr=u''):
@@ -346,14 +352,19 @@ class scbtimer(scbwin):
         if not self.paused:
             flush = False
             if self.curtime != self.nexttime:
-                self.scb.postxt(3, self.scb.linelen - 12,
-                                  self.nexttime)
+                #self.scb.postxt(3, self.scb.linelen - 13,
+                     #strops.truncpad(self.nexttime,12,'r'), chan=u'#anascb')
+                self.scb.setline(3,
+                  strops.truncpad(self.timepfx, self.scb.linelen - 13, 'r')
+                  +strops.truncpad(self.nexttime, 13, 'r'),
+                  chan=u'#anascb')
                 self.curtime = self.nexttime
                 flush = True
             if self.curavg != self.nextavg:
                 self.scb.setline(4,
-                      strops.truncpad(self.avgpfx, self.scb.linelen - 13,
-                                       'r') + u' ' + self.nextavg)
+                  strops.truncpad(self.avgpfx, self.scb.linelen - 13, 'r')
+                  +strops.truncpad(self.nextavg, 12, 'r'),
+                  chan=u'#anascb')	# timestr above provides 4places?
                 self.curavg = self.nextavg
                 flush = True
             self.count += 1
@@ -380,10 +391,10 @@ class scbintro(scbwin):
     
     """
     def redraw(self):
-        for i in range(0,10):
-            self.scb.clrline(i)
-        #self.scb.setoverlay(self.tovr)
-        self.scb.setline(0, self.header)
+        self.scb.setline(0, self.header, chan=u'#anascb')
+        for i in range(1,self.scb.pagelen):
+            self.scb.clrline(i,chan=u'#anascb')
+        self.scb.setoverlay(unt4.OVERLAY_MATRIX,chan=u'#anascb')
         self.scb.flush()
 
     def update(self):
@@ -392,15 +403,15 @@ class scbintro(scbwin):
             msgstart = PAGE_INIT + 6
             msgend = msgstart + 25
             if self.count == PAGE_INIT:	# draw prompt at ~+0.5s
-                self.scb.setline(2,self.prompt)
+                self.scb.setline(2,self.prompt,chan=u'#anascb')
                 flush = True
             elif self.count >= msgstart and self.count < msgend:
                 oft = self.count-msgstart
                 if len(self.info) > oft:
-                    self.scb.postxt(3, oft, self.info[oft])
+                    self.scb.postxt(3, oft, self.info[oft], chan=u'#anascb')
                     flush = True
             elif self.count == msgend:
-                self.scb.setline(3, self.info)
+                self.scb.setline(3, self.info,chan=u'#anascb')
                 flush = True
                 self.paused = True
             self.count += 1
@@ -420,10 +431,8 @@ class scbintro(scbwin):
         self.prompt = u''
         self.info = u''
         self.xtra = u''
-        self.tovr = unt4.OVERLAY_3LINE
         if lines != 5:
             self.lines = 7
-            self.tovr = unt4.OVERLAY_4LINE
 
 class logoanim(scbwin):
     """Animated 'logo' screen."""
@@ -435,7 +444,7 @@ class logoanim(scbwin):
         if not self.paused:
             if self.count % self.delay == 0:
                  dbline = 18
-                 #overlay = unt4.OVERLAY_IMAGE
+                 overlay = unt4.OVERLAY_IMAGE
                  # Alternate overlay
                  if self.curpg == 0:
                      self.curpg = 1
@@ -449,10 +458,10 @@ class logoanim(scbwin):
                  if self.llist[curidx] == u'CLOCK':
                      overlay = unt4.OVERLAY_CLOCK
                  else:
-                     self.scb.setline(dbline, self.llist[curidx]) # set img
+                     self.scb.setline(dbline, self.llist[curidx],chan=u'#anascb') # set img
                  
                  # select overlay
-                 #self.scb.setoverlay(overlay)
+                 self.scb.setoverlay(overlay,chan=u'#anascb')
                  if len(self.llist) == 1:
                      self.paused = True
             self.count += 1
@@ -474,8 +483,9 @@ class logoanim(scbwin):
 class scbtest(scbwin):
     """A "test pattern" that loops over all the overlays."""
     def redraw(self):
-        for i in range(0,10):
-            self.scb.setline(i, u'Line ' + unicode(i)) 
+        for i in range(0,self.scb.pagelen):
+            self.scb.setline(i, u'Line ' + unicode(i),chan=u'#anascb') 
+        self.scb.setoverlay(unt4.OVERLAY_MATRIX,chan=u'#anascb')
         self.scb.flush()
 
     def update(self):
@@ -487,8 +497,10 @@ class scbtest(scbwin):
 
     def __init__(self, scb=None, head=u'', lines=5):
         scbwin.__init__(self, scb)
-        self.ovrs = [unt4.OVERLAY_4LINE, unt4.OVERLAY_R2P4, unt4.OVERLAY_24X6,
-                     unt4.OVERLAY_IMAGE]
+        self.ovrs = [unt4.OVERLAY_CLOCK,
+                     unt4.OVERLAY_MATRIX,
+                     unt4.OVERLAY_IMAGE,
+                     unt4.OVERLAY_BLANK]
         self.curov = 0
 
 
@@ -505,7 +517,7 @@ class scbintsprint(scbwin):
                 nr = u''
                 oft = 0
                 for col in coldesc:
-                    if type(col) is unicode:	# assume all cols uni
+                    if type(col) in [unicode, str]:
                         nr += col
                     else:
                         if len(row) > oft:	# space pad 'short' rows
@@ -523,11 +535,11 @@ class scbintsprint(scbwin):
             self.rows.insert(len(self.rows) - 2, u' ')
 
     def redraw(self):
-        for i in range(0,10):
-            self.scb.clrline(i)
-        #self.scb.setoverlay(self.tovr)
-        #self.scb.setline(0, self.line1)
-        self.scb.setline(0, self.line2)	# for NZL -> roll line 2 up
+        self.scb.setline(0, self.line1.strip().center(self.scb.linelen),chan=u'#anascb')
+        self.scb.setline(1, self.line2.strip().center(self.scb.linelen),chan=u'#anascb')
+        for i in range(2,self.scb.pagelen):
+            self.scb.clrline(i,chan=u'#anascb')
+        self.scb.setoverlay(unt4.OVERLAY_MATRIX,chan=u'#anascb')
         self.scb.flush()
 
     def update(self):
@@ -538,7 +550,7 @@ class scbintsprint(scbwin):
             flush = False
             if pclk < self.pagesz + 1:
                 if pclk != self.pagesz:
-                    self.scb.clrline(SP_PAGE_ROWOFT + pclk)
+                    self.scb.clrline(self.rowoft + pclk,chan=u'#anascb')
                     flush = True
                 elif self.nrpages == 1:
                     self.count += 1
@@ -546,8 +558,8 @@ class scbintsprint(scbwin):
                 if pclk != 0:
                     roft = self.pagesz * cpage + pclk-1
                     if roft < len(self.rows):
-                        self.scb.setline(SP_PAGE_ROWOFT + pclk-1,
-                                         self.rows[roft])
+                        self.scb.setline(self.rowoft + pclk-1,
+                                         self.rows[roft],chan=u'#anascb')
                         flush = True
             if flush:
                 self.scb.flush()
@@ -560,17 +572,16 @@ class scbintsprint(scbwin):
         self.pagesz = 4
         self.nrpages = 0
         self.delay = delay
-        self.tovr = unt4.OVERLAY_R2P4
+        self.rowoft = 3		# check for < 7
 
-        # prepare header -> must be preformatted
-        self.line1 = line1[0:25]
-        self.line2 = line2[0:25]
+        # prepare header
+        self.line1 = line1[0:self.scb.linelen]
+        self.line2 = line2[0:self.scb.linelen]
 
         # load rows
         self.rows = []		# formatted rows
         self.loadrows(coldesc, rows)
 
-
 class scbtable(scbwin):
     """A self-looping info table.
 
@@ -598,7 +609,7 @@ class scbtable(scbwin):
                 nr = u''
                 oft = 0
                 for col in coldesc:
-                    if type(col) is unicode:
+                    if type(col) in [unicode, str]:	# HACK py<3.0
                         nr += col
                     else:
                         if len(row) > oft:	# space pad 'short' rows
@@ -615,26 +626,20 @@ class scbtable(scbwin):
         if len(self.rows) % self.pagesz == 1:
             self.rows.insert(len(self.rows) - 2, u' ')
 
-        # if time field set and not a round number of rows, append
-        # time line to last row of last page
-        finalmod = len(self.rows) % self.pagesz
-        if self.timestr is not None and finalmod != 0:
-            padrows = (self.pagesz - 1) - finalmod
-            while padrows > 0:
-                self.rows.append(u' ')
-                padrows -= 1
-            self.rows.append(strops.truncpad(self.timepfx,
-                                  self.scb.linelen - 13, 'r')
-                                      + u' ' + self.timestr[0:12])
-
     def redraw(self):
-        for i in range(0,10):
-            self.scb.clrline(i)
-        #self.scb.setoverlay(self.tovr)
-        self.scb.setline(0, self.header)	# ? use header?
+        self.scb.setline(0, self.header.center(self.scb.linelen),chan=u'#anascb')
+        j = 1
+        if self.rowoft == 2:	# space for subheader
+            self.scb.setline(1, self.subhead.center(self.scb.linelen),chan=u'#anascb')
+            j = 2
+        for i in range(j,self.scb.pagelen):
+            self.scb.clrline(i,chan=u'#anascb')
+        self.scb.setoverlay(unt4.OVERLAY_MATRIX,chan=u'#anascb')
         self.scb.flush()
 
     def update(self):
+        # if time field set and not a round number of rows, append
+        # time line to last row of last page
         if self.count%2 == 0 and self.count > PAGE_INIT: # wait ~1/2 sec
             lclk = (self.count - PAGE_INIT) // 2
 	    cpage = (lclk//self.delay) % self.nrpages
@@ -642,40 +647,54 @@ class scbtable(scbwin):
             flush = False
             if pclk < self.pagesz + 1:
                 if pclk != self.pagesz:
-                    self.scb.clrline(PAGE_ROWOFT + pclk)
+                    self.scb.clrline(self.rowoft + pclk,chan=u'#anascb')
                     flush = True
-                elif self.nrpages == 1:
-                    self.count += 1
-                    self.paused = True       # no animate on single page
+                else:	# end of page
+                    if self.nrpages == 1:
+                        self.count += 1
+                        self.paused = True # no further animate on single page
+                    if self.timestr is not None:
+                        self.scb.setline(self.rowoft + pclk,
+                                 strops.truncpad(self.timepfx,
+                                  self.scb.linelen - 13, 'r')
+                                      + u' ' + self.timestr[0:12],chan=u'#anascb')
+                flush = True
                 if pclk != 0:
                     roft = self.pagesz * cpage + pclk-1
                     if roft < len(self.rows):
-                        self.scb.setline(PAGE_ROWOFT + pclk-1,
-                                         self.rows[roft])
+                        self.scb.setline(self.rowoft + pclk-1,
+                                         self.rows[roft],chan=u'#anascb')
                         flush = True
             if flush:
                 self.scb.flush()
         if not self.paused:
             self.count += 1
 
-    def __init__(self, scb=None, head=u'',
-                 coldesc=None, rows=None, pagesz=4,
+    def __init__(self, scb=None, head=u'', subhead=u'',
+                 coldesc=None, rows=None, pagesz=None,
                  timepfx=u'', timestr=None, delay=PAGE_DELAY):
         scbwin.__init__(self, scb)
         # set page size
-        self.pagesz = 4
+        self.pagesz = self.scb.pagelen - 2
+        # page row offset ... hardcoded for now
+        self.rowoft = 2
         self.nrpages = 0
         self.delay = delay
         self.timestr = timestr
         self.timepfx = timepfx
-        self.tovr = unt4.OVERLAY_T1P5
-        if pagesz == 4:
-            self.pagesz = 4		# ignore any other size
-            self.tovr = unt4.OVERLAY_T1P4
+        if pagesz and pagesz > 5:
+            self.pagesz = 6		# grab a line from the top
+            self.rowoft = 2
+
+        if self.timestr is not None:
+            self.pagesz -= 1		# grab a line from the bottom
 
         # prepare header -> must be preformatted
-        ## TODO : selective pagesz with offset based on header
-        self.header = head[0:self.scb.linelen]
+        if pagesz < 6:	# this is a hack for the madison, maybe replace?
+            self.header = head[0:self.scb.linelen].strip()
+        else:
+            self.header = head[0:self.scb.linelen]
+        self.subhead = subhead
 
         # load rows
         self.rows = []		# formatted rows
