@@ -32,14 +32,15 @@ import Queue
 import logging
 import socket
 import metarace
+import serial
 
 from metarace import unt4
 from metarace import strops
 
 # Default Galactica encoding is cp1252 (ansi)
 ENCODING = 'cp1252'
-LINELEN = 32
-PAGELEN = 8
+LINELEN = 24
+PAGELEN = 7
 
 # protocol to object mappings
 PROTOCOLS = {
@@ -48,6 +49,42 @@ PROTOCOLS = {
 # dispatch thread queue commands
 TCMDS = ('EXIT', 'PORT', 'MSG')
 
+class serialport(object):
+    """Scoreboard communication port object."""
+    def __init__(self, addr, protocol):
+        """Constructor.
+
+        Parameters:
+
+          addr -- socket style 2-tuple (host, port)
+          protocol -- one of socket.SOCK_STREAM or socket.SOCK_DGRAM
+
+        """
+        self.__s = serial.Serial(addr,38400,xonxoff=True)
+        self.send = self.__s.write  # local cache the send() method
+        self.read = self.__s.read
+        self.running = True
+
+    def sendall(self, buf):
+        """Send all of buf to port."""
+        msglen = len(buf)
+        sent = 0
+        while sent < msglen:
+            out = self.send(buf[sent:])
+            #junk = self.read(1024)
+            #if out == 0:
+                #raise socket.error("Serial port write error")
+            sent += len(buf)	# LAME
+        pass
+
+    def close(self):
+        """Shutdown socket object."""
+        self.running = False
+        try:
+            self.__s.close()
+        except:
+            pass	# error here should not leak out
+
 class scbport(object):
     """Scoreboard communication port object."""
     def __init__(self, addr, protocol):
@@ -153,7 +190,10 @@ def mkport(port=None):
             nport = socket.getservbyname(vels[2])
     
     ## split port string into [PROTOCOL:]ADDR[:PORT]
-    return scbport((naddr, nport), nprot)
+    if u'/dev/' in naddr:
+        return serialport(naddr, nprot)
+    else:
+        return scbport((naddr, nport), nprot)
 
 def mksender(port=None, protocol=None):
     nprot = u'dhi'
@@ -193,36 +233,39 @@ class sender(threading.Thread):
 
     """
 
-    def clrall(self):
+    def clrall(self, chan=None):
         """Clear all lines in DHI database."""
         self.sendmsg(unt4.GENERAL_CLEARING)
 
-    def clrline(self, line):
+    def clrline(self, line, chan=None):
         """Clear the specified line in DHI database."""
         self.sendmsg(unt4.unt4(xx=0,yy=int(line),erl=True))
 
-    def setline(self, line, msg):
+    def setline(self, line, msg, chan=None):
         """Set the specified DHI database line to msg."""
-        msg = msg[0:self.linelen].ljust(self.linelen)
+        #msg = msg
+        #msg = msg[0:self.linelen].ljust(self.linelen)
         self.sendmsg(unt4.unt4(xx=0,yy=int(line),text=msg))
 
-    def flush(self):
-        """Send a flush packet if possible."""
+    def flush(self, chan=None):
+        """Send a flush packet if possible (or required?)."""
         pass
 
-    def linefill(self, line, char=u'_'):
+    def linefill(self, line, char=u'_', chan=None):
         """Use char to fill the specified line."""
-        msg = char * self.linelen
+        #msg = char * self.linelen
+        msg = char * 100
         self.sendmsg(unt4.unt4(xx=0,yy=int(line),text=msg))
 
-    def postxt(self, line, oft, msg):
+    def postxt(self, line, oft, msg, chan=None):
         """Position msg at oft on line in DHI database."""
         # TODO: Handle negative offsets? From right?
-        if oft < self.linelen:
-            msg = msg[0:(self.linelen-oft)]
+        #if oft < self.linelen:
+        if True:
+            #msg = msg[0:(self.linelen-oft)]
             self.sendmsg(unt4.unt4(xx=int(oft),yy=int(line),text=msg))
 
-    def setoverlay(self, newov):
+    def setoverlay(self, newov, chan=None):
         """Request overlay newov to be displayed on the scoreboard."""
         if self.curov != newov:
             self.sendmsg(newov)
@@ -245,7 +288,7 @@ class sender(threading.Thread):
         if port is not None:
             self.setport(port)
 
-    def sendmsg(self, unt4msg=None):
+    def sendmsg(self, unt4msg=None, chan=None):
         """Pack and send a unt4 message to the DHI."""
         self.queue.put_nowait(('MSG', unt4msg.pack()))
 
@@ -313,7 +356,7 @@ class sender(threading.Thread):
                     if self.port is not None:
                         self.port.close()
                         self.port = None
-                    if m[1] is not None and m[1] != '' and m[1] != 'NULL':
+                    if m[1] not in [None, '', 'none', 'NULL']:
                         self.log.debug(u'Re-Connect port: ' + str(m[1]))
                         self.port = mkport(m[1])
                         self.curov = None
@@ -332,48 +375,54 @@ class sender(threading.Thread):
         self.log.info(u'Exiting')
 
 class altsender(sender):
-    def sendmsg(self, unt4msg=None):
+    def sendmsg(self, unt4msg=None, chan=None):
         """Pack and send a unt4 message to the altscb."""
         self.queue.put_nowait(('MSG', unt4msg.altpack()))
+        #self.log.debug(u'DAK: ' + unt4msg.altpack())
 
-    def setoverlay(self, newov):
+    def setoverlay(self, newov, chan=None):
         pass
 
     def flush(self):
         """Send a flush packet."""
-        self.queue.put_nowait(('MSG', unt4.ALT_ACK.altpack()+(9 * unichr(unt4.NUL))))
+        pass
+        #self.queue.put_nowait(('MSG', unt4.ALT_ACK.altpack()+(9 * unichr(unt4.NUL))))
 
     def linefill(self, line, char=u'_'):
         """Use char to fill the specified line."""
-        msg = char * self.linelen
+        #msg = char * self.linelen
+        msg = char * 100
         umsg = unt4.unt4(xx=0,yy=int(line),text=msg)
         self.sendmsg(umsg)
-        self.sendmsg(umsg)
+        #self.sendmsg(umsg)
 
-    def postxt(self, line, oft, msg):
+    def postxt(self, line, oft, msg, chan=None):
         """Position msg at oft on line in DHI database."""
         # TODO: Handle negative offsets? From right?
-        if oft < self.linelen:
-            msg = msg[0:(self.linelen-oft)]
+        #if oft < self.linelen:
+        if True:
+            #msg = msg[0:(self.linelen-oft)]
             umsg = unt4.unt4(xx=int(oft),yy=int(line),text=msg)
             self.sendmsg(umsg)
-            self.sendmsg(umsg)
+            #self.sendmsg(umsg)
 
-    def clrline(self, line):
+    def clrline(self, line, chan=None):
         """Clear the specified line in DHI database."""
-        umsg = unt4.unt4(xx=0,yy=int(line),text=u' '*self.linelen)
-        self.sendmsg(umsg)
+        umsg = unt4.unt4(xx=0,yy=int(line),text=u' '*100)
+        #umsg = unt4.unt4(xx=0,yy=int(line),text=u' '*self.linelen)
         self.sendmsg(umsg)
+        #self.sendmsg(umsg)
 
-    def setline(self, line, msg):
+    def setline(self, line, msg, chan=None):
         """Set the specified DHI database line to msg."""
-        msg = msg[0:self.linelen].ljust(self.linelen)
+        #msg = msg[0:self.linelen].ljust(self.linelen)
+        #msg = msg[0:self.linelen].ljust(self.linelen)
         umsg = unt4.unt4(xx=0,yy=int(line),text=msg)
         self.sendmsg(umsg)
-        self.sendmsg(umsg)
+        #self.sendmsg(umsg)
 
 PROTOCOLS[u'dhi']=sender
-PROTOCOLS[u'ivc']=altsender
+PROTOCOLS[u'dak']=altsender
 
 if __name__ == "__main__":
     """Simple 'Hello World' example with logging."""
