@@ -78,7 +78,7 @@ key_raceover = 'F10'
 key_abort = 'F5'
 
 # config version string
-EVENT_ID = 'sportif-1.5'
+EVENT_ID = 'sportif-1.6'
 
 def key_bib(x):
     """Sort on bib field of aux row."""
@@ -88,6 +88,15 @@ def key_name(x):
     """Sort on sort order key."""
     return x[1]
 
+def key_tod(x):
+    """Sort on arrival time."""
+    ret = 90000000	# highest
+    if x[2] is not None:
+        ret = int(x[2].as_seconds())*1000 + strops.riderno_key(x[1])
+    else:
+        ret += strops.riderno_key(x[1])
+    return ret
+
 def key_team_res(x):
     """Sort on team aggregate time."""
     return int(x[4].timeval)
@@ -95,19 +104,37 @@ def key_team_res(x):
 class sportif(object):
     """Sportif ride handler."""
 
+    def loadcats(self, cats=u''):
+        self.cats = []  # clear old cat list
+        catlist = cats.split()
+        if u'AUTO' in catlist:  # ignore any others and re-load from rdb
+            self.cats = self.meet.rdb.listcats()
+            self.autocats = True
+        else:
+            self.autocats = False
+            for cat in catlist:
+                if cat != u'':
+                    cat = cat.upper()
+                    self.cats.append(cat)
+        self.cats.append(u'')   # always include one empty cat
+        self.log.debug(u'Result category list updated: ' + repr(self.cats))
+
     def loadconfig(self):
         """Load race config from disk."""
         self.riders.clear()
         self.resettimer()
+        self.cats = []
         cr = jsonconfig.config({u'event':{
                                           u'start':u'',
-                                          u'lstart':u'',
                                           u'finish':u'',
                                           u'categories':[],
+                                          u'mintime':u'5:00',
+                                          u'maxtime':u'20h00:00',
                                           u'minimums':[],
                                           u'maximums':[],
                                           u'timerstat':u'idle',
                                           u'finished':False,
+                                          u'timeorder':False,
                                           u'id':EVENT_ID,
                                           u'startlist':[]}})
         cr.add_section(u'event')
@@ -121,31 +148,41 @@ class sportif(object):
 
         # load _result_ categories
         catlist = cr.get(u'event', u'categories')
-        self.cats = []
-        for cat in catlist:
-            if cat != '':
-                cat = cat.strip().upper()
-                self.cats.append(cat)
-        self.cats.append(u'')    # always include one empty cat
-        minlist = []
-        for mtm in cr.get(u'event', u'minimums'):
-            minlist.append(tod.str2tod(mtm))
-        while len(minlist) < len(self.cats):
-            minlist.append(None)	# extend as necessary
+        if u'AUTO' in catlist:  # ignore any others and re-load from rdb
+            self.cats = self.meet.rdb.listcats()
+            self.autocats = True
+        else:
+            self.autocats = False
+            for cat in catlist:
+                if cat != u'':
+                    cat = cat.upper()
+                    self.cats.append(cat)
+        self.cats.append(u'')   # always include one empty cat
 
-        # and repead for maxlist
-        maxlist = []
-        for mtm in cr.get(u'event', u'maximums'):
-            maxlist.append(tod.str2tod(mtm))
-        while len(maxlist) < len(self.cats):
-            maxlist.append(None)	# extend as necessary
+        # Load abs minimum and maximum
+        mtm = tod.str2tod(cr.get(u'event', u'mintime'))
+        if mtm is not None:
+            self.mintime = mtm.truncate(0)
+        mtm = tod.str2tod(cr.get(u'event', u'maxtime'))
+        if mtm is not None:
+            self.maxtime = mtm.truncate(0)
 
-        # then populate maps	HACK	should be stored this way
+        # Load maps for minimums and maximums
         self.minmap = {}
         self.maxmap = {}
-        for j in range(0, len(self.cats)):
-            self.minmap[self.cats[j]] = minlist[j]
-            self.maxmap[self.cats[j]] = maxlist[j]
+        minsrc = cr.get(u'event', u'minimums')
+        maxsrc = cr.get(u'event', u'maximums')
+        for cat in self.cats:
+            if cat:	# use mintime/maxtime for u'' cat
+                mcat = cat.upper()
+                mtm = None
+                if mcat in minsrc:
+                    mtm = tod.str2tod(minsrc[mcat]) 
+                self.minmap[mcat] = mtm
+                mtm = None
+                if mcat in maxsrc:
+                    mtm = tod.str2tod(maxsrc[mcat]) 
+                self.maxmap[mcat] = mtm
 
         starters = cr.get(u'event', u'startlist')
         # for a sportif - always sort by bib
@@ -167,7 +204,8 @@ class sportif(object):
                         laptod = tod.str2tod(ril[i])
                         if laptod is not None:
                             nr[COL_RFSEEN].append(laptod)
-        self.set_start(cr.get(u'event', u'start'), cr.get(u'event', u'lstart'))
+        self.timeorder = strops.confopt_bool(cr.get(u'event', u'timeorder'))
+        self.set_start(cr.get(u'event', u'start'))
         self.set_finish(cr.get(u'event', u'finish'))
         if strops.confopt_bool(cr.get(u'event', u'finished')):
             self.set_finished()
@@ -213,32 +251,34 @@ class sportif(object):
         cw.add_section(u'event')
         if self.start is not None:
             cw.set(u'event', u'start', self.start.rawtime())
-        if self.lstart is not None:
-            cw.set(u'event', u'lstart', self.lstart.rawtime())
         if self.finish is not None:
             cw.set(u'event', u'finish', self.finish.rawtime())
         cw.set(u'event', u'finished', self.timerstat == 'finished')
         cw.set(u'event', u'timerstat', self.timerstat)
-        cw.set(u'event', u'startlist', self.get_startlist())    
-        catlist = []
-        minlist = []
-        maxlist = []
+        cw.set(u'event', u'timeorder', self.timeorder)
+
+        cw.set(u'event', u'mintime', self.mintime.rawtime(0))
+        cw.set(u'event', u'maxtime', self.maxtime.rawtime(0))
+        minmap = {}
+        maxmap = {}
         for cat in self.cats:
-            if cat:
-                catlist.append(cat)
-                mtm = self.minmap[cat]
-                if mtm is not None:
-                    minlist.append(mtm.rawtime())
-                else:
-                    minlist.append(u'')
-                mtm = self.maxmap[cat]
-                if mtm is not None:
-                    maxlist.append(mtm.rawtime())
-                else:
-                    maxlist.append(u'')
-        cw.set(u'event', u'categories', catlist)
-        cw.set(u'event', u'minimums', minlist)
-        cw.set(u'event', u'maximums', maxlist)
+            if cat:	# use mintime/maxtime for u'' cat
+                mcat = cat.upper()
+                mtm = u''
+                if mcat in self.minmap and self.minmap[mcat] is not None:
+                    mtm = self.minmap[mcat].rawtime(0)
+                minmap[mcat] = mtm
+                mtm = u''
+                if mcat in self.maxmap and self.maxmap[mcat] is not None:
+                    mtm = self.maxmap[mcat].rawtime(0)
+                maxmap[mcat] = mtm
+        cw.set(u'event', u'minimums', minmap)
+        cw.set(u'event', u'maximums', maxmap)
+        cw.set(u'event', u'startlist', self.get_startlist())    
+        if self.autocats:
+            cw.set(u'event', u'categories', [u'AUTO'])
+        else:
+            cw.set(u'event', u'categories', self.get_catlist())
 
         cw.add_section(u'riders')
         for r in self.riders:
@@ -330,6 +370,18 @@ class sportif(object):
             self.riders.reorder([a[0] for a in aux])
         return cnt
 
+    def reorder_bytime(self):
+        """Reorder riders for a result."""
+        aux = []
+        cnt = 0
+        for r in self.riders:
+            aux.append([cnt, r[COL_BIB], r[COL_RFTIME]])
+            cnt += 1
+        if len(aux) > 1:
+            aux.sort(key=key_tod)
+            self.riders.reorder([a[0] for a in aux])
+        return cnt
+
     def result_report(self):
         """Return a result report."""
         ret =  []
@@ -343,7 +395,10 @@ class sportif(object):
         sec = printing.section()
         sec.heading = u'Result'
 
-        self.reorder_byname()
+        if self.timeorder:
+            self.reorder_bytime()
+        else:
+            self.reorder_byname()
 
         for r in self.riders:
             et = self.get_rider_elapsed(self.getiter(r[COL_BIB]))
@@ -656,7 +711,7 @@ class sportif(object):
     def starttrig(self, e):
         """Process a 'start' trigger signal."""
         if self.timerstat == 'armstart':
-            self.set_start(e, tod.tod('now'))
+            self.set_start(e)
         return False
 
     def rfidtrig(self, e):
@@ -667,8 +722,10 @@ class sportif(object):
         # else assume a passing
         r = self.meet.rdb.getrefid(e.refid)
         if r is None:
-            r = self.meet.rdb.addempty(e.refid, self.series)
-            self.meet.rdb.editrider(r, refid=e.refid)
+            self.log.info('Unknown tag: ' + e.refid + '@' + e.rawtime(2))
+            return False
+            #r = self.meet.rdb.addempty(e.refid, self.series)
+            #self.meet.rdb.editrider(r, refid=e.refid)
 
         bib = self.meet.rdb.getvalue(r, riderdb.COL_BIB)
         ser = self.meet.rdb.getvalue(r, riderdb.COL_SERIES)
@@ -679,10 +736,15 @@ class sportif(object):
 
         lr = self.getrider(bib)
         if lr is None:	# sportif uses relaxed 'club mode'
-            self.addrider(bib)
-            lr = self.getrider(bib)
-            self.log.info(u'Added non starter: ' + repr(bib)
-                          + u' @ ' + e.rawtime(1))
+            if self.timerstat in ['armstart','armfinish']:
+                self.addrider(bib)
+                lr = self.getrider(bib)
+                self.log.info(u'Added starter: ' + repr(bib)
+                              + u' @ ' + e.rawtime(1))
+            else:
+                self.log.warn('Saw: ' + repr(bib)
+                          + ' @ ' + e.rawtime(1))
+                return False
 
         # at this point should always have a valid rider vector
         if self.timerstat == 'armstart':
@@ -712,12 +774,10 @@ class sportif(object):
     def new_start_trigger(self, rfid):
         """Collect a RFID trigger signal and apply it to the model."""
         if self.newstartdlg is not None and self.newstartent is not None:
-            nt = tod.tod('now')
             et = tod.str2tod(self.newstartent.get_text())
             if et is not None:
                 st = rfid - et
-                lt = nt - et
-                self.set_start(st, lt)
+                self.set_start(st)
                 self.newstartdlg.response(1)
                 self.newstartdlg = None # try to ignore the 'up' impulse
             else:
@@ -804,20 +864,12 @@ class sportif(object):
             self.recalculate()
         return True
 
-    def set_start(self, start='', lstart=None):
+    def set_start(self, start=''):
         """Set the start time."""
         if type(start) is tod.tod:
             self.start = start
-            if lstart is not None:
-                self.lstart = lstart
-            else:
-                self.lstart = self.start
         else:
             self.start = tod.str2tod(start)
-            if lstart is not None:
-                self.lstart = tod.str2tod(lstart)
-            else:
-                self.lstart = self.start
         if self.start is not None and self.finish is None:
             self.set_running()
 
@@ -837,19 +889,12 @@ class sportif(object):
     def set_elapsed(self):
         """Update the elapsed time field."""
         pass
-        #if self.start is not None and self.finish is not None:
-            #self.time_lbl.set_text((self.finish - self.start).timestr(0))
-        #elif self.start is not None:    # Note: uses 'local start' for RT
-            #self.time_lbl.set_text((tod.tod('now') - self.lstart).timestr(0))
-        #elif self.timerstat == 'armstart':
-            #self.time_lbl.set_text(tod.tod(0).timestr(0))
-        #else:
-            #self.time_lbl.set_text('')
 
     def set_running(self):
         """Update event status to running."""
-        self.timerstat = 'running'
-        uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Running')
+        self.log.info(u'Running trigger')
+        #self.timerstat = 'running'
+        #uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Running')
 
     def set_finished(self):
         """Update event status to finished."""
@@ -860,6 +905,14 @@ class sportif(object):
             self.set_finish(tod.tod('now'))
         self.set_elapsed()
 
+    def get_catlist(self):
+        """Return the ordered list of categories."""
+        rvec = []
+        for cat in self.cats:
+            if cat != '':
+                rvec.append(cat)
+        return rvec
+
     def ridercat(self, cat):
         """Return a category from the result for the riders cat."""
         ret = u''        # default is the 'None' category - uncategorised
@@ -950,7 +1003,6 @@ class sportif(object):
         # race run time attributes
         self.readonly = not ui
         self.start = None
-        self.lstart = None
         self.finish = None
         self.winopen = True
         self.timerstat = 'idle'
@@ -960,6 +1012,7 @@ class sportif(object):
         self.maxmap = {}
         self.mintime = tod.tod('5:00')		# abs min
         self.maxtime = tod.tod('20h00:00')	# abs max
+        self.autocats = False
 
         # new start dialog
         self.newstartent = None
