@@ -55,12 +55,6 @@ COL_200M = 7		# time for last 200m
 COL_WINNER = 8		# no of 'winner'
 COL_COMMENT = 9		# decisions of commissaire's panel
 
-# scb colspecs
-SCB_STARTERS_NW = metarace.SCB_LINELEN - 4
-SCB_STARTERS_FMT = [(3,u'r'), u' ', (SCB_STARTERS_NW, u'l')]
-SCB_RESULT_NW = SCB_STARTERS_NW
-SCB_RESULT_FMT = SCB_STARTERS_FMT
-
 # scb function key mappings
 key_startlist = 'F3'			# show starters in table
 key_results = 'F4'			# recalc/show result window
@@ -92,6 +86,13 @@ class sprnd(object):
                 cr[COL_A_NO] = bib
                 cr[COL_A_STR] = self.rider_name(bib)
                 cr[COL_A_PLACE] = u''	# LOAD?
+                ## special case the bye here
+                if cr[COL_CONTEST] == u'bye':
+                    cr[COL_A_PLACE] = u' '
+                    cr[COL_B_STR] = u' '
+                    cr[COL_B_PLACE] = u' '
+                    cr[COL_B_NO] = u' '
+                    cr[COL_WINNER] = bib	# auto win the bye rider
                 return
             cstack.insert(0, cr)	# do these get re-used?
         for cr in cstack:
@@ -180,7 +181,7 @@ class sprnd(object):
         if eid and eid != EVENT_ID:
             self.log.error('Event configuration mismatch: '
                            + repr(eid) + ' != ' + repr(EVENT_ID))
-            self.readonly = True
+            #self.readonly = True
 
     def rider_name(self, bib):
         """Return a formated rider name string."""
@@ -239,7 +240,10 @@ class sprnd(object):
     def startlist_report(self, program=False):
         """Return a startlist report."""
         ret = []
-        sec = printing.sprintround()
+        if self.event[u'type'] == 'sprint final':
+            sec = printing.sprintfinal()
+        else:
+            sec = printing.sprintround()
         headvec = [u'Event', self.evno, u':', self.event[u'pref'],
                          self.event[u'info']]
         if not program:
@@ -253,12 +257,28 @@ class sprnd(object):
             sec.subheading = substr
 
         for cr in self.contests:
-            sec.lines.append([cr[COL_CONTEST] + u':',
+            byeflag = None
+            bno = cr[COL_B_NO]
+            bname = cr[COL_B_STR].decode('utf-8')
+            timestr = None
+            if cr[COL_CONTEST] == u'bye':
+                timestr = u' '
+                bno = u' '
+                bname = u' '
+                byeflag = u' '
+            if self.event[u'type'] == 'sprint final':
+                ## HACK: assume no bye
+                sec.lines.append([cr[COL_CONTEST] + u':',
+                         [None, cr[COL_A_NO], cr[COL_A_STR].decode('utf-8'),
+                                 None, None, None, None],
+                         [None, cr[COL_B_NO], cr[COL_B_STR].decode('utf-8'),
+                                 None, None, None, None] ])
+            else:
+                sec.lines.append([cr[COL_CONTEST] + u':',
                              [None, cr[COL_A_NO],
                                     cr[COL_A_STR].decode('utf-8'), None],
-                             [None, cr[COL_B_NO],
-                                    cr[COL_B_STR].decode('utf-8'), None],
-                              None])
+                             [byeflag, bno, bname, None],
+                              timestr])
         ret.append(sec)
         return ret
 
@@ -512,14 +532,16 @@ class sprnd(object):
         """Show contest result on scoreboard."""
         self.meet.scbwin = None
         self.timerwin = False
-        startlist = [ [u'1.', winner],
-                      [u'2.', loser]]
+        startlist = [ [u'1.', winno, winner],
+                      [u'2.', loseno, loser]]
         if ftime:
-            startlist.append([u'', u''])
-            startlist.append([u'', u'200m: ' + ftime])
+            startlist.append([u'', u'', u''])
+            startlist.append([u'', u'', u'200m: ' + ftime])
+        name_w = self.meet.scb.linelen-8
+        fmt = [(3,'l'),(4,'r'),u' ',(name_w,'l')]
         self.meet.scbwin = scbwin.scbintsprint(self.meet.scb,
                                    self.meet.racenamecat(self.event),
-                                   contest, SCB_STARTERS_FMT, startlist)
+                                   contest, fmt, startlist)
 
         self.meet.announce.gfx_overlay(2)
         self.meet.announce.gfx_set_title(u'Result: '
@@ -556,9 +578,11 @@ class sprnd(object):
             startlist.append([bn, br])
         self.meet.scbwin = None
         self.timerwin = False
+        name_w = self.meet.scb.linelen-5
+        fmt = [(4,'r'),u' ',(name_w,'l')]
         self.meet.scbwin = scbwin.scbintsprint(self.meet.scb,
                                    self.meet.racenamecat(self.event),
-                                   cid, SCB_STARTERS_FMT, startlist)
+                                   cid, fmt, startlist)
         self.meet.scbwin.reset()
 
     def update_expander_lbl_cb(self):
@@ -593,6 +617,7 @@ class sprnd(object):
             if i is not None:	# contest selected ok
                 cid = self.contests.get_value(i, COL_CONTEST)
                 self.contests.set_value(i, COL_200M, self.curelap)
+                self.ctrl_winner.grab_focus()
             self.log_elapsed(cid)
             if self.timerwin and type(self.meet.scbwin) is scbwin.scbtimer:
                 self.showtimer()
@@ -600,6 +625,10 @@ class sprnd(object):
                     self.meet.gemini.rtick(self.finish - self.start, 2)
             glib.idle_add(self.delayed_announce)
 
+    def rftimercb(self, e):
+        """Handle a RF timer event."""
+        return False	# todo... auto win if thresh met
+
     def timercb(self, e):
         """Handle a timer event."""
         chan = timy.chan2id(e.chan)
@@ -697,6 +726,28 @@ class sprnd(object):
                 self.start = None
                 self.finish = None
                 self.set_elapsed()
+            winner = self.contests.get_value(i, COL_WINNER)
+            self.ctrl_winner.set_text(winner)
+
+    def race_ctrl_winner_activate_cb(self, entry, data=None):
+        """Manual entry of race winner."""
+        winner = entry.get_text().decode('ascii', 'ignore')
+        i = self.current_contest_combo.get_active_iter()
+        if i is not None:	# contest selected ok
+            cid = self.contests.get_value(i, COL_CONTEST)
+            self.ctrl_winner.grab_focus()
+            ano = self.contests.get_value(i, COL_A_NO)
+            bno = self.contests.get_value(i, COL_B_NO)
+            if winner == ano:
+                self.set_winner(u'A')
+                glib.idle_add(self.delayed_announce)
+            elif winner == bno:
+                self.set_winner(u'B')
+                glib.idle_add(self.delayed_announce)
+            else:
+                self.log.error(u'Ignored rider not in contest.')
+        else:
+            self.log.info(u'No contest selected.')
 
     def race_info_time_edit_activate_cb(self, button):
         """Display contest timing edit dialog."""
@@ -790,7 +841,10 @@ class sprnd(object):
                 rr = u''
                 sep = u' v '
                 if win:
-                    sep = 'def'
+                    if c[COL_CONTEST] == u'bye':
+                        sep = '   '
+                    else:
+                        sep = 'def'
                 if win == c[COL_B_NO]:
                     lr = (c[COL_B_NO].rjust(3) + u' '
                           + strops.truncpad(c[COL_B_STR].decode('utf-8'), 29))
@@ -840,7 +894,7 @@ class sprnd(object):
             placeoft += 1
         
 
-    def result_report(self):
+    def result_report(self, recurse=False):
         """Return a list of printing sections containing the race result."""
         ret = []
         sec = printing.sprintround()
@@ -864,6 +918,8 @@ class sprnd(object):
                 ft = None
                 if cr[COL_200M] is not None:
                     ft = cr[COL_200M].rawtime(2)
+                else:
+                    ft = u' '
                 if cr[COL_WINNER] == cr[COL_A_NO]:
                     sec.lines.append([cprompt, avec, bvec, ft])
                 else:
@@ -971,7 +1027,7 @@ class sprnd(object):
 
         # ctrl pane
         self.stat_but = b.get_object('race_ctrl_stat_but')
-        self.ctrl_places = b.get_object('race_ctrl_places')
+        self.ctrl_winner = b.get_object('race_ctrl_winner')
         self.ctrl_action_combo = b.get_object('race_ctrl_action_combo')
         self.ctrl_action = b.get_object('race_ctrl_action')
         self.action_model = b.get_object('race_action_model')
