@@ -21,7 +21,6 @@ from __future__ import (division, print_function, absolute_import)
 import re
 import os
 import random
-from . import timy	# import timy channel strings
 
 # replace codepoints 0->255 with space unless overridden
 # "protective" against unencoded ascii strings and control chars
@@ -39,12 +38,15 @@ class unicodetrans:
     return self.comp.get(k)
 
 INTEGER_UTRANS = unicodetrans(u'-0123456789')
+NUMERIC_UTRANS = unicodetrans(u'-0123456789.e')
 PLACELIST_UTRANS = unicodetrans(
 u'-0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz')
 PLACESERLIST_UTRANS = unicodetrans(
 u'-.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz')
 BIBLIST_UTRANS = unicodetrans(
 u'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz')
+RIDERNO_UTRANS = unicodetrans(
+u'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz',u'',u'')
 BIBSERLIST_UTRANS = unicodetrans(
 u'.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz')
 WEBFILE_UTRANS = unicodetrans(
@@ -62,6 +64,22 @@ PRINT_UTRANS[0x180e] = u' '
 PRINT_UTRANS[0x202f] = u' '
 PRINT_UTRANS[0x205f] = u' '
 PRINT_UTRANS[0x3000] = u' '
+PRINT_UTRANS[0xffa0] = u' '
+
+# timing channels - this duplicates defs in timy, but breaks the dep
+# strops should not have any dependencies
+CHAN_START = 0
+CHAN_FINISH = 1
+CHAN_PA = 2
+CHAN_PB = 3
+CHAN_200 = 4
+CHAN_100 = 5
+CHAN_50 = 0  # TODO: use for AUX line into C0
+CHAN_AUX = 6 # channels 6-8 are not connected with original TSW-1 cable
+CHAN_7 = 7
+CHAN_8 = 8
+CHAN_INT = 9  # complete the keypad - a key not from timy
+CHAN_UNKNOWN = -1
 
 def img_filename(path, basename=u''):
     """Return the 'best' filename for the given basename image."""
@@ -77,8 +95,8 @@ def img_filename(path, basename=u''):
 
 DNFCODEMAP = { u'hd': 0,
                u'dsq': 1,
-               u'dnf': 2,
-               u'dns': 3,
+               u'dnf': 3,
+               u'dns': 4,
                u'': 2}
 
 def cmp_dnf(x, y):
@@ -93,6 +111,21 @@ def riderno_key(bib):
     """Return a comparison key for sorting rider number strings."""
     return bibstr_key(bib)
 
+def dnfcode_key(code):
+    """Return a dnf code sorting key."""
+    # '' [rel] dsq hd|otl dnf dns
+    dnfordmap = {u'':0, u'rel':2,
+                 u'dsq':2,
+                 u'hd':3,u'otl':3,
+                 u'dnf':4,
+                 u'dns':5}
+    ret = 0
+    if code is not None:
+        code = code.lower()
+        if code in dnfordmap:
+            ret = dnfordmap[code]
+    return ret
+
 def bibstr_key(bibstr=u''):
     """Return a comparison key for sorting rider bib.ser strings."""
     (bib, ser) = bibstr2bibser(bibstr)
@@ -100,7 +133,11 @@ def bibstr_key(bibstr=u''):
     if bib.isdigit():
         bval = int(bib)
     else:
-        bval = id(bib)
+        sbib = bib.translate(INTEGER_UTRANS)
+        if sbib:
+            bval = int(sbib)
+        else:
+            bval = id(bib)
     sval = 0
     if ser != u'':
         sval = ord(ser[0])<<12
@@ -205,6 +242,15 @@ def num2ord(place):
     else:
         return place
 
+def rank2int(rank):
+    """Convert a rank/placing string into an integer."""
+    ret = None
+    try:
+        ret = int(rank.replace(u'.',u''))
+    except:
+        pass
+    return ret
+
 def mark2int(handicap):
     """Convert a handicap string into an integer number of metres."""
     handicap = handicap.decode('utf-8','replace').strip().lower()
@@ -241,6 +287,8 @@ def resname_bib(bib, first, last, club):
     """Return rider name formatted for results with bib (champs/live)."""
     ret = bib + u' ' + fitname(first, last, 64)
     if club is not None and club != u'':
+        if len(club) < 4:
+            club=club.upper()
         ret += u' (' + club + u')'
     return ret
 
@@ -248,6 +296,8 @@ def resname(first, last=None, club=None):
     """Return rider name formatted for results."""
     ret = fitname(first, last, 64)
     if club is not None and club != u'':
+        if len(club) < 4:
+            club=club.upper()
         ret += u' (' + club + u')'
     return ret
 
@@ -255,6 +305,8 @@ def listname(first, last=None, club=None):
     """Return a rider name summary field for non-edit lists."""
     ret = fitname(first, last, 32)
     if club:
+        if len(club) < 4:
+            club=club.upper()
         ret += u' (' + club + u')'
     return ret
 
@@ -285,10 +337,11 @@ def reformat_riderlist(riderstr, rdb=None, series=u''):
     riderstr = riderstr.decode('utf-8', 'replace').lower()
 
     # special case: 'all' -> return all riders from the sepcified series.
-    if riderstr.strip() == u'all':
-        for cat in rdb.listcats(series):
-            ret += u' ' + rdb.biblistfromcat(cat, series)
-            riderstr = u''
+    if rdb is not None and riderstr.strip().lower() == u'all':
+        riderstr = u''
+        for r in rdb:
+            if r[5] == series:
+                ret += u' ' + r[0]
     
     # pass 1: search for categories
     if rdb is not None:
@@ -406,7 +459,11 @@ def plural(count=0):
     if count == 1:
         ret = ''
     return ret
-    
+
+def confopt_riderno(confstr, default=u''):
+    """Check and return rider number, filtered only."""
+    return confstr.translate(RIDERNO_UTRANS).strip()
+
 def confopt_float(confstr, default=None):
     """Check and return a floating point number."""
     ret = default
@@ -436,11 +493,33 @@ def confopt_dist(confstr, default=None):
     """Check and return a valid distance unit."""
     return confopt_posint(confstr, default)
 
+def chan2id(chanstr='0'):
+    """Return a channel ID for the provided string, without fail."""
+    ret = CHAN_UNKNOWN
+    if (type(chanstr) in [unicode, str] and len(chanstr) > 1
+        and chanstr[0] == u'C' and chanstr[1].isdigit()):
+        ret = int(chanstr[1])
+    else:
+        try:
+            ret = int(chanstr)
+        except:
+            pass # other errors will re-occur later anyhow
+    if ret < CHAN_UNKNOWN or ret > CHAN_INT:
+        ret = CHAN_UNKNOWN
+    return ret
+
+def id2chan(chanid=0):
+    """Return a normalised channel string for the provided channel id."""
+    ret = u'C?'
+    if type(chanid) is int and chanid >= CHAN_START and chanid <= CHAN_INT:
+        ret = u'C' + unicode(chanid)
+    return ret
+
 def confopt_chan(confstr, default=None):
     """Check and return a valid timing channel id string."""
-    ret = timy.chan2id(default)
-    ival = timy.chan2id(confstr)
-    if ival != timy.CHAN_UNKNOWN:
+    ret = chan2id(default)
+    ival = chan2id(confstr)
+    if ival != CHAN_UNKNOWN:
         ret = ival
     return ret
 
@@ -496,7 +575,7 @@ def titlesplit(src=u'', linelen=24):
         ret[line] = words.pop(0)
         for word in words:
             pos = len(ret[line])
-            if pos + len(word) > linelen:
+            if pos + len(word) >= linelen:
                 # new line
                 line += 1
                 if line > 2:
@@ -509,8 +588,10 @@ def titlesplit(src=u'', linelen=24):
 class countback(object):
     __hash__ = None
     """Simple dict wrapper for countback store/compare."""
-    def __init__(self):
+    def __init__(self, cbstr=None):
         self.__store = {}
+        if cbstr is not None:
+            self.fromstring(cbstr)
 
     def maxplace(self):
         """Return maximum non-zero place."""
@@ -518,6 +599,17 @@ class countback(object):
         if len(self.__store) > 0:
             ret = max(self.__store.keys())
         return ret
+    def fromstring(self, cbstr):
+        propmap = {}
+        cbvec = cbstr.split(u',')
+        if len(cbvec) > 0:
+            for i in range(0,len(cbvec)):
+                if cbvec[i].isdigit():
+                    propmap[i] = int(cbvec[i])
+        self.__store = {}
+        for k in propmap:
+            self.__store[k] = propmap[k]
+
     def __str__(self):
         ret = []
         for i in range(0,self.maxplace()+1):
@@ -547,7 +639,7 @@ class countback(object):
     def __lt__(self, other):
         if type(other) is not countback: return NotImplemented
         ret = False # assume all same
-        for i in range(1,max(self.maxplace(), other.maxplace())+1):
+        for i in range(0,max(self.maxplace(), other.maxplace())+1):
             a = self[i]
             b = other[i]
             if a != b:
@@ -557,7 +649,7 @@ class countback(object):
     def __le__(self, other):
         if type(other) is not countback: return NotImplemented
         ret = True # assume all same
-        for i in range(1,max(self.maxplace(), other.maxplace())+1):
+        for i in range(0,max(self.maxplace(), other.maxplace())+1):
             a = self[i]
             b = other[i]
             if a != b:
@@ -567,7 +659,7 @@ class countback(object):
     def __eq__(self, other):
         if type(other) is not countback: return NotImplemented
         ret = True
-        for i in range(1,max(self.maxplace(), other.maxplace())+1):
+        for i in range(0,max(self.maxplace(), other.maxplace())+1):
             if self[i] != other[i]:
                 ret = False
                 break
@@ -575,7 +667,7 @@ class countback(object):
     def __ne__(self, other):
         if type(other) is not countback: return NotImplemented
         ret = False
-        for i in range(1,max(self.maxplace(), other.maxplace())+1):
+        for i in range(0,max(self.maxplace(), other.maxplace())+1):
             if self[i] != other[i]:
                 ret = True
                 break
@@ -583,7 +675,7 @@ class countback(object):
     def __gt__(self, other):
         if type(other) is not countback: return NotImplemented
         ret = False # assume all same
-        for i in range(1,max(self.maxplace(), other.maxplace())+1):
+        for i in range(0,max(self.maxplace(), other.maxplace())+1):
             a = self[i]
             b = other[i]
             if a != b:
@@ -593,7 +685,7 @@ class countback(object):
     def __ge__(self, other):
         if type(other) is not countback: return NotImplemented
         ret = True # assume all same
-        for i in range(1,max(self.maxplace(), other.maxplace())+1):
+        for i in range(0,max(self.maxplace(), other.maxplace())+1):
             a = self[i]
             b = other[i]
             if a != b:
