@@ -19,6 +19,11 @@
 
 This module provides a telegraph endpoint for IPC over IRC
 
+TODO:
+	- Test complex fragmented JSON samples
+	- Test interaction with remote servers
+	- Simplify multi-channel setup
+	- Add client-client facility for file xfer (DCC?)
 """
 
 import threading
@@ -34,19 +39,30 @@ from metarace import unt4
 from metarace import tod
 from metarace import strops
 
+# IRC Limits
+IRC_MAXMSGLEN = 512		# RFC1459 (enforced by hybrid)
+IRC_FAKEHOSTLEN = 60		# fake len till server sends the real one
+IRC_PRIVHDRLEN = 14		# :nick!user@spoof PRIVMSG #chan :payload \r\n
+				# ': PRIVMSG  :\r\n' len == 14
+
 # Global Defaults
 TELEGRAPH_USER=u'yak'		# default username root part
 TELEGRAPH_HOST=u''		# default is "not present"
 TELEGRAPH_PORT=6667
-TELEGRAPH_FULLNAME=u'telegraph client'	# default fullname
+TELEGRAPH_USERNAME=u'tclient'	# default username
+TELEGRAPH_FULLNAME=u'telegraph client'	# default ircname
+TELEGRAPH_FAKEHOST=u'o'*IRC_FAKEHOSTLEN
 TELEGRAPH_CHANNEL=u'#agora'	# default channel
-CMDMAX = 400
+TELEGRAPH_PRIVATESERVERS=[u'127.0.0.1',u'localhost',
+                          u'192.168.95.64',u'192.168.96.64']
+TELEGRAPH_OPERNAME=u'metairc'
+TELEGRAPH_OPERPASS=u'ogfPOHYkaw'
 
 # dispatch thread queue commands
 TCMDS = ('EXIT', 'PORT', 'MSG')
 
 # command read queue timeout value
-QUEUE_TIMEOUT = 5
+QUEUE_TIMEOUT = 2
 
 # client initiated ping interval - helps to break a network error
 PING_INTERVAL = tod.tod('30')
@@ -101,6 +117,8 @@ class telegraph(threading.Thread):
         """Add channel to the set of joined channels on reconnect."""
         if channel:
             self.addchans.add(channel)
+            if self.connected():
+                self.queue.put_nowait(('ADDCHAN', channel))
 
     def clrall(self, chan=None):
         """Clear the live announce screen."""
@@ -169,20 +187,27 @@ class telegraph(threading.Thread):
 
     def setline(self, line, msg, chan=None):
         """Set the specified DHI database line to msg."""
-        msg = msg[0:self.linelen].ljust(self.linelen)
-        msg = msg + ' ' * (self.linelen - len(msg))
+        #msg = msg[0:self.linelen].ljust(self.linelen)
+        msg = msg + ' ' * 100
+        #msg = msg + ' ' * (self.linelen - len(msg))
         self.sendmsg(unt4.unt4(xx=0,yy=int(line),text=msg),chan)
 
+    def flush(self):
+        """Send a flush packet if possible (or required?)."""
+        pass
+
     def linefill(self, line, char='_', chan=None):
         """Use char to fill the specified line."""
-        msg = char * self.linelen
+        msg = char * 100
+        #msg = char * self.linelen
         self.sendmsg(unt4.unt4(xx=0,yy=int(line),text=msg), chan)
 
     def postxt(self, line, oft, msg, chan=None):
         """Position msg at oft on line in DHI database."""
         assert oft >= 0, 'Offset should be >= 0'
-        if oft < self.linelen:
-            msg = msg[0:(self.linelen-oft)]
+        #if oft < self.linelen:
+        if True:
+            #msg = msg[0:(self.linelen-oft)]
             self.sendmsg(unt4.unt4(xx=int(oft),yy=int(line),text=msg), chan)
 
     def setoverlay(self, newov, chan=None):
@@ -226,7 +251,7 @@ class telegraph(threading.Thread):
 
     def server_join_cb(self, c, e):
         """Register server join."""
-        self.log.debug('Connected to server')
+        self.log.debug(u'Connected to server:' + repr(c) + repr(e))
         self.queue.put_nowait(('CHAN', ''))
         for chan in self.addchans:
             self.queue.put_nowait(('ADDCHAN', chan))
@@ -264,6 +289,22 @@ class telegraph(threading.Thread):
         """Set the public message callback function."""
         self.pub_cb = cb
 
+    def mode_cb(self, c, e):
+        """Catch the MODE command to register server spoofed addr"""
+        # This is required in order to determine the maximum message length
+        # without having to fudge the server-assigned user@host portion.
+        # If the server does not issue MODE, this is assumed to be an
+        # arbitrartily long string, long enough to catch most cases.
+        src = self.il.nm_to_n(e.source()).lower()
+        #src = e.source().lower()
+        dest = e.target().lower()
+        if src == self.nick.lower():
+            self.srvid = e.source()
+            self.log.debug(u'Registering server id: ' + repr(self.srvid)
+                             + u' :: ' + repr(src) + u'->' 
+                             + repr(dest) + u' :: '
+                             + ''.join(e.arguments()).decode('utf-8'))
+
     def msg_cb(self, c, e):
         """Handle message in channel."""
         source = self.il.nm_to_n(e.source()).lower()
@@ -301,12 +342,18 @@ class telegraph(threading.Thread):
         self.running = False
         self.debug = False
         self.il = None
-        self.localsrv = False	# if set, try to oper
+        self.localsrv = False
         self.rdbuf = {}	# source -> dest -> buf[]
         self.pub_cb = None
         self.iohandle = {}
         self.addchans = set()
 
+        ## HACK
+        self.linelen = linelen
+        self.pagelen = 8
+        self.encoding = 'utf-8'
+
+
         self.log = logging.getLogger('telegraph')
         self.log.setLevel(logging.DEBUG)
 
@@ -329,19 +376,33 @@ class telegraph(threading.Thread):
         self.nick = TELEGRAPH_USER + unicode(random.randint(1000,9999))
         self.host = TELEGRAPH_HOST
         self.port = TELEGRAPH_PORT
+        self.username = TELEGRAPH_USERNAME
         self.fullname = TELEGRAPH_FULLNAME
         self.channel = TELEGRAPH_CHANNEL
+        self.srvid = TELEGRAPH_FAKEHOST
+        self.privateservers = TELEGRAPH_PRIVATESERVERS
+        self.opername = TELEGRAPH_OPERNAME
+        self.operpass = TELEGRAPH_OPERPASS
         self.doreconnect = False
         self.connect_pending = False
         self.dumbcnt = 0
         self.curov = None	# allow use as a sender
-        self.linelen = linelen
+        #self.linelen = linelen
         self.queue = Queue.Queue()
         # check system config for overrides
+        if metarace.sysconf.has_option(u'telegraph', u'username'):
+            self.username = metarace.sysconf.get(u'telegraph', u'username')
         if metarace.sysconf.has_option(u'telegraph', u'fullname'):
             self.fullname = metarace.sysconf.get(u'telegraph', u'fullname')
         if metarace.sysconf.has_option(u'telegraph', u'channel'):
             self.channel = metarace.sysconf.get(u'telegraph', u'channel')
+        if metarace.sysconf.has_option(u'telegraph', u'privateservers'):
+            self.privateservers = metarace.sysconf.get(u'telegraph',
+                                                       u'privateservers')
+        if metarace.sysconf.has_option(u'telegraph', u'opername'):
+            self.opername = metarace.sysconf.get(u'telegraph', u'opername')
+        if metarace.sysconf.has_option(u'telegraph', u'operpass'):
+            self.operpass = metarace.sysconf.get(u'telegraph', u'operpass')
         if metarace.sysconf.has_option(u'telegraph', u'ping_interval'):
             ck = tod.str2tod(metarace.sysconf.get(u'telegraph',
                                                     u'ping_interval'))
@@ -380,8 +441,8 @@ class telegraph(threading.Thread):
                 self.ic.disconnect()
                 if osock:	# irclib has an error that stops delsock call
                     self._delsock(osock)
-            if self.host == 'localhost' or self.host == '127.0.0.1':
-                self.localsrv = True	# assume oper for same machine
+            if self.host in self.privateservers:
+                self.localsrv = True	# try metalan oper if private server
             try:		# dump queue before reconnect
                 while True:	# QUERY: is this necessary?
                     self.queue.get_nowait()
@@ -422,19 +483,36 @@ class telegraph(threading.Thread):
         self.ic.join(adchan)
 
     def _rechan(self):
+        # TODO: allow flag for oper connections
         if self.localsrv:
-            self.ic.oper('metairc', 'ogfPOHYkaw')
+            self.log.debug(u'Sending OPER: ' + repr(self.opername))
+            self.ic.oper(self.opername, self.operpass)
         self.ic.join(self.channel)
-        if self.localsrv:
-            self.ic.send_raw('OPME ' + self.channel)
 
     def _reconnect(self):
         if not self.connect_pending:
             self.log.debug('Connecting to '
                             + self.host + ':' + str(self.port))
             self.connect_pending = True
-            self.ic.connect(self.host, self.port,
-                            self.nick,'',self.fullname)
+            self.ic.connect(server=self.host,
+                            port=self.port,
+                            nickname=self.nick,
+                            username=self.username,
+                            ircname=self.fullname)
+
+    def __streammsg(self, chan, msg):
+        """Output the message in as many blocks as are required."""
+        # :NICK!USER@SPOOF PRIVMSG #CHAN :payload \r\n
+        maxlen = IRC_MAXMSGLEN - (IRC_PRIVHDRLEN + len(chan) + len(self.srvid))
+        #self.log.debug(u'Truncating with: ' + repr(chan)
+                       #+ u',' + repr(self.srvid))
+        pos = 0
+        while pos < len(msg):
+            self.ic.privmsg(chan, msg[pos:pos+maxlen])
+            pos += maxlen
+        if not self.localsrv:
+            # avoid channel flooding by sleep here
+            time.sleep(0.3+0.8*random.random())
 
     def run(self):
         """Called via threading.Thread.start()."""
@@ -444,6 +522,7 @@ class telegraph(threading.Thread):
         self.ic.add_global_handler('privmsg', self.msg_cb, -10)
         self.ic.add_global_handler('welcome', self.server_join_cb, -10)
         self.ic.add_global_handler('join', self.channel_join_cb, -10)
+        self.ic.add_global_handler('umode', self.mode_cb, -10)
         self.ic.add_global_handler('part', self.channel_part_cb, -10)
         self.ic.add_global_handler('kick', self.channel_part_cb, -10)
         self.ic.add_global_handler('disconnect', self.disconnect_cb, -10)
@@ -451,7 +530,7 @@ class telegraph(threading.Thread):
             self.ic.add_global_handler('all_events', self.irc_event_cb, 0)
         while self.running:
             try:
-                if self.host != '':
+                if self.host not in [None, '', 'none', 'NULL']:
                     # irc process phase
                     if not self.connected() or self.doreconnect:
                         self.doreconnect = False
@@ -470,11 +549,14 @@ class telegraph(threading.Thread):
                 while True:
                     m = self.queue.get(timeout=QUEUE_TIMEOUT)
                     self.queue.task_done()
-                    if m[0] == 'MSG' and self.host != '':
+                    if m[0] == 'MSG' and self.host not in [None,
+                                                   '', 'none', 'NULL']:
                         chan = self.channel
                         if len(m) > 2 and m[2]:
                             chan = m[2]
-                        self.ic.privmsg(chan, unt4.encode(m[1]).encode('utf-8'))
+                        self.__streammsg(chan,
+                                         unt4.encode(m[1]).encode('utf-8',
+                                                                  'replace'))
                     elif m[0] == 'CHAN':
                         if m[1] != '':
                             self.channel = m[1]
@@ -547,19 +629,24 @@ if __name__ == '__main__':
         te.set_portstr(force=True)
         return False
 
+    def delaycmd(te):
+        print(u'Delayed cmd.')
+
     def msgcb(cmd, nick, chan):
-        print(u'Telegraph: ' + repr(cmd.pack()) + u' ' + repr(nick)
-                                         + u' ' + repr(chan))
+        print(u'Telegraph: ' + repr(nick) + u'/' + repr(chan)
+                       + u' :: ' + repr(cmd.pack()))
         return False
 
     logging.basicConfig()
     metarace.init()
     t = telegraph()
+    #t.debug = True
     try:
         t.start()
+        t.add_channel('#otherone')
         t.set_portstr(force=True)
         t.set_pub_cb(msgcb)
-        #glib.timeout_add_seconds(20, recon, t)
+        glib.timeout_add_seconds(40, delaycmd, t)
         metarace.mainloop()
     except:
         t.exit()
