@@ -200,13 +200,13 @@ offset	option			values
 11	Sync ToD on CELL	0x00=>off, 0x01=>on
 12	Sync Time hours		0x00=>0h, ... , 0x23=>23h (Question Function?)
 13	Sync Time min		0x00=>:00, ... , 0x59=>:59
-14	Language		0x00=>Eng, 0x01=>Fr
+14	Active Loop		0x00=>passive, 0x01=>powered (active)
 15,16	STA Tone		0x12,0x34=>1234Hz 0x00,0x00=>no tone
 17,18	BOX Tone		"
 19,20	MAN Tone		"
 21,22	CEL Tone		"
 23,24	BXX Tone		"
-25,26	Spare?			0x20 + 0x20
+25,26	Levels			Detect Level
 
 IPCONFIG (16 bytes):
 
@@ -241,19 +241,17 @@ import socket
 import logging
 import glib
 
+import metarace
 from metarace import tod
 from metarace import crc_algorithms
 
-# System default timy serial port
-DEFPORT = '/dev/ttyUSB0'
-
 # Serial baudrate
 THBC_BAUD = 19200
 
 # GTK Priority for timing message callbacks sent to main loop
 THBC_PRIORITY = glib.PRIORITY_HIGH
 
-# UDP Port number for ethernet connection
+# UDP Port number for ethernet connection (can this be configured?)
 THBC_UDP_PORT = 2008
 
 # Photofinish threshold - ~20cm based on tests at DISC,
@@ -279,18 +277,20 @@ STALVL = ESCAPE + chr(0x1e)
 BOXLVL = ESCAPE + chr(0x1f)
 
 NACK = chr(0x07)
+CR = chr(0x0d)
 LF = chr(0x0a)
 SETTIME = ESCAPE + chr(0x48)
 STATSTART = '['
 PASSSTART = '<'
 
 # thread queue commands -> private to timy thread
-TCMDS = ('EXIT', 'PORT', 'MSG', 'TRIG', 'SYNC', 'REPL')
+TCMDS = ('EXIT', 'PORT', 'MSG', 'TRIG', 'SYNC', 'SANE', 'IPCFG', 'REPL')
 
 RFID_LOG_LEVEL = 16     # lower so not in status and on-screen logger.
 logging.addLevelName(RFID_LOG_LEVEL, 'RFID')
 
 # decoder config consts
+IPCONFIG_LEN = 16
 CONFIG_LEN = 27
 CONFIG_TOD = 0
 CONFIG_GPS = 1
@@ -306,12 +306,12 @@ CONFIG_PULSEINT = 10
 CONFIG_CELLSYNC = 11
 CONFIG_CELLTOD_HOUR = 12
 CONFIG_CELLTOD_MIN = 13
-CONFIG_LANG = 14
 CONFIG_TONE_STA = 15
 CONFIG_TONE_BOX = 17
 CONFIG_TONE_MAN = 19
 CONFIG_TONE_CEL = 21
 CONFIG_TONE_BXX = 23
+CONFIG_ACTIVE_LOOP = 14
 CONFIG_SPARE = 25
 CONFIG_FLAGS = {
 	CONFIG_TOD: u'Time of Day',
@@ -328,14 +328,21 @@ CONFIG_FLAGS = {
 	CONFIG_CELLSYNC: u'CELL Sync',
 	CONFIG_CELLTOD_HOUR: u'CELL Sync Hour',
 	CONFIG_CELLTOD_MIN: u'CELL Sync Min',
-	CONFIG_LANG: u'Lang Fr',
         CONFIG_TONE_STA: u'STA Tone',
         CONFIG_TONE_BOX: u'BOX Tone',
         CONFIG_TONE_MAN: u'MAN Tone',
         CONFIG_TONE_CEL: u'CEL Tone',
         CONFIG_TONE_BXX: u'BXX Tone',
+        CONFIG_ACTIVE_LOOP: u'Active Loop',
         CONFIG_SPARE: u'[spare]'
 }
+DEFAULT_IPCFG = {
+  u'IP':u'192.168.0.10',
+  u'Netmask':u'255.255.255.0',
+  u'Gateway':u'0.0.0.0',
+  u'Host':u'192.168.0.255'	# default is broadcast :/
+}
+DEFPORT = u'/dev/ttyUSB0'	# fallback on serial
 
 # Initialise crc algorithm for CRC-16/MCRF4XX
 crc_alg = crc_algorithms.Crc(width=16, poly=0x1021,
@@ -377,7 +384,7 @@ class dgram(object):
         self.buf = ''	# these are not decoded!
 
     def read(self, sz=1):
-        ret = None
+        ret = u''	# check this condition
         # buf empty?
         if len(self.buf) == 0:
             nb, addr = self.s.recvfrom(4096)	# timeout raises exception
@@ -413,6 +420,7 @@ class thbc(threading.Thread):
         self.errstr = ''
         self.unitno = u''		# fetched on reload
         self.decoderconfig = {}		# fetched on reload
+        self.decoderipconfig = {}	# fetched on reload
         self.cqueue = Queue.Queue()	# command queue
         self.log = logging.getLogger(self.name)
         self.log.setLevel(logging.DEBUG)
@@ -471,10 +479,53 @@ class thbc(threading.Thread):
         self.cqueue.put_nowait(('SYNC', None))
 
     def sane(self):
-        """No other config."""
-        pass
+        """Request sanity check in decoder thread."""
+        self.cqueue.put_nowait(('SANE', None))
 
-    def trig(self, timeval='now', index='FAKE', chan='MAN', refid='0'):
+    def ipconfig(self):
+        """Request sanity check in decoder thread."""
+        self.cqueue.put_nowait(('IPCFG', None))
+
+    def __ipcfg(self):
+        """Alter the attached decoder's IP address."""
+        if type(self.port) is not serial.Serial:
+            self.log.error(u'Decoder not on serial link - IP update ignored.')
+            return False
+        ipcfg = metarace.sysconf.get(u'thbc',u'ipconfig')
+        cmd = chr(0x09) + chr(0x09)
+        for i in [u'IP', u'Netmask', u'Gateway', u'Host']:
+            if i not in ipcfg:
+                ipcfg[i] = DEFAULT_IPCFG[i]
+            cmd += socket.inet_aton(socket.gethostbyname(ipcfg[i]))
+        self.log.info(u'Attempting IP config update...')
+        self.__v3_cmd(cmd)
+    
+    def __sane(self):
+        """Check decoder config against system settings."""
+        doconf = False
+        if self.unitno:	# set if config message parsed ok
+            if metarace.sysconf.has_option(u'thbc',u'decoderconfig'):
+                oconf = metarace.sysconf.get(u'thbc',u'decoderconfig')
+                for flag in self.decoderconfig:
+                    key = CONFIG_FLAGS[flag]
+                    if key in oconf:
+                        if oconf[key] != self.decoderconfig[flag]:
+                            self.log.info(u'Key mismatch: ' + repr(key))
+                            self.decoderconfig[flag] = oconf[key]
+                            doconf = True
+
+        # force decoder levels
+        if metarace.sysconf.has_option(u'thbc',u'levels'):
+            lvl = metarace.sysconf.get(u'thbc',u'levels')
+            self.setlvl(box=lvl[0], sta=lvl[1])
+
+        # re-write config if required
+        if doconf:
+            self.log.info(u'Re-configuring decoder ' + repr(self.unitno))
+            self.__set_config()
+
+    def trig(self, timeval='now', index='FAKE', chan='MAN',
+                   refid='0', sourceid=None):
         """Create a fake timing event.
 
 	   Generate a new tod object to mimic a message as requested
@@ -482,7 +533,11 @@ class thbc(threading.Thread):
 	   'now' time in the calling thread.
 
         """
-        t = tod.tod(timeval, index, chan, refid.lstrip('0'))
+        src=self.name
+        if sourceid is not None:
+           src=sourceid
+        t = tod.tod(timeval, index, chan, refid.lstrip('0'), source=src)
+
         self.cqueue.put_nowait(('TRIG', t))
 
     def start_session(self):
@@ -501,11 +556,13 @@ class thbc(threading.Thread):
         """Request decoder configuration."""
         self.write(QUECMD)
 
-    def v3_cmd(self, cmdstr=''):
-        """Pack and send a v3 command"""
+    def __v3_cmd(self, cmdstr=''):
+        """Pack and send a v3 command: NOTE does not queue!"""
         crc = thbc_crc(cmdstr)
         crcstr = chr(crc>>8) + chr(crc&0xff)
-        self.write(ESCAPE + cmdstr + crcstr + '>')
+        self.port.write(ESCAPE + cmdstr + crcstr + '>')
+        # Danger: writes to port
+        ##self.write(ESCAPE + cmdstr + crcstr + '>')
 
     def __serialise_config(self):
         """Convert current decoder setting into a config string"""
@@ -530,19 +587,20 @@ class thbc(threading.Thread):
         # fill in flags
 	for opt in [CONFIG_TOD, CONFIG_GPS, CONFIG_485, CONFIG_FIBRE,
 	            CONFIG_PRINT, CONFIG_MAX, CONFIG_PULSE,
-                    CONFIG_CELLSYNC, CONFIG_LANG]:
+                    CONFIG_CELLSYNC, CONFIG_ACTIVE_LOOP]:
             if opt in self.decoderconfig:
                 if self.decoderconfig[opt]:
                     obuf[opt] = 0x01
         return str(obuf)	# !! not unicode
 
-    def set_config(self):
+    def __set_config(self):
         """Write current configuration to decoder."""
         cmd = chr(0x08) + chr(0x08) + self.__serialise_config()
-        self.v3_cmd(cmd)
-        self.get_config()
+        self.__v3_cmd(cmd)
+        self.port.write(QUECMD)
+        ##self.get_config()
        
-    def set_date(self, timestruct=None):
+    def __set_date(self, timestruct=None):
         """Set the date on the decoder."""
         if timestruct is None:
             timestruct = time.localtime()
@@ -552,7 +610,7 @@ class thbc(threading.Thread):
         cmd += chr(0xff&timestruct[2])		# day
         cmd += chr(0xff&timestruct[1])		# day
         cmd += chr(0xff&(timestruct[0]-2000))	# year, after 2000
-        self.v3_cmd(cmd)
+        self.__v3_cmd(cmd)
 
     def setlvl(self, box=u'10', sta=u'10'):
         """Set the read level on box and sta channels."""
@@ -579,40 +637,49 @@ class thbc(threading.Thread):
     def __parse_config(self, msg):
         # decoder configuration message.
         ibuf = bytearray(msg)
+        #print(repr(ibuf))
+        #self.log.debug(u'msg = ' + repr(msg))
+        #self.log.debug(u'ibuf = ' + repr(ibuf))
         self.decoderconfig = {}
-        showflags = {}
+        #showflags = {}
         for flag in sorted(CONFIG_FLAGS):	# import all
             # tone values
             if flag in [CONFIG_TONE_STA, CONFIG_TONE_BOX, CONFIG_TONE_MAN,
                      CONFIG_TONE_CEL, CONFIG_TONE_BXX]:
                 self.decoderconfig[flag] = 100*hexval2val(ibuf[flag])
                 self.decoderconfig[flag] += hexval2val(ibuf[flag+1])
-                showflags[CONFIG_FLAGS[flag]] = self.decoderconfig[flag]
+                #showflags[CONFIG_FLAGS[flag]] = self.decoderconfig[flag]
 
             # single byte values
             elif flag in [CONFIG_TZ_HOUR, CONFIG_TZ_MIN, CONFIG_PROT,
                     CONFIG_PULSEINT, CONFIG_CELLTOD_HOUR, CONFIG_CELLTOD_MIN]:
                 self.decoderconfig[flag] = hexval2val(ibuf[flag])
-                showflags[CONFIG_FLAGS[flag]] = self.decoderconfig[flag]
+                #showflags[CONFIG_FLAGS[flag]] = self.decoderconfig[flag]
 
             # 'booleans'
             elif flag in [CONFIG_TOD, CONFIG_GPS, CONFIG_485, CONFIG_FIBRE,
                     CONFIG_PRINT, CONFIG_MAX, CONFIG_PULSE,
-                    CONFIG_CELLSYNC, CONFIG_LANG]:
+                    CONFIG_CELLSYNC, CONFIG_ACTIVE_LOOP]:
                 self.decoderconfig[flag] = bool(ibuf[flag])
-                if self.decoderconfig[flag]:
-                    showflags[CONFIG_FLAGS[flag]] = True
+                #if self.decoderconfig[flag]:
+                    #showflags[CONFIG_FLAGS[flag]] = True
 
         self.unitno = u''
         for c in msg[43:47]:
             self.unitno += unichr(ord(c)+ord('0'))
         stalvl = hex(ord(msg[25]))	# ? question this
         boxlvl = hex(ord(msg[26]))
+        self.log.info(u'Decoder: ' + repr(self.unitno) + u' connected.')
 
-        self.log.info(u'Decoder ID: ' + self.unitno 
-                               + '\tConfig: ' + repr(showflags))
-        self.log.debug(u'Decoder Levels: STA=' + stalvl
-                               + u', BOX=' + boxlvl)
+        # Decoder info
+        self.log.debug(u'Decoder Levels: STA='+stalvl + u', BOX='+boxlvl)
+        self.decoderipconfig[u'IP'] = socket.inet_ntoa(msg[27:31])
+        self.decoderipconfig[u'Mask'] = socket.inet_ntoa(msg[31:35])
+        self.decoderipconfig[u'Gateway'] = socket.inet_ntoa(msg[35:39])
+        self.decoderipconfig[u'Host'] = socket.inet_ntoa(msg[39:43])
+        for key in [u'IP', u'Mask', u'Gateway', u'Host']:
+            self.log.debug(u'Decoder ' + key + ': ' 
+                            + repr(self.decoderipconfig[key]))
 
     def __parse_message(self, msg, ack=True):
         """Return tod object from timing msg or None."""
@@ -631,7 +698,7 @@ class thbc(threading.Thread):
                         if pvec[5] == '3': # LOW BATTERY ALERT
                             self.log.warn('Low battery on id: ' + repr(rstr))
                         ret = tod.tod(pvec[2], index=istr, chan=pvec[0],
-                                      refid=rstr)
+                                      refid=rstr, source=self.name)
                         self.log.log(RFID_LOG_LEVEL, msg.strip())
                         if ack:
                             self.port.write(ACKCMD)	# Acknowledge if ok
@@ -658,16 +725,10 @@ class thbc(threading.Thread):
                     # Note: this path is not immune to error in stream
                     # but in the case of exception from tod construct
                     # it will be collected by the thread 'main loop'
-                    #rstr = ':'.join(pvec[1:])
-                    #ret = tod.tod(pvec[0].rstrip('"'), index='', chan='STS',
-                                      #refid=rstr)
-
-                    ## ! Choice: leave status logging via RFID
-                    ## OR provide an alternate callback structure for
-                    ## status... probably better and allows for battery
-                    ## alerts
-                    #self.log.log(RFID_LOG_LEVEL, msg.strip())
-                    self.log.info(msg.strip())
+                    rstr = ':'.join(pvec[1:])
+                    ret = tod.tod(pvec[0].rstrip('"'), index='', chan='STS',
+                                      refid=rstr, source=self.name)
+                    self.log.log(RFID_LOG_LEVEL, msg.strip())
                 else:
                     self.log.info('Invalid status: ' + repr(msg))
             elif '+++' == msg[0:3] and len(msg) > 53:
@@ -678,20 +739,31 @@ class thbc(threading.Thread):
             self.log.info('Short message: ' + repr(msg))
         return ret
 
+    def __ipcompletion(self):
+        """Blocking wait for ipconfig completion - not elegant."""
+        time.sleep(2)
+        self.log.info(u'IP Config complete.')
+        self.port.write(QUECMD)
+ 
     def __read(self):
         """Read messages from the decoder until a timeout condition."""
         ch = self.port.read(1)
         while ch != '':
-            if ch == NACK:
+            #if ch == NACK:
                 # decoder has a passing to report
-                self.port.write(REPEATCMD)
-            elif ch == LF:
-                # Newline ends the current 'message'
+                #self.port.write(REPEATCMD)
+                # 0x07 can appear in-band, so this is not allowed
+            if ch == LF and len(self.__rdbuf) > 0 and self.__rdbuf[-1] == CR:
+                # Return ends the current 'message', if preceeded by return
                 self.__rdbuf += ch	# include trailing newline
                 t = self.__parse_message(self.__rdbuf.lstrip('\0'))
                 if t is not None:
                     glib.idle_add(self.__cb, t, priority=THBC_PRIORITY)
                 self.__rdbuf = ''
+            elif len(self.__rdbuf) > 40 and '\x1e\x86\x98' in self.__rdbuf:
+                # Assume acknowledge from IP Command
+                self.__rdbuf = ''
+                self.__ipcompletion()
             else:
                 self.__rdbuf += ch
             ch = self.port.read(1)
@@ -743,6 +815,12 @@ class thbc(threading.Thread):
                             self.log.log(RFID_LOG_LEVEL, str(m[1]))
                             glib.idle_add(self.__cb, m[1],
                                           priority=THBC_PRIORITY)
+                    elif m[0] == 'SANE':
+                        self.log.debug('Checking config.')
+                        self.__sane()
+                    elif m[0] == 'IPCFG':
+                        self.log.debug('Updating Decoder IP.')
+                        self.__ipcfg()
                     elif m[0] == 'SYNC':
                         t = tod.tod('now')
                         # DANGER: Busy wait may interfere with caller
@@ -768,7 +846,7 @@ class thbc(threading.Thread):
                             self.port = self.__mkport(m[1])
                             self.error = False
                             self.unitno = u''
-                            self.get_config()	# re-identify decoder
+                            self.port.write(QUECMD) # re-identify decoder
                         else:
                             self.log.debug('Not connected.')
                             self.error = True
@@ -778,17 +856,23 @@ class thbc(threading.Thread):
                     self.log.warn(u'Unknown message: ' + repr(m))
             except Queue.Empty:
                 pass
-            except (serial.SerialException, socket.error) as e:
+            except serial.SerialException as e:
                 if self.port is not None:
                     self.port.close()
                     self.port = None
-                self.errstr = 'Port error.'
+                self.errstr = 'Serial Port error.'
                 self.error = True
                 self.log.error('Closed port: ' + str(type(e)) + str(e))
+            except socket.error as e:
+                self.log.error('Network error: ' + str(type(e)) + str(e))
             except Exception as e:
                 self.log.error('Exception: ' + str(type(e)) + str(e))
+                raise
                 #self.errstr = str(e)
                 #self.error = True
+        if self.port is not None:
+            self.port.close()
+            self.port = None
         self.setcb()	# make sure callback is unrefed
         self.log.info('Exiting')
 
@@ -801,25 +885,23 @@ def showstatus(data=None):
     return True
 
 def doconfig(t):
-    print 'CONFIG'
-    t.decoderconfig[CONFIG_MAX] = False
-    t.decoderconfig[CONFIG_485] = False
-    t.decoderconfig[CONFIG_PULSE] = True
-    t.decoderconfig[CONFIG_PULSEINT] = 1
-    t.decoderconfig[CONFIG_CELLSYNC] = True
-    t.decoderconfig[CONFIG_CELLTOD_HOUR] = 0
-    t.decoderconfig[CONFIG_CELLTOD_MIN] = 0
-    t.set_config()
+    t.sane()
+    return False
+
+def doipconfig(t):
+    t.ipconfig()
     return False
 
 def dostart(t):
     t.start_session()
-    print 'START'
+    return False
+
+def dosync(t):
+    t.sync()
     return False
 
 def dostop(t):
     t.stop_session()
-    print 'STOP'
     return False
 
 if __name__ == "__main__":
@@ -827,32 +909,38 @@ if __name__ == "__main__":
     import gtk
     import time
     import random
+    import json
     metarace.init()
     t = thbc()
     lh = logging.StreamHandler()
+    #lh = logging.FileHandler(u'batlog')
     lh.setLevel(logging.DEBUG)
     lh.setFormatter(logging.Formatter(
                       "%(asctime)s %(levelname)s:%(name)s: %(message)s"))
     t.log.addHandler(lh)
     try:
         t.start()
-        t.setport(u'192.168.95.252')
+        #t.setport(u'192.168.95.240')
+        t.setport(u'192.168.95.253')
         #t.setport(u'/dev/ttyUSB0')
-        t.sane()
-        t.setlvl(u'30', u'30')
-        #t.set_date()
-        t.sync()
+
+        #t.set_date()	#?where did this go?
         t.setcb(printtag)
-        t.stop_session()
-        glib.timeout_add_seconds(10, showstatus, t)
-        #glib.timeout_add_seconds(5, doconfig, t)
-        #glib.timeout_add_seconds(4, dostop, t)
-        #glib.timeout_add_seconds(6, doconfig, t)
-        #glib.timeout_add_seconds(15, dostart, t)
-        t.start_session()
+        glib.timeout_add_seconds(4, dostop, t)
+        glib.timeout_add_seconds(6, doconfig, t)
+        #glib.timeout_add_seconds(8, doipconfig, t)
+        glib.timeout_add_seconds(10, dostart, t)
+        glib.timeout_add_seconds(12, dosync, t)
         metarace.mainloop()
     except:
         t.stop_session()
+        showconfig = {}
+        for k in t.decoderconfig:
+            key = k
+            if k in CONFIG_FLAGS:
+                key = CONFIG_FLAGS[k]
+            showconfig[key] = t.decoderconfig[k]
+        print(json.dumps(showconfig, indent=2, sort_keys=True))
         t.wait()
         t.exit('Exception')
         t.join()
