@@ -34,6 +34,7 @@ import pango
 
 import metarace
 from metarace import tod
+from metarace import unt4
 from metarace import uiutil
 
 class timerpane(object):
@@ -72,6 +73,12 @@ class timerpane(object):
 
     def set_time(self, tstr='             '):
         """Set timer string."""
+        #if self.uport is not None:
+            #msg = unt4.unt4(xx=0, yy=self.urow,
+                             #header=unichr(unt4.DC3) + u'R F$',
+                                    #text=tstr[0:12])
+            #self.uport.sendto(msg.pack(),
+                                   #('192.168.0.20',6789))
         self.ck.set_text(tstr)
 
     def show_laps(self):
@@ -134,9 +141,9 @@ class timerpane(object):
         self.set_time((self.finishtod-self.starttod).timestr(3))
         self.tofinish()
 
-    def tofinish(self):
+    def tofinish(self, status='finish'):
         """Set timer to finished."""
-        self.status = 'finish'
+        self.status = status
         uiutil.buttonchg(self.b, uiutil.bg_none, 'Finished')
 
     def toarmfin(self):
@@ -210,6 +217,8 @@ class timerpane(object):
 
     def __init__(self, label='Timer', doser=False):
         """Constructor."""
+        self.uport = None
+        self.urow = 0
         self.namevec = None
         s = gtk.Frame(label)
         s.set_border_width(5)
@@ -368,9 +377,9 @@ class mantimerpane(object):
         self.set_time((self.finishtod-self.starttod).timestr(3))
         self.tofinish()
 
-    def tofinish(self):
+    def tofinish(self, status='finish'):
         """Set timer to finished."""
-        self.status = 'finish'
+        self.status = status
         uiutil.buttonchg(self.b, uiutil.bg_none, 'Finished')
 
     def toarmfin(self):
