@@ -76,11 +76,11 @@ import threading
 import Queue
 import serial
 import logging
-import decimal
 import glib
 import metarace	# already in this module?
 
 from metarace import tod
+from metarace import strops
 
 # System default timy serial port
 DEFPORT = u'/dev/ttyUSB0'
@@ -90,10 +90,11 @@ BACKUPPORT = u'/dev/ttyUSB1'
 ENCODING = 'cp437'	# Timy serial interface encoding
 
 # TIMY serial baudrate
-TIMY_BAUD = 38400	# 	but timy2 seems to cope ok :/
+TIMY_BAUD = 38400	# baudrate
+TIMY_CTSRTS = True	# hardware flow override in default config
 
 # thread queue commands -> private to timy thread
-TCMDS = ('EXIT', 'PORT', 'MSG', 'ARM', 'DEARM', 'TRIG', 'RCLR')
+TCMDS = ('EXIT', 'PORT', 'MSG', 'TRIG', 'RCLR')
 
 # timing channels
 CHAN_START = 0
@@ -208,6 +209,7 @@ class timy(threading.Thread):
         self.arms = [False, False, False, False, False,
                      False, False, False, False, False]
         self.lindex = 0
+        self.unitno = u'timy01'
         self.clearing = False
         self.error = False
         self.errstr = u''
@@ -255,6 +257,26 @@ class timy(threading.Thread):
         """Request a dump of all times to host."""
         self.cqueue.put_nowait(('MSG', 'RSM\r'))
 
+    def delaytime(self, newdelay):
+        """Update the timy channel delays."""
+        dt = tod.str2tod(newdelay)
+        if dt is not None:
+            if dt > tod.ZERO and dt < tod.tod(u'99.99'):
+                nt = dt.rawtime(2, zeros=True)[6:].encode('ascii','ignore')
+                self.cqueue.put_nowait(('MSG', 'DTS' + nt + '\r'))
+                self.cqueue.put_nowait(('MSG', 'DTF' + nt + '\r'))
+            else:
+                sef.log.info(u'Ignoring invalid delay time: ' + dt.rawtime())
+        else:
+            sef.log.info(u'Ignoring invalid delay time.')
+
+    def printer(self, enable=False):
+        """Enable or disable printer."""
+        cmd = '0'
+        if enable:
+            cmd = '1'
+        self.cqueue.put_nowait(('MSG', 'PRINTER' + cmd + '\r'))
+
     def printimp(self, doprint=True):
         """Enable or disable internal printing of timing impulses."""
         cmd = '1'
@@ -291,19 +313,32 @@ class timy(threading.Thread):
 
     def arm(self, channel=0):
         """Arm timing channel 0 - 8 for response through rqueue."""
-        self.cqueue.put_nowait(('ARM', channel))
+        chan = chan2id(channel)
+        self.log.debug(u'Arming channel ' + id2chan(chan))
+        self.arms[chan] = True;
 
     def dearm(self, channel=0):
         """Disarm timing channel 0 - 8 for response through rqueue."""
-        self.cqueue.put_nowait(('DEARM', channel))
+        chan = chan2id(channel)
+        self.log.debug(u'Disarming channel ' + id2chan(chan))
+        self.arms[chan] = False;
 
     def armlock(self, lock=True):
-        """Set or clear the arming lock."""
+        """Set or clear the arming lock - flag only."""
+        # thread ok, but needs help
         self.armlocked = bool(lock)
 
+    def start_session(self):
+        """Not used."""
+        self.log.info(u'Start Session')
+
+    def stop_session(self):
+        """Not used."""
+        self.log.info(u'Stop Session')
+
     def sync(self):
         """[deprecated] Roughly synchronise Timy to PC clock."""
-        self.log.info('Timy Sync deprecated.')
+        self.log.info(u'Timy Sync deprecated.')
 
     def sane(self):
         """Initialise Timy to 'sane' values.
@@ -316,9 +351,10 @@ class timy(threading.Thread):
 	    PRE4		- 10,000th sec precision
 	    RR0			- Round by 'cut'
 	    BE1			- Beep on
-	    DTS00.02		- Start delay 0.02s
-	    DTF00.02		- Finish & intermediate delay 0.02s
+	    DTS01.00		- Start delay 1.0
+	    DTF01.00		- Finish & intermediate delay 1.0
 	    EMU0		- Running time off
+	    PRINTER0		- Printer off
 	    PRIIGN1		- Don't print all impulses to receipt
 	    PRILF		- Linefeed
 	
@@ -338,11 +374,13 @@ class timy(threading.Thread):
 
         """
         for msg in ['TIMYINIT', 'NSF', 'PROG?', 'KL0', 'CHK1', 'PRE4',
-                    'RR0', 'BE1', 'DTS00.02', 'DTF00.02', 'EMU0',
-                    'PRIIGN1', 'DTPMetarace ' + metarace.VERSION, 'PRILF']:
+                    'RR0', 'BE1', 'DTS01.00', 'DTF01.00', 'EMU0',
+                    'PRINTER0', 'PRIIGN1',
+                    'DTPMetarace ' + metarace.VERSION, 'PRILF']:
             self.write(msg)
 
-    def trig(self, timeval='now', index='FAKE', chan='0', refid=''):
+    def trig(self, timeval='now', index='FAKE', chan='0',
+                   refid='', sourceid=None):
         """Create a fake timing event.
 
         Parameters:
@@ -359,7 +397,11 @@ class timy(threading.Thread):
         ensure correct function.
 
         """
-        t = tod.tod(timeval, index, id2chan(chan2id(chan)), refid.lstrip('0'))
+        srcid = self.name
+        if sourceid:
+            srcid = sourceid
+        t = tod.tod(timeval, index, id2chan(chan2id(chan)),
+                             refid.lstrip('0'), source=srcid)
         self.cqueue.put_nowait(('TRIG', t))
 
     def wait(self):
@@ -382,7 +424,8 @@ class timy(threading.Thread):
             if tsum == csum:
                 e = msg.split()
                 if len(e) == 4:
-                    ret = tod.tod(timeval = e[2], index = e[0], chan = e[1])
+                    ret = tod.tod(timeval = e[2], index = e[0], chan = e[1],
+                                                  source = self.name)
                 else:
                     self.log.error(u'Invalid message: ' + repr(msg))
             else:
@@ -438,6 +481,7 @@ class timy(threading.Thread):
     def __read(self):
         """Read messages from timy until a timeout condition."""
         ch = self.port.read(1)	# decode via cp437?
+        mcnt = 0
         while ch != '':
             if ch == CR:
                 # Return ends the current 'message'
@@ -446,6 +490,9 @@ class timy(threading.Thread):
                 if t is not None:
                     self.__proc_impulse(t)
                 self.__rdbuf = ''
+                mcnt += 1
+                if mcnt > 4:	# break to allow write back
+                    return
             else:
                 self.__rdbuf += ch
             ch = self.port.read(1)
@@ -454,6 +501,14 @@ class timy(threading.Thread):
         """Called via threading.Thread.start()."""
         running = True
         self.log.debug(u'Starting')
+        baudrate = TIMY_BAUD
+        if metarace.sysconf.has_option(u'timy', u'baudrate'):
+            baudrate = strops.confopt_posint(metarace.sysconf.get(u'timy',
+                                               u'baudrate'), baudrate)
+        ctsrts = TIMY_CTSRTS
+        if metarace.sysconf.has_option(u'timy', u'ctsrts'):
+            ctsrts = strops.confopt_bool(metarace.sysconf.get(u'timy',
+                                               u'ctsrts'))
         while running:
             try:
                 # Read phase
@@ -470,14 +525,6 @@ class timy(threading.Thread):
                     if m[0] == 'MSG' and self.port is not None and not self.error:
                         self.log.debug(u'Sending rawmsg ' + repr(m[1]))
                         self.port.write(m[1].encode(ENCODING,'replace'))
-                    elif m[0] == 'ARM':
-                        chan = chan2id(m[1])
-                        self.log.debug(u'Arming channel ' + id2chan(chan))
-                        self.arms[chan] = True;
-                    elif m[0] == 'DEARM':
-                        chan = chan2id(m[1])
-                        self.log.debug(u'Disarming channel ' + id2chan(chan))
-                        self.arms[chan] = False;
                     elif m[0] == 'TRIG':
                         if type(m[1]) is tod.tod:
                             self.__proc_impulse(m[1])
@@ -492,8 +539,9 @@ class timy(threading.Thread):
                             self.port = None
                         if m[1] is not None and m[1] != '' and m[1] != 'NULL':
                             self.log.debug(u'Re-Connect port: ' + repr(m[1]))
-                            self.port = serial.Serial(m[1], TIMY_BAUD,
-                                                      rtscts=1, timeout=0.2)
+                            self.port = serial.Serial(m[1], baudrate,
+                                                      rtscts=ctsrts,
+                                                      timeout=0.2)
                             self.error = False
                         else:
                             self.log.debug(u'Not connected.')
