@@ -15,13 +15,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-##TODO:
-#
-# Performance - require better input detection switch for string
-# Formats - split on decimal chars . , " instead of string replace
-#
-#
-#
 """Time of Day (ToD) functions and abstract class.
 
 This module defines the tod class and some utility functions.
@@ -45,12 +38,16 @@ A time of day object includes:
    - index   : 4 character identifier string (eg '1' to '9999')
    - chan    : 3 character channel string from source (eg 'C0', 'C2M', etc)
    - refid   : string reference id, used for RFID tag events (eg '75ae7f')
+   - source  : string identifier for timer that sourced the event (eg 'main')
 
 Supported ToD String Patterns:
 
    [[HH:]MM:]SS[.dcmz]		Canonical
    [[HH-]MM-]SS[.dcmz]		Keypad
    [[HHh]MM:]SS[.dcmz]		Result
+   [[HH:]MM']SS["dcmz]		Chronelec
+   S+[.dcmz]			Seconds
+   now				Localtime at object creation
 
 Arithmetic operations on ToD types:
 
@@ -102,16 +99,22 @@ printed, space padded, according to the requested precision.
 
 """
 
-import decimal		# ToD internal representation
+dmod = None
+try:
+    import cdecimal		# ToD internal representation
+    dmod = cdecimal
+except ImportError:
+    import decimal
+    dmod = decimal
 import re		# used to scan ToD string: HH:MM:SS.dcmz 
 import time
 
-QUANT_5PLACES = decimal.Decimal('0.00001') # does not work with Timy printer
-QUANT_4PLACES = decimal.Decimal('0.0001')
-QUANT_3PLACES = decimal.Decimal('0.001')
-QUANT_2PLACES = decimal.Decimal('0.01')
-QUANT_1PLACE = decimal.Decimal('0.1')
-QUANT_0PLACES = decimal.Decimal('1')
+QUANT_5PLACES = dmod.Decimal('0.00001') # does not work with Timy printer
+QUANT_4PLACES = dmod.Decimal('0.0001')
+QUANT_3PLACES = dmod.Decimal('0.001')
+QUANT_2PLACES = dmod.Decimal('0.01')
+QUANT_1PLACE = dmod.Decimal('0.1')
+QUANT_0PLACES = dmod.Decimal('1')
 QUANT = [QUANT_0PLACES, QUANT_1PLACE, QUANT_2PLACES,
          QUANT_3PLACES, QUANT_4PLACES, QUANT_5PLACES]
 QUANT_FW = [2, 4, 5, 6, 7, 8]
@@ -120,6 +123,8 @@ QUANT_PAD = ['     ', '   ', '  ', ' ', '', '']
 QUANT_OPAD = ['    ', '  ', ' ', '', '', '']
 TOD_RE=re.compile(r'^(?:(?:(\d{1,2})[h:-])?(\d{1,2})[:\'-])?(\d{1,2}(?:[\.\"]\d+)?)$',flags=re.UNICODE)
 
+# assume no run over change
+DSTHACK = time.timezone - time.localtime().tm_isdst * 3600
 
 def str2tod(timeval=u''):
     """Return tod for given string without fail."""
@@ -159,16 +164,16 @@ def dec2str(dectod=None, places=4, zeros=False, hoursep=u'h', minsep=u':'):
             strtod = fmt.format(int(dectod)//3600,
                 (int(dectod)%3600)//60,
                 dectod.quantize(QUANT[places],
-                rounding=decimal.ROUND_FLOOR)%60,
+                rounding=dmod.ROUND_FLOOR)%60,
                 QUANT_FW[places])
         elif dectod >= 60:	# MM:SS.dcmz
             strtod = (u'{0}'+minsep+u'{1:0{2}}').format(int(dectod)//60,
                 dectod.quantize(QUANT[places],
-                rounding=decimal.ROUND_FLOOR)%60,
+                rounding=dmod.ROUND_FLOOR)%60,
                 QUANT_FW[places])
         else: 			# SS.dcmz or -SSSSS.dcmz
             strtod = u'{0}'.format(dectod.quantize(QUANT[places],
-                rounding=decimal.ROUND_FLOOR))
+                rounding=dmod.ROUND_FLOOR))
     return strtod
 
 def str2dec(timestr=u''):
@@ -195,44 +200,42 @@ def str2dec(timestr=u''):
     dectod=None
     timestr=timestr.strip()	# assumes string
     if timestr == u'now':
-        ltoft = time.localtime().tm_isdst * 3600	# DST Hack
-        dectod = decimal.Decimal(str(
-                    (time.time() - (time.timezone - ltoft)) % 86400))
-                    # !!ERROR!! 2038, UTC etc -> check def Unix time
+        dectod = dmod.Decimal(str((time.time() - DSTHACK) % 86400))
     else:
         m = TOD_RE.match(timestr)
         if m is not None:
-            dectod = decimal.Decimal(m.group(3).replace(u'"', u'.'))
-            dectod += decimal.Decimal(m.group(2) or 0) * 60
-            dectod += decimal.Decimal(m.group(1) or 0) * 3600
+            dectod = dmod.Decimal(m.group(3).replace(u'"', u'.'))
+            dectod += dmod.Decimal(m.group(2) or 0) * 60
+            dectod += dmod.Decimal(m.group(1) or 0) * 3600
         else:
             # last attempt - try and handle as other decimal constructor
-            dectod = decimal.Decimal(timestr)
-    return dectod.quantize(QUANT[4], rounding=decimal.ROUND_FLOOR)
-
+            dectod = dmod.Decimal(timestr)
+    return dectod.quantize(QUANT[4], rounding=dmod.ROUND_FLOOR)
 
 class tod(object):
     """A class for representing time of day and RFID events."""
-    def __init__(self, timeval=0, index=u'', chan=u'', refid=u''):
+    def __init__(self, timeval=0, index=u'', chan=u'', refid=u'', source=u''):
         """Construct tod object.
 
         Keyword arguments:
         timeval -- time value to be represented (string/int/decimal/tod)
         index -- tod index identifier string
         chan -- channel string
-        refed -- a reference identifier string
+        refid -- a reference identifier string
+        source -- timer source identifier or u'' if not sourced
 
         """
 
         self.index = index[0:4]	# numeric raises exception here
         self.chan = chan[0:3]
         self.refid = refid
+        self.source = source
         if type(timeval) in [unicode, str]:
             self.timeval = str2dec(timeval)
         elif type(timeval) is tod:
             self.timeval = timeval.timeval
         else:
-            self.timeval = decimal.Decimal(timeval)
+            self.timeval = dmod.Decimal(timeval)
         assert self.timeval >= 0 and self.timeval < 86400, 'timeval not in range [0, 86400)'
 
     def __str__(self):
@@ -262,22 +265,22 @@ class tod(object):
     def truncate(self, places=4):
         """Return a new ToD object with a truncated time value."""
         return tod(timeval=self.timeval.quantize(QUANT[places],
-                rounding=decimal.ROUND_FLOOR), index='', chan='ToD', refid='')
+                rounding=dmod.ROUND_FLOOR), index='', chan='ToD', refid='')
 
     def as_hours(self, places=0):
         """Return the tod value in hours, truncated to the desired places."""
         return (self.timeval / 3600).quantize(QUANT[places],
-                                            rounding=decimal.ROUND_FLOOR)
+                                            rounding=dmod.ROUND_FLOOR)
 
     def as_seconds(self, places=0):
         """Return the tod value in seconds, truncated to the desired places."""
         return self.timeval.quantize(QUANT[places],
-                                     rounding=decimal.ROUND_FLOOR)
+                                     rounding=dmod.ROUND_FLOOR)
 
     def as_minutes(self, places=0):
         """Return the tod value in minutes, truncated to the desired places."""
         return (self.timeval / 60).quantize(QUANT[places],
-                                            rounding=decimal.ROUND_FLOOR)
+                                            rounding=dmod.ROUND_FLOOR)
 
     def timestr(self, places=4, zeros=False, hoursep=u'h', minsep=u':'):
         """Return time string component of the tod, whitespace padded."""
@@ -313,6 +316,12 @@ class tod(object):
             return '---.- km/h'
         return '{0:5.1f} km/h'.format(3.6 * float(dist) / float(self.timeval))
 
+    def rawspeed(self, dist=200):
+        """Return an average speed estimate without units."""
+        if self.timeval == 0:
+            return '-.-'
+        return '{0:0.1f}'.format(3.6 * float(dist) / float(self.timeval))
+
     def copy(self):
         """Return a copy of the supplied tod."""
         return tod(self.timeval, self.index, self.chan, self.refid)
@@ -386,6 +395,10 @@ class tod(object):
             raise TypeError('Cannot add {0} to tod.'.format(
                                 str(type(other).__name__)))
 
+def now():
+    """Return a now tod."""
+    return tod(dmod.Decimal(str((time.time() - DSTHACK) % 86400)))
+
            
 class agg(tod):
     """A non-wrapped aggregate time.
@@ -393,21 +406,22 @@ class agg(tod):
        Promotion to aggregate time is 'sticky' and may be negative.
 
     """
-    def __init__(self, timeval=0, index=u'', chan=u'', refid=u''):
+    def __init__(self, timeval=0, index=u'', chan=u'', refid=u'', source=u''):
         self.index = index[0:4]
         self.chan = chan[0:3]
         self.refid = refid
+        self.source = source	# not really relevant in agg
         if type(timeval) in [unicode, str]:
             self.timeval = str2dec(timeval)
         elif type(timeval) is tod:
             self.timeval = timeval.timeval
         else:
-            self.timeval = decimal.Decimal(timeval)
+            self.timeval = dmod.Decimal(timeval)
 
     def truncate(self, places=4):
         """Return a new ToD object with a truncated time value."""
         return agg(timeval=self.timeval.quantize(QUANT[places],
-                rounding=decimal.ROUND_FLOOR), index=u'',
+                rounding=dmod.ROUND_FLOOR), index=u'',
                                  chan=u'AGG', refid=u'')
 
     def __add__(self, other):
@@ -448,8 +462,8 @@ FAKETIMES = {
  u'dsq':MAX.copy(),	# disqualified
  u'dnf':MAX.copy(),	# did not finish
  u'dns':MAX.copy()}	# did not start
-extra = decimal.Decimal(u'0.00001')
-cof = decimal.Decimal(u'0.00001')
+extra = dmod.Decimal(u'0.00001')
+cof = dmod.Decimal(u'0.00001')
 for c in [u'caught', u'rel', u'abort', u'otl', u'dsq', u'dnf', u'dns']:
     FAKETIMES[c].timeval += cof
     cof += extra
@@ -525,10 +539,10 @@ if __name__ == "__main__":
                '3:45',         '3-45',
                  '45.6789',        '5.6',
                  '45', FAKETIMES['dnf'], FAKETIMES[u'dnf'],
-            1.4, float('1.4'), decimal.Decimal('1.4'), '1.4',
-            10123, float('10123'), decimal.Decimal('10123'), '10123',
+            1.4, float('1.4'), dmod.Decimal('1.4'), '1.4',
+            10123, float('10123'), dmod.Decimal('10123'), '10123',
             10123.456, float('10123.456'),
-            decimal.Decimal('10123.456'), '10123.456',
+            dmod.Decimal('10123.456'), '10123.456',
             '-10234', '87012', '0', '86400', '86399.9999',
             'inf', 'nan', 'zero', 'now', '-inf',
             tod(0, 'ZERO'), tod('now', 'NOW') ]
