@@ -1,10 +1,13 @@
 
 #
 # transitional Python 2.x unicode csv helper module,
-# copied verbatim from the Python manual:
+# copied from the Python manual:
 #
 #  http://docs.python.org/library/csv.html#csv-examples
 #
+# Note: Contrary to the typical behaviour of a utf-8 decoder,
+#       invalid, erroeneous or non-unicode chars are silently ignored
+#       and filtered from the input.
 
 import csv, codecs, cStringIO
 
@@ -13,13 +16,13 @@ class UTF8Recoder:
     Iterator that reads an encoded stream and reencodes the input to UTF-8
     """
     def __init__(self, f, encoding):
-        self.reader = codecs.getreader(encoding)(f)
+        self.reader = codecs.getreader(encoding)(f, errors='ignore')
 
     def __iter__(self):
         return self
 
     def next(self):
-        return self.reader.next().encode("utf-8")
+        return self.reader.next().encode("utf-8", 'ignore')
 
 class UnicodeReader:
     """
@@ -49,13 +52,13 @@ class UnicodeWriter:
         self.queue = cStringIO.StringIO()
         self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
         self.stream = f
-        self.encoder = codecs.getincrementalencoder(encoding)()
+        self.encoder = codecs.getincrementalencoder(encoding)(errors='ignore')
 
     def writerow(self, row):
-        self.writer.writerow([s.encode("utf-8") for s in row])
+        self.writer.writerow([s.encode("utf-8", 'ignore') for s in row])
         # Fetch UTF-8 output from the queue ...
         data = self.queue.getvalue()
-        data = data.decode("utf-8")
+        data = data.decode("utf-8", 'ignore')
         # ... and reencode it into the target encoding
         data = self.encoder.encode(data)
         # write to the target stream
