@@ -327,7 +327,7 @@ def edit_times_dlg(window, stxt=None, ftxt=None,
         pe.show()
         b.get_object('timing_penalty_label').show()
 
-    fe.grab_focus()	# hack for CATN
+    #fe.grab_focus()	# hack for CATN
     ret = dlg.run()
     stxt = se.get_text().strip()
     ftxt = fe.get_text().strip()
