@@ -17,7 +17,7 @@
 
 """UNT4 Protocol Wrapper.
 
-Pack and unpack UNT4 protocol messages for Omega Galactica DHI.
+Pack and unpack Omega style UNT4 protocol messages.
 
 Note: Normal program code will not use this library directly
       except to access the pre-defined overlay messages. All
@@ -46,11 +46,15 @@ DLE = 0x10
 DC2 = 0x12
 DC3 = 0x13
 DC4 = 0x14
+SYN = 0x16
+ETB = 0x17
 FS = 0x1c
 GS = 0x1d
 RS = 0x1e
 US = 0x1f
 
+DAKLINELEN = 32	# HACK and limit
+
 
 ENCMAP = {
    unichr(SOH):'<O>',
@@ -73,17 +77,31 @@ ENCMAP = {
   
 def encode(ubuf=u''):
     """Encode the unt4 buffer for use with IRC."""
+    # escape special char
+    ubuf = ubuf.replace(u'<', u'<>')
+    # escape control chars
     for key in ENCMAP:
         ubuf = ubuf.replace(key, ENCMAP[key])
     return ubuf
 
 def decode(ircbuf=u''):
     """Decode the irc buffer to unt4msg pack."""
+    # decode nulls (is this required anymore?)
     ircbuf = ircbuf.replace(u'<00>',u'')
+    # decode control chars
     for key in ENCMAP:
         ircbuf = ircbuf.replace(ENCMAP[key], key)
+    # decode special char
+    ircbuf = ircbuf.replace(u'<>', u'<')
     return ircbuf
 
+# byte type checksum for dak -> note will break with unichars
+def daksum(msgbody):
+    sum = 0x00
+    for c in msgbody:
+        sum += ord(c)
+    return u'{0:02X}'.format(sum & 0xff)
+
 # UNT4 Packet class
 class unt4(object):
     """UNT4 Packet Class."""
@@ -116,7 +134,7 @@ class unt4(object):
             self.unpack(unt4str)
 
     def unpack(self, unt4str=u''):
-        """Unpack the UNT4 string into this object."""
+        """Unpack the UNT4 unicode string into this object."""
         if len(unt4str) > 2 and unt4str[0] is unichr(SOH) \
                             and unt4str[-1] is unichr(EOT):
             self.prefix = None
@@ -161,28 +179,24 @@ class unt4(object):
             self.text = newtext
 
     def altpack(self):
-        """Return UDP for Invercargill velodrome scoreboard."""
-        # ERP, head, ERL, prefix all ignored
+        """Return DAK protocol (Hisense/Dunc Gray)."""
         ret = u''
-        if self.xx is not None and self.yy is not None and self.text:
-            text = u''
-            for c in [SOH, NUL, BEL, NUL,
-                      (len(self.text)+3)&0xff, NUL, STX, 0x01,
-                      (self.xx+1)&0xff, (self.yy+1)&0xff]:
-                text += unichr(c)
-            text += self.text + unichr(ETX)
-            ret = text
+        if self.erp:
+            # special case 1: write space to all chars
+            pass
+        elif self.xx is not None and self.yy is not None and self.text:
+            oft = self.yy * DAKLINELEN + self.xx
+            control = u'004010{0:04d}'.format(oft)
+            header = u'20000000'
+            text = (header + unichr(SOH) + control
+                           + unichr(STX) + self.text + unichr(EOT))
+            ret = unichr(SYN) + text + daksum(text) + unichr(ETB)
         elif self.header:	# assume well formed packet in self.header
-            text = u''
-            for c in [SOH, NUL, BEL, NUL,
-                      len(self.header)&0xff, NUL, STX]:
-                text += unichr(c)
-            text += self.header + unichr(ETX)
-            ret = text
+            pass	# todo
         return ret
 
     def pack(self):
-        """Return UNT4 string packet."""
+        """Return Omega Style UNT4 unicode string packet."""
         head = u''
         text = u''
         if self.erp:	# overrides any other message content
@@ -205,21 +219,10 @@ class unt4(object):
 GENERAL_CLEARING = unt4(erp=True)
 OVERLAY_ON = unt4(header=u'OVERLAY ON')
 OVERLAY_OFF = unt4(header=u'OVERLAY OFF')
-OVERLAY_1LINE = unt4(header=u'OVERLAY 00')
-OVERLAY_2LINE = unt4(header=u'OVERLAY 01')
-OVERLAY_3LINE = unt4(header=u'OVERLAY 02')
-OVERLAY_4LINE = unt4(header=u'OVERLAY 03')
-OVERLAY_R1P4 = unt4(header=u'OVERLAY 04')
-OVERLAY_R2P2 = unt4(header=u'OVERLAY 05')
-OVERLAY_R2P3 = unt4(header=u'OVERLAY 06')
-OVERLAY_R2P4 = unt4(header=u'OVERLAY 07')
-OVERLAY_T1P4 = unt4(header=u'OVERLAY 08')
-OVERLAY_T1P5 = unt4(header=u'OVERLAY 09')
-OVERLAY_24X5 = unt4(header=u'OVERLAY 10')
-OVERLAY_24X6 = unt4(header=u'OVERLAY 11')
-OVERLAY_CLOCK = unt4(header=u'OVERLAY 12')
-OVERLAY_IMAGE = unt4(header=u'OVERLAY 13')
-OVERLAY_BLANK = unt4(header=u'OVERLAY 14')
+OVERLAY_CLOCK = unt4(header=u'OVERLAY 00')
+OVERLAY_MATRIX = unt4(header=u'OVERLAY 01')
+OVERLAY_IMAGE = unt4(header=u'OVERLAY 02')
+OVERLAY_BLANK = unt4(header=u'OVERLAY 03')
 
 # Invercargill 'flush'/'ack' packet
 ALT_ACK = unt4(header=unichr(ACK))
